@@ -3,10 +3,10 @@
  *  Module    : nntplib.h
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2005-07-21
+ *  Updated   : 2009-01-10
  *  Notes     : nntp.h 1.5.11/1.6 with extensions for tin
  *
- * Copyright (c) 1991-2007 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -87,57 +87,44 @@
 #	define	CHAR_FATAL		'5'
 #endif /* 0 */
 
+/* TODO: cleanup against RFC 2980, RFC 3977 */
 #define	INF_HELP		100	/* Help text on way */
 #define	INF_CAPABILITIES	101	/* Capability list follows */
-#define	INF_AUTH		180	/* Authorization capabilities */
-#define	INF_DEBUG		199	/* Debug output */
+#define INF_DATE		111 /* yyyymmddhhmmss Server date and time */
 
 #define	OK_CANPOST		200	/* Hello; you can post */
 #define	OK_NOPOST		201	/* Hello; you can't post */
 #define	OK_EXTENSIONS	202	/* extensions supported follow */
-#define	OK_NOIHAVE		202	/* Hello; you can't IHAVE; discussed on the IETF-mailinglist */
-#define	OK_SLAVE		202	/* Slave status noted */
-#define	OK_NOPOSTIHAVE	203	/* Hello; you can't POST or IHAVE; iscussed on the IETF-mailinglist */
 #define	OK_GOODBYE		205	/* Closing connection */
 #define	OK_GROUP		211	/* Group selected */
 #define	OK_GROUPS		215	/* Newsgroups follow */
 #define	OK_MOTD			215	/* News motd follows */
 
-/* obsolete */
-#define	OK_XINDEX		218	/* Tin style index follows */
-
 #define	OK_ARTICLE		220	/* Article (head & body) follows */
 #define	OK_HEAD			221	/* Head follows */
 #define	OK_BODY			222	/* Body follows */
 #define	OK_NOTEXT		223	/* No text sent -- stat, next, last */
 #define	OK_XOVER		224	/* .overview data follows */
 #define	OK_NEWNEWS		230	/* New articles by message-id follow */
-#define	OK_NEWGROUPS		231	/* New newsgroups follow */
+#define	OK_NEWGROUPS	231	/* New newsgroups follow */
 #define	OK_XFERED		235	/* Article transferred successfully */
 #define	OK_POSTED		240	/* Article posted successfully */
 #define	OK_AUTHSYS		280	/* Authorization system ok */
 #define	OK_AUTH			281	/* Authorization (user/pass) ok */
 #define	OK_BIN			282	/* binary data follows */
 #define	OK_LIST			282	/* list follows */
-#define	OK_SPLIST		283	/* spooldir list follows */
-#define	OK_SPSWITCH		284	/* Switching to a different spooldir */
-#define	OK_SPNOCHANGE		285	/* Still using same spooldir */
-#define	OK_SPLDIRCUR		286	/* Current spooldir */
-#define	OK_SPLDIRAVL		287	/* Available spooldir */
-#define	OK_SPLDIRERR		288	/* Unavailable spooldir or invalid entry */
+#define OK_AUTH_SASL	283 /* authentication accepted (with success data) */
 
 #define	CONT_XFER		335	/* Continue to send article */
 #define	CONT_POST		340	/* Continue to post article */
-#define	NEED_AUTHINFO		380	/* authorization is required */
-#define	NEED_AUTHDATA		381	/* <type> authorization data required */
+#define	NEED_AUTHINFO	380	/* authorization is required */
+#define	NEED_AUTHDATA	381	/* <type> authorization data required */
+#define NEED_AUTHDATA_SASL	383 /* continue with SASL exchange */
 
 #define	ERR_GOODBYE		400	/* Have to hang up for some reason */
 #define	ERR_NOGROUP		411	/* No such newsgroup */
 #define	ERR_NCING		412	/* Not currently in newsgroup */
 
-#define	ERR_XINDEX		418	/* No tin index for this group */
-#define	ERR_XOVERVIEW		419	/* No .overview index for this group */
-
 #define	ERR_NOCRNT		420	/* No current article selected */
 #define	ERR_NONEXT		421	/* No next article in this group */
 #define	ERR_NOPREV		422	/* No previous article in this group */
@@ -149,10 +136,7 @@
 #define	ERR_NOPOST		440	/* Posting not allowed */
 #define	ERR_POSTFAIL		441	/* Posting failed */
 #define	ERR_NOAUTH		480	/* authorization required for command */
-#define	ERR_AUTHSYS		481	/* Authorization system invalid */
-#define	ERR_AUTHREJ		482	/* Authorization data rejected */
-#define	ERR_INVALIAS		483	/* Invalid alias on spooldir cmd */
-#define	ERR_INVNOSPDIR		484	/* No spooldir file found */
+#define	ERR_ENCRYPT		483	/* encrpytion required */
 
 #define	ERR_COMMAND		500	/* Command not recognized */
 #define	ERR_CMDSYN		501	/* Command syntax error */
@@ -166,14 +150,21 @@
  */
 #define NNTP_STRLEN		512
 
+/*
+ * OVERVIEW.FMT field types
+ */
+enum f_type { OVER_T_ERROR, OVER_T_INT, OVER_T_STRING, OVER_T_FSTRING };
+
 /*
  * CAPABILITIES
  */
+enum extension_type { NONE, LIST_EXTENSIONS, CAPABILITIES, BROKEN };
+
 struct t_capabilities {
-	unsigned int type:3;			/* none, LIST EXTENSIONS, CAPABILITIES */
+	enum extension_type type;		/* none, LIST EXTENSIONS, CAPABILITIES, BROKEN */
 	unsigned int version;			/* CAPABILITIES version */
 	t_bool mode_reader:1;			/* MODE-READER: "MODE READER" */
-	t_bool reader:1;				/* READER: "ARTCILE", "BODY", "LISTGROUP" */
+	t_bool reader:1;				/* READER: "ARTCILE", "BODY", "DATE", "GROUP", "LAST", "LISTGROUP", "NEWGROUPS", "NEXT" */
 	t_bool post:1;					/* POST */
 	t_bool list_active:1;			/* LIST ACTIVE */
 	t_bool list_active_times:1;		/* LIST ACTIVE.TIMES, optional */
@@ -196,12 +187,15 @@ struct t_capabilities {
 	t_bool starttls:1;				/* STARTTLS */
 	t_bool authinfo_user:1;			/* AUTHINFO USER/PASS */
 	t_bool authinfo_sasl:1;			/* AUTHINFO SASL */
-#if 0
 	t_bool sasl_cram_md5:1;			/* SASL CRAM-MD5 */
 	t_bool sasl_digest_md5:1;		/* SASL DIGEST-MD5 */
 	t_bool sasl_plain:1;			/* SASL PLAIN */
 	t_bool sasl_gssapi:1;			/* SASL GSSAPI */
 	t_bool sasl_external:1;			/* SASL EXTERNAL */
+#if 0
+	t_bool sasl_otp:1;				/* SASL OTP */
+	t_bool sasl_ntlm:1;				/* SASL NTLM */
+	t_bool sasl_login:1;			/* SASL LOGIN */
 	t_bool streaming:1;				/* STREAMING: "MODE STREAM", "CHECK", "TAKETHIS" */
 	t_bool ihave:1;					/* IHAVE: "IHAVE" */
 #endif /* 0 */
