@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2006-10-01
+ *  Updated   : 2009-01-09
  *  Notes     :
  *
- * Copyright (c) 1997-2007 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2009 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -75,7 +75,10 @@ extern void sort_arts(unsigned /* int */ sort_art_type);
 extern void write_overview(struct t_group *group);
 
 /* attrib.c */
-extern void read_attributes_files(void);
+extern int add_tempscope(const char *scope);
+extern void assign_attributes_to_groups(void);
+extern void build_news_headers_array(struct t_attribute *scope, t_bool header_to_display);
+extern void read_attributes_file(t_bool global_file);
 extern void write_attributes_file(const char *file);
 
 /* auth.c */
@@ -86,7 +89,7 @@ extern void write_attributes_file(const char *file);
 /* charset.c */
 extern char *convert_to_printable(char *buf);
 extern t_bool is_art_tex_encoded(FILE *fp);
-extern void convert_iso2asc(char *iso, char **asc_buffer, int *max_line_len, int t);
+extern void convert_iso2asc(char *iso, char **asc_buffer, size_t *max_line_len, int t);
 extern void convert_tex2iso(char *from, char *to);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern wchar_t *wconvert_to_printable(wchar_t *wbuf);
@@ -97,6 +100,9 @@ extern void draw_pager_line(const char *str, int flags, t_bool raw_data);
 #ifdef HAVE_COLOR
 	extern void bcol(int color);
 	extern void fcol(int color);
+#	ifdef USE_CURSES
+		extern void free_color_pair_arrays(void);
+#	endif /* USE_CURSES */
 #endif /* HAVE_COLOR */
 
 /* config.c */
@@ -116,7 +122,7 @@ extern void write_config_file(char *file);
 /* cook.c */
 extern const char *get_filename(t_param *ptr);
 extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int tabs, int hide_uue);
-extern t_bool expand_ctrl_chars(char **line, int *length, size_t cook_width);
+extern t_bool expand_ctrl_chars(char **line, size_t *length, size_t cook_width);
 
 /* curses.c */
 extern OUTC_RETTYPE outchar(OUTC_ARGS);
@@ -157,23 +163,15 @@ extern void word_highlight_string(int row, int col, int size, int color);
 /* debug.c */
 #ifdef DEBUG
 	extern void debug_delete_files(void);
-	extern void debug_nntp(const char *func, const char *fmt, ...);
+	extern void debug_print_file(const char *fname, const char *fmt, ...);
 	extern void debug_print_active(void);
 	extern void debug_print_arts(void);
+	extern void debug_print_bitmap(struct t_group *group, struct t_article *art);
+	extern void debug_print_comment(const char *comment);
 	extern void debug_print_filters(void);
 	extern void debug_print_header(struct t_article *s);
 	extern void debug_print_malloc(int is_malloc, const char *xfile, int line, size_t size);
-#	ifdef NNTP_ABLE
-		extern void debug_print_nntp_extensions(void);
-#	endif /* NNTP_ABLE */
 #endif /* DEBUG */
-#ifdef DEBUG_NEWSRC
-	extern void debug_print_newsrc(struct t_newsrc *NewSrc, FILE *fp);
-#endif /* DEBUG_NEWSRC */
-#if defined(DEBUG) || defined(DEBUG_NEWSRC)
-	extern void debug_print_bitmap(struct t_group *group, struct t_article *art);
-	extern void debug_print_comment(const char *comment);
-#endif /* DEBUG || DEBUG_NEWSRC */
 
 /* envarg.c */
 extern void envargs(int *Pargc, char ***Pargv, const char *envstr);
@@ -247,9 +245,12 @@ extern void postinit_regexp(void);
 #ifdef HAVE_COLOR
 	extern void postinit_colors(void);
 #endif /* HAVE_COLOR */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	extern t_bool utf8_pcre(void);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 /* joinpath.c */
-extern void joinpath(char *result, const char *dir, const char *file);
+extern void joinpath(char *result, size_t result_size, const char *dir, const char *file);
 
 /* keymap.c */
 extern t_bool read_keymap_file(void);
@@ -263,9 +264,9 @@ extern void setup_default_keys(void);
 
 /* list.c */
 extern char *random_organization(char *in_org);
-extern int find_group_index(const char *group);
+extern int find_group_index(const char *group, t_bool ignore_case);
 extern struct t_group *group_add(const char *group);
-extern struct t_group *group_find(const char *group_name);
+extern struct t_group *group_find(const char *group_name, t_bool ignore_case);
 extern unsigned long hash_groupname(const char *group);
 extern void group_rehash(t_bool yanked_out);
 extern void init_group_hash(void);
@@ -302,11 +303,10 @@ extern void expand_active(void);
 extern void expand_art(void);
 extern void expand_newnews(void);
 extern void expand_save(void);
+extern void expand_scope(void);
 extern void init_alloc(void);
 extern void free_all_arrays(void);
 extern void free_art_array(void);
-extern void free_attributes_array(void);
-extern void free_if_not_default(char **attrib, char *deflt);
 extern void free_save_array(void);
 extern void *my_malloc1(const char *file, int line, size_t size);
 extern void *my_calloc1(const char *file, int line, size_t nmemb, size_t size);
@@ -335,7 +335,6 @@ extern int gnksa_split_from(const char *from, char *address, char *realname, int
 extern int get_initials(int respnum, char *s, int maxsize);
 extern int gnksa_do_check_from(const char *from, char *address, char *realname);
 extern int my_chdir(char *path);
-extern int my_isprint(int c);
 extern int my_mkdir(char *path, mode_t mode);
 extern int parse_from(const char *from, char *address, char *realname);
 extern int strfmailer(const char *mail_prog, char *subject, char *to, const char *filename, char *dest, size_t maxsize, const char *format);
@@ -347,7 +346,7 @@ extern long file_size(const char *file);
 extern t_bool backup_file(const char *filename, const char *backupname);
 extern t_bool copy_fp(FILE *fp_ip, FILE *fp_op);
 extern t_bool invoke_cmd(const char *nam);
-extern t_bool invoke_editor(const char *filename, int lineno);
+extern t_bool invoke_editor(const char *filename, int lineno, struct t_group *group);
 extern t_bool mail_check(void);
 extern void append_file(char *old_filename, char *new_filename);
 extern void asfail(const char *file, int line, const char *cond);
@@ -360,9 +359,9 @@ extern void draw_mark_selected(int i);
 extern void draw_percent_mark(long cur_num, long max_num);
 extern void get_author(t_bool thread, struct t_article *art, char *str, size_t len);
 extern void get_cwd(char *buf);
-extern void make_base_group_path(const char *base_dir, const char *group_name, char *group_path);
+extern void make_base_group_path(const char *base_dir, const char *group_name, char *group_path, size_t group_path_len);
 extern void make_group_path(const char *name, char *path);
-extern void process_charsets(char **line, int *max_line_len, const char *network_charset, const char *local_charset, t_bool conv_tex2iso);
+extern void process_charsets(char **line, size_t *max_line_len, const char *network_charset, const char *local_charset, t_bool conv_tex2iso);
 extern void read_input_history_file(void);
 extern void rename_file(const char *old_filename, const char *new_filename);
 extern void show_inverse_video_status(void);
@@ -372,6 +371,9 @@ extern void toggle_inverse_video(void);
 #if defined(CHARSET_CONVERSION) || (defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE))
 	extern char *utf8_valid(char *line);
 #endif /* CHARSET_CONVERSION || (MULTIBYTE_ABLE && !NO_LOCALE) */
+#if defined(NO_LOCALE) || !defined(MULTIBYTE_ABLE)
+	extern int my_isprint(int c);
+#endif /* NO_LOCALE || !MULTIBYTE_ABLE */
 #ifdef CHARSET_CONVERSION
 	extern t_bool buffer_to_network(char *line, int mmnwcharset);
 #endif /* CHARSET_CONVERSION */
@@ -408,30 +410,26 @@ extern void subscribe(struct t_group *group, int sub_state, t_bool get_info);
 extern void thd_mark_read(struct t_group *group, long thread);
 extern void thd_mark_unread(struct t_group *group, long thread);
 extern void set_default_bitmap(struct t_group *group);
-#ifdef DEBUG_NEWSRC
-#	if 0
-	extern void newsrc_test_harness(void);
-#	endif /* 0 */
-#endif /* DEBUG_NEWSRC */
 
 /* nntplib.c */
 extern FILE *get_nntp_fp(FILE *fp);
 extern FILE *get_nntp_wr_fp(FILE *fp);
 extern char *getserverbyfile(const char *file);
-extern char *get_server(char *string, int size);
-extern int get_respcode(char *, size_t);
-extern int get_only_respcode(char *, size_t);
 extern int nntp_open(void);
 extern void nntp_close(void);
-extern void put_server(const char *string);
-extern void u_put_server(const char *string);
 #ifdef NNTP_ABLE
 	extern FILE *nntp_command(const char *, int, char *, size_t);
+	extern char *get_server(char *string, int size);
+	extern int get_respcode(char *, size_t);
+	extern int get_only_respcode(char *, size_t);
+	extern int new_nntp_command(const char *command, int success, char *message, size_t mlen);
+	extern void put_server(const char *string);
+	extern void u_put_server(const char *string);
 #endif /* NNTP_ABLE */
 
 /* nrctbl.c */
-extern int get_newsrcname(char *newsrc_name, const char *nntpserver_name);
-extern void get_nntpserver(char *nntpserver_name, char *nick_name);
+extern int get_newsrcname(char *newsrc_name, size_t newsrc_name_len, const char *nntpserver_name);
+extern void get_nntpserver(char *nntpserver_name, size_t nntpserver_name_len, char *nick_name);
 
 /* options_menu.c */
 extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, enum option_enum option);
@@ -470,7 +468,7 @@ extern time_t parsedate(char *p, TIMEINFO *now);
 #endif /* !HAVE_VSNPRINTF */
 
 /* post.c */
-extern char *checknadd_headers(const char *infile);
+extern char *checknadd_headers(const char *infile, struct t_group *group);
 extern int count_postponed_articles(void);
 extern int mail_to_author(const char *group, int respnum, t_bool copy_text, t_bool with_headers, t_bool raw_data);
 extern int mail_to_someone(const char *address, t_bool confirm_to_mail, t_openartinfo *artinfo, const struct t_group *group);
@@ -531,8 +529,8 @@ extern t_mailcap *get_mailcap_entry(t_part *part, const char *path);
 extern void free_mailcap(t_mailcap *tmailcap);
 
 /* rfc2045.c */
-extern int read_decoded_base64_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read, char **rest);
-extern int read_decoded_qp_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read);
+extern int read_decoded_base64_line(FILE *file, char **line, size_t *max_line_len, const int max_lines_to_read, char **rest);
+extern int read_decoded_qp_line(FILE *file, char **line, size_t *max_line_len, const int max_lines_to_read);
 extern void rfc1521_encode(char *line, FILE *f, int e);
 
 /* rfc2046.c */
@@ -571,7 +569,7 @@ extern void center_line(int line, t_bool inverse, const char *str);
 extern void clear_message(void);
 extern void draw_arrow_mark(int line);
 extern void erase_arrow(void);
-extern void error_message(const char *fmt, ...);
+extern void error_message(unsigned int sdelay, const char *fmt, ...);
 extern void info_message(const char *fmt, ...);
 extern void perror_message(const char *fmt, ...);
 extern void ring_bell(void);
@@ -591,7 +589,7 @@ extern int search_body(struct t_group *group, int current_art, t_bool repeat);
 extern void reset_srch_offsets(void);
 
 /* select.c */
-extern int add_my_group(const char *group, t_bool add);
+extern int add_my_group(const char *group, t_bool add, t_bool ignore_case);
 extern int choose_new_group(void);
 extern int skip_newgroups(void);
 extern void selection_page(int start_groupnum, int num_cmd_line_groups);
@@ -665,7 +663,6 @@ extern void str_lwr(char *str);
 	extern wchar_t *char2wchar_t(const char *str);
 	extern wchar_t *wcspart(const wchar_t *wstr, int columns, t_bool pad);
 	extern wchar_t *wstrunc(const wchar_t *wmessage, int len);
-	extern wchar_t *my_wcsdup(const wchar_t *wstr);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	extern char *render_bidi(const char *str, t_bool *is_rtl);
@@ -713,8 +710,8 @@ extern int thread_page(struct t_group *group, int respnum, int thread_depth, t_p
 extern void fixup_thread(int respnum, t_bool redraw);
 
 /* version.c */
-extern int check_upgrade(char *line, const char *skip, const char *version);
-extern void upgrade_prompt_quit(int reason, const char *file);
+extern enum rc_state check_upgrade(char *line, const char *skip, const char *version);
+extern void upgrade_prompt_quit(enum rc_state reason, const char *file);
 
 /* wildmat.c */
 extern t_bool wildmat(const char *text, char *p, t_bool icase);
