@@ -3,10 +3,10 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2006-10-01
+ *  Updated   : 2009-02-10
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2007 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1997-2009 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -290,19 +290,19 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  */
 #ifdef HAVE_DIRENT_H
 #	include <dirent.h>
-#	define DIR_BUF	struct dirent
 #else
-#	ifdef HAVE_SYS_DIR_H
-#		include <sys/dir.h>
-#	endif /* HAVE_SYS_DIR_H */
 #	ifdef HAVE_SYS_NDIR_H
 #		include <sys/ndir.h>
 #	endif /* HAVE_SYS_NDIR_H */
-#	define DIR_BUF	struct direct
+#	ifdef HAVE_SYS_DIR_H
+#		include <sys/dir.h>
+#	endif /* HAVE_SYS_DIR_H */
+#	if HAVE_NDIR_H
+#		include <ndir.h>
+#	endif /* HAVE_NDIR_H */
 #endif /* HAVE_DIRENT_H */
 
 #ifndef DIR_BUF
-#	include <dirent.h>
 #	define DIR_BUF	struct dirent
 #endif /* !DIR_BUF */
 
@@ -352,6 +352,12 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	endif /* HAVE_PGP && HAVE_GPG */
 #endif /* HAVE_PGP || HAVE_PGPK || HAVE_GPG */
 
+/*
+ * slrnface requires some things
+ */
+#if defined(HAVE_SLRNFACE) && defined(HAVE_MKFIFO) && defined(HAVE_FORK) && defined(HAVE_EXECLP) && defined(HAVE_WAITPID) && !defined(DONT_HAVE_PIPING) && !defined(X_DISPLAY_MISSING)
+#	define XFACE_ABLE
+#endif /* HAVE_SLRNFACE && HAVE_MKFIFO && HAVE_FORK && HAVE_EXECLP && HAVE_WAITPID && !DONT_HAVE_PIPING && !X_DISPLAY_MISSING */
 
 /*
  * Setup support for reading from NNTP
@@ -531,6 +537,8 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* SMALL_MEMORY_MACHINE */
 #define DEFAULT_ACTIVE_NUM	1800
 #define DEFAULT_NEWNEWS_NUM	5
+#define DEFAULT_MAPKEYS_NUM 100	/* ~remappable keys per level (avoid massiv reallocs) */
+#define DEFAULT_SCOPE_NUM	8
 
 #define RCDIR	".tin"
 #define INDEX_MAILDIR	".mail"
@@ -546,7 +554,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define SERVERCONFIG_FILE	"serverrc"
 #define DEFAULT_MAILDIR	"Mail"
 #define DEFAULT_SAVEDIR	"News"
-#define DEFAULT_URL_HANDLER "url_handler.sh"
+#define DEFAULT_URL_HANDLER "url_handler.pl"
 /* Prefixes saved attachments with no set filename */
 #define SAVEFILE_PREFIX		"unknown"
 
@@ -583,11 +591,11 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 /* case sensitive && ^-anchored */
 #define DEFAULT_STRIP_RE_REGEX	"(?:R[eE](?:\\^\\d+|\\[\\d\\])?|A[wW]|Odp|Sv):\\s"
 /* case sensitive */
-#define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):.*\\)\\s*$"
-#define DEFAULT_U8_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l\\x{0142}]o):.*\\)\\s*$"
+#define DEFAULT_STRIP_WAS_REGEX	"(?:(?<=\\S)|\\s)\\((?:[Ww]a[rs]|[Bb]y[l\\xb3]o):.*\\)\\s*$"
+#define DEFAULT_U8_STRIP_WAS_REGEX	"(?:(?<=\\S)|\\s)\\((?:[Ww]a[rs]|[Bb]y[l\\x{0142}]o):.*\\)\\s*$"
 /*
  * overkill regexp for balanced '()':
- * #define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):(?:(?:[^)(])*(?:\\([^)(]*\\))*)+\\)\\s*$"
+ * #define DEFAULT_STRIP_WAS_REGEX	"(?:(?<=\\S)|\\s)\\((?:[Ww]a[rs]|[Bb]y[l\xb3]o):(?:(?:[^)(])*(?:\\([^)(]*\\))*)+\\)\\s*$"
  */
 
 /* case sensitive & ^-anchored */
@@ -599,7 +607,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define SHAR_REGEX	"\\#(?:!\\s?(?:/usr)?/bin/sh|\\s?(?i)this\\sis\\sa\\sshell\\sarchive)"
 
 /* slrn verbatim marks, case sensitive & ^-anchored */
-#define DEFAULT_VERBATIM_BEGIN_REGEX	"#v+"
+#define DEFAULT_VERBATIM_BEGIN_REGEX	"#v\\+"
 #define DEFAULT_VERBATIM_END_REGEX	"#v-"
 
 /*
@@ -674,14 +682,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define NEWSGROUPS_FILE	"newsgroups"
 #define KEYMAP_FILE	"keymap"
 
-
-#ifdef USE_INN_NNTPLIB
-#	include <libinn.h> /* TODO: add configure chek for it */
-#	define _CONF_FROMHOST	"fromhost"
-#	define _CONF_ORGANIZATION	"organization"
-#	define _CONF_SERVER	"server"
-#endif /* USE_INN_NNTPLIB */
-
 #define SIGDASHES "-- \n"
 
 #ifndef BOOL_H
@@ -862,6 +862,13 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define TINRC_CONFIRM_TO_QUIT	(tinrc.confirm_choice == 3 || tinrc.confirm_choice == 4 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
 #define TINRC_CONFIRM_SELECT	(tinrc.confirm_choice == 2 || tinrc.confirm_choice == 5 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
 
+/*
+ * defines for tinrc.auto_cc_bcc
+ */
+#define AUTO_CC					1
+#define AUTO_BCC				2
+#define AUTO_CC_BCC				3
+
 /*
  * defines for tinrc.goto_next_unread
  */
@@ -869,6 +876,14 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define GOTO_NEXT_UNREAD_PGDN	1
 #define GOTO_NEXT_UNREAD_TAB	2
 
+/*
+ * defines for tinrc.trim_article_body
+ */
+#define NUM_TRIM_ARTICLE_BODY	8
+#define SKIP_LEADING			1
+#define SKIP_TRAILING			2
+#define COMPACT_MULTIPLE		4
+
 /*
  * MIME Encodings
  */
@@ -925,7 +940,7 @@ enum {
 #define IGNORE_ART_THREAD(i)	(arts[i].thread != ART_UNTHREADED || (tinrc.kill_level == KILL_NOTHREAD && arts[i].killed))
 
 /*
- * Is this part text/plain ?
+ * Is this part text/plain?
  */
 #define IS_PLAINTEXT(x) \
 			(x->type == TYPE_TEXT && strcasecmp("plain", x->subtype) == 0)
@@ -1008,12 +1023,6 @@ enum {
 #define FEED_AUTOSAVE	5
 #define FEED_REPOST		6
 
-#if 0
-#	define DEBUG_IO(x)	fprintf x
-#else
-#	define DEBUG_IO(x)	/* nothing */
-#endif /* 0 */
-
 
 /*
  * Threading strategies available
@@ -1180,7 +1189,8 @@ enum {
 #	define EXIT_FAILURE	1	/* Failing exit status */
 #endif /* !EXIT_FAILURE */
 
-#define NNTP_ERROR_EXIT	2
+#define NEWS_AVAIL_EXIT 2
+#define NNTP_ERROR_EXIT	3
 
 /*
  * Assertion verifier
@@ -1208,7 +1218,7 @@ enum {
 #define ART_EXPIRED		-2
 
 /*
- * Where does this belong ?? It is overloaded
+ * Where does this belong?? It is overloaded
  */
 #define ART_NORMAL		-1
 
@@ -1303,7 +1313,6 @@ enum {
 /*
  * used in checking article header before posting
  */
-#define NGLIMIT		20	/* Max. num. of crossposted groups before warning */
 #define MAX_COL		78	/* Max. line length before issuing a warning */
 #define MAX_SIG_LINES	4	/* Max. num. of signature lines before warning */
 
@@ -1330,7 +1339,7 @@ struct t_archive {
 };
 
 /*
- *	struct t_msgid - message id
+ * struct t_msgid - message id
  */
 struct t_msgid {
 	struct t_msgid *next;		/* Next in hash chain */
@@ -1383,59 +1392,177 @@ struct t_article {
 	t_bool matched:1;	/* TRUE = article matched regex in feed.c */
 };
 
+/*
+ * struct t_newsheader - holds an array of which news headers to [not] display
+ */
+struct t_newsheader {
+	char **header;	/* array of which headers to [not] display */
+	int num;		/* number of headers in array header */
+};
+
 /*
  * struct t_attribute - configurable attributes on a per group basis
  */
 struct t_attribute {
 	char *maildir;				/* mail dir if other than ~/Mail */
 	char *savedir;				/* save dir if other than ~/News */
-	char *savefile;			/* save articles to specified file */
+	char *savefile;				/* save articles to specified file */
 	char *sigfile;				/* sig file if other than ~/.Sig */
+	char *date_format;			/* format string for the date display */
+	char *editor_format;		/* editor + parameters  %E +%N %F */
 	char *organization;			/* organization name */
+	char *fcc;					/* Fcc folder for mail */
 	char *followup_to;			/* where posts should be redirected */
-	char *quick_kill_scope;			/* quick filter kill scope */
-	char *quick_select_scope;		/* quick filter select scope */
+	char *quick_kill_scope;		/* quick filter kill scope */
+	char *quick_select_scope;	/* quick filter select scope */
 	char *mailing_list;			/* mail list email address */
+	char *news_headers_to_display;	/* which headers to display */
+	char *news_headers_to_not_display;	/* which headers to not display */
 	char *x_headers;			/* extra headers for message header */
-	char *x_body;				/* bolierplate text for message body */
-	char *from;				/* from line */
-	char *news_quote_format;		/* another way to begin a posting format */
+	char *x_body;				/* boilerplate text for message body */
+	char *from;					/* from line */
+	char *news_quote_format;	/* another way to begin a posting format */
 	char *quote_chars;			/* string to precede quoted text on each line */
 	char *mime_types_to_save;	/* MIME content major/minors we want to save */
 #ifdef HAVE_ISPELL
-	char *ispell;			/* path to ispell and options */
+	char *ispell;				/* path to ispell and options */
 #endif /* HAVE_ISPELL */
+#ifdef CHARSET_CONVERSION
+	char *undeclared_charset;		/* charset of articles without MIME charset declaration */
+	int mm_network_charset;			/* network charset */
+#endif /* CHARSET_CONVERSION */
+	struct t_newsheader *headers_to_display;	/* array of which headers to display */
+	struct t_newsheader *headers_to_not_display;	/* array of which headers to not display */
 	unsigned global:1;			/* global/group specific */
-	unsigned quick_kill_header:3;		/* quick filter kill header */
-	unsigned quick_kill_expire:1;		/* quick filter kill limited/unlimited time */
+	unsigned quick_kill_header:3;	/* quick filter kill header */
+	unsigned quick_kill_expire:1;	/* quick filter kill limited/unlimited time */
 	unsigned quick_kill_case:1;		/* quick filter kill case sensitive? */
-	unsigned quick_select_header:3;		/* quick filter select header */
-	unsigned quick_select_expire:1;		/* quick filter select limited/unlimited time */
-	unsigned quick_select_case:1;		/* quick filter select case sensitive? */
+	unsigned quick_select_header:3;	/* quick filter select header */
+	unsigned quick_select_expire:1;	/* quick filter select limited/unlimited time */
+	unsigned quick_select_case:1;	/* quick filter select case sensitive? */
+	unsigned add_posted_to_filter:1;	/* add posted articles to filter */
+	unsigned advertising:1;			/* add User-Agent: -header */
+	unsigned alternative_handling:1;	/* skip multipart/alternative parts */
+	unsigned ask_for_metamail:1;	/* ask before using MIME viewer */
+	unsigned auto_cc_bcc:2;			/* add your name to cc/bcc automatically */
+	unsigned auto_list_thread:1;	/* list thread when entering it using right arrow */
 	unsigned auto_select:1;			/* 0=show all unread, 1='X' just hot arts */
 	unsigned auto_save:1;			/* 0=none, 1=save */
 	unsigned batch_save:1;			/* 0=none, 1=save -S/mail -M */
-	unsigned delete_tmp_files:1;		/* 0=leave, 1=delete */
-	unsigned show_only_unread:1;		/* 0=all, 1=only unread */
-	unsigned thread_arts:3;			/* 0=unthread, 1=subject, 2=refs, 3=both, 4=multipart, 5=percentage */
-	unsigned thread_perc:7;			/* percentage thrreading threshold */
+	unsigned delete_tmp_files:1;	/* 0=leave, 1=delete */
+	unsigned group_catchup_on_exit:1;	/* ask if read groups are to be marked read */
+	unsigned mail_8bit_header:1;	/* allow 8bit chars. in header of mail message */
+	unsigned mail_mime_encoding:2;
+	unsigned mark_ignore_tags:1;	/* Ignore tags for GROUP_MARK_THREAD_READ/THREAD_MARK_ARTICLE_READ */
+	unsigned mark_saved_read:1;		/* mark saved article/thread as read */
+	unsigned pos_first_unread:1;	/* position cursor at first/last unread article */
+	unsigned post_8bit_header:1;	/* allow 8bit chars. in header when posting to newsgroup */
+	unsigned post_mime_encoding:2;
+	unsigned post_process_view:1;	/* set TRUE to invoke mailcap viewer app */
+#ifndef DISABLE_PRINTING
+	unsigned print_header:1;		/* print all of mail header or just Subject: & From lines */
+#endif /* !DISABLE_PRINTING */
+	unsigned process_only_unread:1;	/* save/print//mail/pipe unread/all articles */
+	unsigned prompt_followupto:1;	/* display empty Followup-To header in editor */
+	unsigned show_only_unread_arts:1;	/* 0=all, 1=only unread */
+	unsigned sigdashes:1;			/* set TRUE to prepend every signature with dashes */
+	unsigned signature_repost:1;	/* set TRUE to add signature when reposting articles */
+	unsigned start_editor_offset:1;	/* start editor with line offset */
+	unsigned thread_articles:3;			/* 0=unthread, 1=subject, 2=refs, 3=both, 4=multipart, 5=percentage */
+	unsigned thread_catchup_on_exit:1;	/* catchup thread with left arrow key or not */
+	unsigned thread_perc:7;			/* percentage threading threshold */
 	unsigned show_author:2;			/* 0=none, 1=name, 2=addr, 3=both */
 	unsigned show_info:2;			/* 0=none, 1=lines, 2=score, 3=both */
-	unsigned sort_art_type:4;		/* 0=none, 1=subj descend, 2=subj ascend,
+	unsigned show_signatures:1;		/* 0=none, 1=show signatures */
+	unsigned trim_article_body:3;	/* 0=Don't trim article body, 1=Skip leading blank lines,
+						2=Skip trailing blank lines, 3=Skip leading and trailing blank lines,
+						4=Compact multiple blank lines between textblocks,
+						5=Compact multiple blank lines between textblocks and skip leading blank lines,
+						6=Compact multiple blank lines between textblocks and skip trailing blank lines,
+						7=Compact multiple blank lines between textblocks and skip leading and trailing
+						  blank lines */
+	unsigned verbatim_handling:1;	/* 0=none, 1=detect verbatim blocks */
+	unsigned wrap_on_next_unread:1;	/* Wrap around threads when searching next unread article */
+	unsigned sort_article_type:4;		/* 0=none, 1=subj descend, 2=subj ascend,
 						   3=from descend, 4=from ascend,
 						   5=date descend, 6=date ascend,
 						   7=score descend, 8=score ascend */
-	unsigned sort_threads_type:3;		/* 0=none, 1=score descend, 2=score ascend,
+	unsigned sort_threads_type:3;	/* 0=none, 1=score descend, 2=score ascend,
 						   3=last posting date descend, 4=last posting date ascend */
-	unsigned int post_proc_type:2;		/* 0=none, 1=shar, 2=uudecode */
-	unsigned int x_comment_to:1;		/* insert X-Comment-To: in Followup */
-	unsigned int tex2iso_conv:1;		/* Convert TeX2ISO */
-	unsigned int mime_forward:1;		/* forward articles as attachment or inline */
-	char *fcc;							/* Fcc folder for mail */
+	unsigned post_process_type:2;	/* 0=none, 1=shar, 2=uudecode */
+	unsigned x_comment_to:1;	/* insert X-Comment-To: in Followup */
+	unsigned tex2iso_conv:1;	/* Convert TeX2ISO */
+	unsigned mime_forward:1;	/* forward articles as attachment or inline */
+};
+
+/*
+ * struct t_attribute_state - holds additional information
+ * about numeric attributes within a scope
+ */
+struct t_attribute_state {
+	unsigned add_posted_to_filter:1;
+	unsigned advertising:1;
+	unsigned alternative_handling:1;
+	unsigned ask_for_metamail:1;
+	unsigned auto_cc_bcc:1;
+	unsigned auto_list_thread:1;
+	unsigned auto_save:1;
+	unsigned auto_select:1;
+	unsigned batch_save:1;
+	unsigned delete_tmp_files:1;
+	unsigned group_catchup_on_exit:1;
+	unsigned mail_8bit_header:1;
+	unsigned mail_mime_encoding:1;
+	unsigned mark_ignore_tags:1;
+	unsigned mark_saved_read:1;
+	unsigned mime_forward:1;
 #ifdef CHARSET_CONVERSION
-	int mm_network_charset;				/* network charset */
-	char *undeclared_charset;			/* charset of articles without MIME charset declaration */
+	unsigned mm_network_charset:1;
 #endif /* CHARSET_CONVERSION */
+	unsigned pos_first_unread:1;
+	unsigned post_8bit_header:1;
+	unsigned post_mime_encoding:1;
+	unsigned post_process_view:1;
+	unsigned post_process_type:1;
+#ifndef DISABLE_PRINTING
+	unsigned print_header:1;
+#endif /* !DISABLE_PRINTING */
+	unsigned process_only_unread:1;
+	unsigned prompt_followupto:1;
+	unsigned quick_kill_case:1;
+	unsigned quick_kill_expire:1;
+	unsigned quick_kill_header:1;
+	unsigned quick_select_case:1;
+	unsigned quick_select_expire:1;
+	unsigned quick_select_header:1;
+	unsigned show_author:1;
+	unsigned show_info:1;
+	unsigned show_only_unread_arts:1;
+	unsigned show_signatures:1;
+	unsigned sigdashes:1;
+	unsigned signature_repost:1;
+	unsigned sort_article_type:1;
+	unsigned sort_threads_type:1;
+	unsigned start_editor_offset:1;
+	unsigned tex2iso_conv:1;
+	unsigned thread_articles:1;
+	unsigned thread_catchup_on_exit:1;
+	unsigned thread_perc:1;
+	unsigned trim_article_body:1;
+	unsigned verbatim_handling:1;
+	unsigned wrap_on_next_unread:1;
+	unsigned x_comment_to:1;
+};
+
+/*
+ * struct t_scope
+ */
+struct t_scope {
+	char *scope;				/* scope for these group attributes */
+	struct t_attribute *attribute;	/* the attributes itself */
+	struct t_attribute_state *state;	/* additional information about numeric attributes */
+	unsigned global:1;			/* TRUE for scopes from global_attributes_file */
+	unsigned temp:1;			/* temporary scope for menu changes at group level */
 };
 
 /*
@@ -1445,7 +1572,7 @@ struct t_attribute {
  * struct t_newsrc - newsrc related info.
  */
 struct t_newsrc {
-	t_bool present:1;		/* update newsrc ? */
+	t_bool present:1;		/* update newsrc? */
 	long num_unread;		/* unread articles in group */
 	long xmax;			/* newsrc max */
 	long xmin;			/* newsrc min */
@@ -1460,7 +1587,7 @@ struct t_newsrc {
  * struct t_group - newsgroup info from active file
  */
 struct t_group {
-	char *name;			/* newsgroup / mailbox name */
+	char *name;			/* newsgroup/mailbox name */
 	char *aliasedto;		/* =new.group in active file, NULL if not */
 	char *description;	/* text from NEWSLIBDIR/newsgroups file */
 	char *spooldir;		/* groups spool directory */
@@ -1468,7 +1595,7 @@ struct t_group {
 	long count;			/* article number count */
 	long xmax;			/* max. article number */
 	long xmin;			/* min. article number */
-	unsigned int type:4;		/* grouptype - newsgroup / mailbox / savebox */
+	unsigned int type:4;		/* grouptype - newsgroup/mailbox/savebox */
 	t_bool inrange:1;		/* TRUE if group selected via # range command */
 	t_bool read_during_session:1;	/* TRUE if group entered during session */
 	t_bool art_was_posted:1;	/* TRUE if art was posted to group */
@@ -1539,7 +1666,7 @@ struct t_filter {
 	char *xref;			/* groups in xref line */
 	time_t time;			/* expire time in seconds */
 	struct t_filter *next;		/* next rule valid in group */
-	unsigned int inscope:4;		/* if group matches scope ie. 'comp.os.*' */
+	unsigned int inscope:4;		/* if group matches scope e.g. 'comp.os.*' */
 	unsigned int icase:2;		/* Case sensitive filtering */
 	unsigned int fullref:4;		/* use full references or last entry only */
 };
@@ -1640,7 +1767,7 @@ struct t_option {
 	int *variable;		/* ptr to variable to change */
 	constext **opt_list;	/* ptr to list entries if OPT_LIST */
 	int opt_count;		/* no. of list entries if OPT_LIST */
-	struct opttxt *txt;	/* ptr to information / help on option */
+	struct opttxt *txt;	/* ptr to information/help on option */
 };
 
 /*
@@ -1690,7 +1817,7 @@ typedef struct _TIMEINFO {
 /*
  * mailcap fields
  * the x-token field is missing, we would need something like
- * struct t_xtoken { char *xtoken; t_xtoken *next; } for that...
+ * struct t_xtoken { char *xtoken; t_xtoken *next; } for that ...
  */
 typedef struct {
 	char *type;		/* content-type, mandatory */
@@ -1852,7 +1979,7 @@ typedef int (*t_compfunc)(t_comptype, t_comptype);
 #endif /* !F_OK */
 
 /* Various function redefinitions */
-#ifdef USE_DBMALLOC
+#if defined(USE_DBMALLOC) || defined(USE_DMALLOC)
 #	define my_malloc(size)	malloc(size)
 #	define my_calloc(nmemb, size)	calloc((nmemb), (size))
 #	define my_realloc(ptr, size)	realloc((ptr), (size))
@@ -1860,7 +1987,7 @@ typedef int (*t_compfunc)(t_comptype, t_comptype);
 #	define my_malloc(size)	my_malloc1(__FILE__, __LINE__, (size))
 #	define my_calloc(nmemb, size)	my_calloc1(__FILE__, __LINE__, (nmemb), (size))
 #	define my_realloc(ptr, size)	my_realloc1(__FILE__, __LINE__, (ptr), (size))
-#endif /* USE_DBMALLOC */
+#endif /* USE_DBMALLOC || USE_DMALLOC */
 
 #define ARRAY_SIZE(array)	((int) (sizeof(array) / sizeof(array[0])))
 
@@ -1869,8 +1996,8 @@ typedef int (*t_compfunc)(t_comptype, t_comptype);
 
 #define BlankIfNull(p)	((p) ? (p) : "")
 
-#define my_group_find(x)	add_my_group(x, FALSE)
-#define my_group_add(x)		add_my_group(x, TRUE)
+#define my_group_find(x)	add_my_group(x, FALSE, FALSE)
+#define my_group_add(x, y)		add_my_group(x, TRUE, y)
 #define for_each_group(x)	for (x = 0; x < num_active; x++)
 #define for_each_art(x)		for (x = 0; x < top_art; x++)
 #define for_each_art_in_thread(x, y)	for (x = (int) base[y]; x >= 0; x = arts[x].thread)
@@ -1956,6 +2083,9 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #ifdef USE_DMALLOC
 #	include <dmalloc.h>
 #	define DMALLOC_FUNC_CHECK
+#	ifdef HAVE_STRDUP
+#		define my_strdup(s) strdup((s))
+#	endif /* HAVE_STRDUP */
 #endif /* USE_DMALLOC */
 
 #ifdef DOALLOC
@@ -1980,9 +2110,9 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 
 #ifndef my_tmpfile_only
 /*
- * shortcut if we arn't interrested in the tmpfiles filename/location
+ * shortcut if we aren't interested in the tmpfiles filename/location
  * argument can't be a pointer and if argument is changed on return
- * we must unlik the tmp-file ourself
+ * we must unlink the tmp-file ourself
  */
 #	define my_tmpfile_only(a)	my_tmpfile(a, sizeof(a) - 1, FALSE, (char *) 0)
 #endif /* !my_tmpfile_only */
@@ -2023,7 +2153,7 @@ extern struct tm *localtime(time_t *);
 #ifdef CLOSEDIR_VOID
 #	define CLOSEDIR(DIR)	closedir(DIR)
 #else
-#	define CLOSEDIR(DIR)	if (closedir(DIR)) error_message("closedir() failed: %s %s", __FILE__, __LINE__)
+#	define CLOSEDIR(DIR)	if (closedir(DIR)) error_message(2, "closedir() failed: %s %s", __FILE__, __LINE__)
 #endif /* CLOSEDIR_VOID */
 
 #ifdef HAVE_GETTIMEOFDAY
@@ -2048,9 +2178,14 @@ extern struct tm *localtime(time_t *);
 
 /* libcanlock */
 #ifdef USE_CANLOCK
-#	include "../libcanlock/canlock.h"
+#	include "../libcanlock/include/canlock.h"
 #endif /* USE_CANLOCK */
 
+/* gsasl */
+#ifdef USE_SASL
+#	include <gsasl.h>
+#endif /* USE_SASL */
+
 /* snprintf(), vsnprintf() */
 #ifndef HAVE_SNPRINTF
 #	define snprintf	plp_snprintf
@@ -2088,5 +2223,20 @@ extern struct tm *localtime(time_t *);
 #define TIN_ARTICLE_NAME	".article"
 #define TIN_CANCEL_NAME	".cancel"
 #define TIN_LETTER_NAME	".letter"
+#define TIN_BUGREPORT_NAME	".bugreport"
+
+/* read_news_active_file() / open_newsgroups_fp() */
+#ifndef DISABLE_PIPELINING
+#	define PIPELINE_LIMIT 30
+#endif /* DISABLE_PIPELINING */
+
+#ifndef DEBUG_H
+#	include "debug.h"
+#endif /* !DEBUG_H */
+
+struct t_overview_fmt {
+	char *name;
+	enum f_type type;
+};
 
 #endif /* !TIN_H */
