@@ -3,10 +3,10 @@
  *  Module    : art.c
  *  Author    : I.Lea & R.Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2006-06-21
+ *  Updated   : 2009-01-15
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,11 +57,11 @@ int top_art = 0;				/* # of articles in arts[] */
 /*
  * Local prototypes
  */
-static FILE *open_art_header(long art, long *next);
+static FILE *open_art_header(char *groupname, long art, long *next);
 static FILE *open_xover_fp(struct t_group *group, const char *mode, long min, long max, t_bool local);
 static char *find_nov_file(struct t_group *group, int mode);
 static char *print_date(time_t secs);
-static char *print_from(struct t_article *article);
+static char *print_from(struct t_group *group, struct t_article *article);
 static int artnum_comp(t_comptype p1, t_comptype p2);
 static int base_comp(t_comptype p1, t_comptype p2);
 static int date_comp_asc(t_comptype p1, t_comptype p2);
@@ -135,7 +135,7 @@ find_base(
 		if (grpmenu.max >= max_art)
 			expand_art();
 
-		if (group->attribute->show_only_unread) {
+		if (group->attribute->show_only_unread_arts) {
 			if (arts[i].status != ART_READ)
 				base[grpmenu.max++] = i;
 			else {
@@ -198,7 +198,6 @@ static long
 setup_hard_base(
 	struct t_group *group)
 {
-	char buf[NNTP_STRLEN];
 	long art;
 	long total = 0;
 
@@ -207,15 +206,17 @@ setup_hard_base(
 	/*
 	 * If reading with NNTP, issue a LISTGROUP
 	 */
-	if (read_news_via_nntp && group->type == GROUP_TYPE_NEWS) {
+	if (read_news_via_nntp && !read_saved_news && group->type == GROUP_TYPE_NEWS) {
 #ifdef NNTP_ABLE
+		char buf[NNTP_STRLEN];
 		FILE *fp;
 
 #	ifdef BROKEN_LISTGROUP
 		/*
 		 * Some nntp servers are broken and need an extra GROUP command
 		 * (reported by reorx@irc.pl). This affects (old?) versions of
-		 * nntpcache and leafnode. Usually this should not be needed.
+		 * nntpcache, leafnode and SurgeNews. Usually this should not be
+		 * needed.
 		 */
 		snprintf(buf, sizeof(buf), "GROUP %s", group->name);
 		if (nntp_command(buf, OK_GROUP, NULL, 0) == NULL)
@@ -224,13 +225,21 @@ setup_hard_base(
 
 		/*
 		 * See if LISTGROUP works
+		 *
+		 * think about something like:
+		 * if (nntp_caps.type == CAPABILITIES && tinrc.getart_limit != 0) {
+		 *    calculate some usefull min vals, eg. if getart_limit < 0
+		 *    use lowest unread art + 2*getart_limit (to include holes)
+		 *    snprintf(buf, sizeof(buf), "LISTGROUP %s %d-%d, group->name, min, max);
+		 * }
 		 */
 		snprintf(buf, sizeof(buf), "LISTGROUP %s", group->name);
 		if ((fp = nntp_command(buf, OK_GROUP, NULL, 0)) != NULL) {
 			char *ptr;
 
 #	ifdef DEBUG
-			debug_nntp("setup_hard_base", buf);
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "setup_hard_base(%s)", buf);
 #	endif /* DEBUG */
 
 			while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
@@ -263,6 +272,10 @@ setup_hard_base(
 			if (sscanf(line, "%ld %ld %ld", &count, &start, &last) != 3)
 				return -1;
 
+#	ifdef DEBUG
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "setup_hard_base(%s)", buf);
+#	endif /* DEBUG */
 			/*
 			 * TODO: AFAICS "total" and "count" arn't used in the code below
 			 *       anymore, why do we bother to set them?
@@ -284,15 +297,16 @@ setup_hard_base(
 	} else {
 		DIR *d;
 		DIR_BUF *e;
+		char group_path[PATH_LEN];
 
-		make_base_group_path(group->spooldir, group->name, buf);
+		make_base_group_path(group->spooldir, group->name, group_path, sizeof(group_path));
 
-		if (access(buf, R_OK) != 0) {
-			error_message(_(txt_not_exist));
+		if (access(group_path, R_OK) != 0) {
+			error_message(2, _(txt_not_exist));
 			return -1;
 		}
 
-		if ((d = opendir(buf)) != NULL) {
+		if ((d = opendir(group_path)) != NULL) {
 			while ((e = readdir(d)) != NULL) {
 				art = atol(e->d_name);
 				if (art >= 1) {
@@ -351,7 +365,7 @@ index_group(
 	free_art_array();
 	free_msgids();
 
-	BegStopWatch("setup_hard_base");
+	BegStopWatch("setup_hard_base()");
 
 	/*
 	 * Get list of valid article numbers
@@ -362,10 +376,12 @@ index_group(
 	EndStopWatch();
 	PrintStopWatch();
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Before read_overview");
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
+		debug_print_comment("Before read_overview");
+		debug_print_bitmap(group, NULL);
+	}
+#endif /* DEBUG */
 
 	min = grpmenu.max ? base[0] : group->xmin;
 	max = grpmenu.max ? base[grpmenu.max - 1] : min - 1;
@@ -431,23 +447,33 @@ index_group(
 	 * Add any articles to arts[] that are new or were killed
 	 */
 	if (total > 0) {
+		/*
+		 * TODO
+		 * his doesn't honor tinrc.getart_limit
+		 * something like (tinrc.getart_limit ? min : last_read_article)
+		 * as 3rd arg to read_art_headers() might solve this
+		 */
 		if ((changed += read_art_headers(group, total, last_read_article)) == -1)
 			return FALSE;		/* user aborted indexing */
 	}
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Before parse_unread_arts()");
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
+		debug_print_comment("Before parse_unread_arts()");
+		debug_print_bitmap(group, NULL);
+	}
+#endif /* DEBUG */
 	/*
 	 * Do this before calling art_mark(,, ART_READ) if you want
 	 * the unread count to be correct.
 	 */
 	parse_unread_arts(group);
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("After parse_unread_arts()");
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
+		debug_print_comment("After parse_unread_arts()");
+		debug_print_bitmap(group, NULL);
+	}
+#endif /* DEBUG */
 
 	/*
 	 * Stat all articles to see if any have expired
@@ -455,9 +481,10 @@ index_group(
 	for_each_art(i) {
 		if (arts[i].thread == ART_EXPIRED) {
 			changed++;
-#ifdef DEBUG_NEWSRC
-			debug_print_comment("art.c: index_group() purging...");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+			if (debug & DEBUG_NEWSRC)
+				debug_print_comment("art.c: index_group() purging...");
+#endif /* DEBUG */
 			art_mark(group, &arts[i], ART_READ);
 		}
 	}
@@ -480,7 +507,7 @@ index_group(
 	 */
 	filtered = filter_articles(group);
 
-	BegStopWatch("make_thread");
+	BegStopWatch("make_threads()");
 
 	/*
 	 * Thread the group
@@ -525,12 +552,14 @@ find_first_unread(
  */
 static FILE *
 open_art_header(
+	char *groupname,
 	long art,
 	long *next)
 {
 	char buf[NNTP_STRLEN];
 #ifdef NNTP_ABLE
 	FILE *fp;
+	int i;
 
 	if (read_news_via_nntp && CURR_GROUP.type == GROUP_TYPE_NEWS) {
 		/*
@@ -549,8 +578,32 @@ open_art_header(
 		 * HEAD failed, try to find NEXT
 		 * Should return "223 artno message-id more text...."
 		 */
-		if (nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf)))
-			*next = atoi(buf);		/* Set next art number */
+		i = new_nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf));
+		switch (i) {
+			case OK_NOTEXT:
+				*next = atoi(buf);		/* Set next art number */
+				break;
+
+#	ifndef BROKEN_LISTGROUP
+			/*
+			 * might happen if LISTGROUP doesn't select group, but
+			 * we are not -DBROKEN_LISTGROUP
+			 */
+			case ERR_NCING:
+				snprintf(buf, sizeof(buf), "GROUP %s", groupname);
+				if (nntp_command(buf, OK_GROUP, NULL, 0) == NULL)
+					return NULL;
+				snprintf(buf, sizeof(buf), "HEAD %ld", art);
+				if ((fp = nntp_command(buf, OK_HEAD, NULL, 0)) != NULL)
+					return fp;
+				if (nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf)))
+					*next = atoi(buf);
+				break;
+#	endif /*! BROKEN_LISTGROUP */
+
+			default:
+				break;
+		}
 
 		return NULL;
 	}
@@ -596,7 +649,7 @@ read_art_headers(
 		char buf[PATH_LEN];
 
 		get_cwd(dir);
-		make_base_group_path(group->spooldir, group->name, buf);
+		make_base_group_path(group->spooldir, group->name, buf, sizeof(buf));
 		my_chdir(buf);
 	}
 
@@ -616,7 +669,7 @@ read_art_headers(
 		/*
 		 * Try and open the article
 		 */
-		if ((fp = open_art_header(art, &head_next)) == NULL)
+		if ((fp = open_art_header(group->name, art, &head_next)) == NULL)
 			continue;
 
 		/*
@@ -639,10 +692,12 @@ read_art_headers(
 
 		if (!res) {
 #ifdef DEBUG
-			char buf[PATH_LEN];
+			if (debug & DEBUG_NNTP) {
+				char buf[PATH_LEN];
 
-			snprintf(buf, sizeof(buf), "FAILED parse_headers(%ld)", art);
-			debug_nntp("read_art_headers", buf);
+				snprintf(buf, sizeof(buf), "FAILED parse_headers(%ld)", art);
+				debug_print_file("NNTP", "read_art_headers() %s", buf);
+			}
 #endif /* DEBUG */
 			continue;
 		}
@@ -724,6 +779,7 @@ thread_by_subject(
 #endif /* 0 */
 }
 
+
 /*
  * This Threading algorithm threads articles into 'buckets' where each bucket
  * contains all the articles which match the root article's subject line to
@@ -796,6 +852,7 @@ thread_by_percentage(
 	}
 }
 
+
 /*
  * This was brought over from tags.c, however this version doesn't not
  * opperate on base_index
@@ -1023,12 +1080,12 @@ make_threads(
 	t_bool rethread)
 {
 	if (!cmd_line && !batch_mode)
-		info_message((group->attribute->thread_arts == THREAD_NONE ? _(txt_unthreading_arts) : _(txt_threading_arts)));
+		info_message((group->attribute->thread_articles == THREAD_NONE ? _(txt_unthreading_arts) : _(txt_threading_arts)));
 
 #ifdef DEBUG
-	if (debug == 2)
-		error_message("rethread=[%d]  thread_arts=[%d]  attr_thread_arts=[%d]",
-				rethread, tinrc.thread_articles, group->attribute->thread_arts);
+	if (debug & DEBUG_MISC)
+		error_message(2, "rethread=[%d]  thread_articles=[%d]  attr_thread_articles=[%d]",
+				rethread, tinrc.thread_articles, group->attribute->thread_articles);
 #endif /* DEBUG */
 
 	/*
@@ -1037,7 +1094,7 @@ make_threads(
 	 * on arts[] and so the base messages under all threading systems
 	 * will be sorted in this way.
 	 */
-	sort_arts(group->attribute->sort_art_type);
+	sort_arts(group->attribute->sort_article_type);
 
 	/*
 	 * Reset all the ptrs to articles following the above sort
@@ -1048,7 +1105,7 @@ make_threads(
 	 * The threading pointers need to be reset if re-threading
 	 * If using ref threading, revector the links back to the articles
 	 */
-	if (rethread || group->attribute->thread_arts) {
+	if (rethread || group->attribute->thread_articles) {
 		int i;
 
 		for_each_art(i) {
@@ -1060,14 +1117,15 @@ make_threads(
 			/* Should never happen if tree is built properly */
 			if (arts[i].refptr == 0) {
 #ifdef DEBUG
-				my_fprintf(stderr, "\nError  : art->refptr is NULL\n");
-				my_fprintf(stderr, "Artnum : %ld\n", arts[i].artnum);
-				my_fprintf(stderr, "Subject: %s\n", arts[i].subject);
-				my_fprintf(stderr, "From   : %s\n", arts[i].from);
-				assert(arts[i].refptr != 0);
-#else
-				continue;
+				if (debug & DEBUG_REFS) {
+					my_fprintf(stderr, "\nError  : art->refptr is NULL\n");
+					my_fprintf(stderr, "Artnum : %ld\n", arts[i].artnum);
+					my_fprintf(stderr, "Subject: %s\n", arts[i].subject);
+					my_fprintf(stderr, "From   : %s\n", arts[i].from);
+					assert(arts[i].refptr != 0);
+				} else
 #endif /* DEBUG */
+					continue;
 			}
 			arts[i].refptr->article = i;
 		}
@@ -1076,7 +1134,7 @@ make_threads(
 	/*
 	 * Do the right thing according to the threading strategy
 	 */
-	switch (group->attribute->thread_arts) {
+	switch (group->attribute->thread_articles) {
 		case THREAD_NONE:
 			break;
 
@@ -1207,7 +1265,6 @@ parse_headers(
 {
 	char art_from_addr[HEADER_LEN];
 	char art_full_name[HEADER_LEN];
-	char art_trunc_subj[HEADER_LEN];
 	char *hdr, *ptr;
 	unsigned int lineno = 0;
 	unsigned int max_lineno = 25;
@@ -1302,25 +1359,17 @@ parse_headers(
 
 				/* Received:  If found it's probably a mail article */
 				if (!got_received) {
-					if ((hdr = parse_header(ptr + 1, "eceived", FALSE, FALSE))) {
+					if (parse_header(ptr + 1, "eceived", FALSE, FALSE)) {
 						max_lineno <<= 1;		/* double the max number of line to read for mails */
 						got_received = TRUE;
 					}
 				}
 				break;
 
-			/*
-			 * FIXME: Subject: truncation is a HACK and it's not multibyte safe
-			 *        the core problem are probably fixed length buffers
-			 *        (i.e. in rfc1522_encode() called from write_overview()
-			 *         with the data read in here).
-			 */
 			case 'S':	/* Subject:  mandatory */
 				if (!h->subject) {
-					if ((hdr = parse_header(ptr + 1, "ubject", FALSE, FALSE))) {
-						strncpy(art_trunc_subj, eat_re(eat_tab(convert_to_printable(rfc1522_decode(hdr))), FALSE), sizeof(art_trunc_subj) - 1);
-						h->subject = hash_str(art_trunc_subj);
-					}
+					if ((hdr = parse_header(ptr + 1, "ubject", FALSE, FALSE)))
+						h->subject = hash_str(eat_re(eat_tab(convert_to_printable(rfc1522_decode(hdr))), FALSE));
 				}
 				break;
 
@@ -1363,11 +1412,6 @@ parse_headers(
 }
 
 
-#ifdef DEBUG
-#	define handle_overview_fmt_error()	else oerror += 1<<count
-#else
-#	define handle_overview_fmt_error()
-#endif /* DEBUG */
 /*
  * Read in an overview index file. Fields are separated by TAB.
  * return the number of expired articles encountered or -1 if the user aborted
@@ -1397,6 +1441,7 @@ read_overview(
 {
 	FILE *fp;
 	char *ptr;
+	char *q;
 	char *buf;
 	char *group_msg;
 	char art_full_name[HEADER_LEN];
@@ -1405,9 +1450,7 @@ read_overview(
 	int expired = 0;
 	long artnum;
 	struct t_article *art;
-#ifdef DEBUG
-	unsigned int oerror = 0;
-#endif /* DEBUG */
+	size_t over_fields = 1;
 
 	/*
 	 * open the overview file (whether it be local or via nntp)
@@ -1419,6 +1462,35 @@ read_overview(
 		group->xmax = max;
 
 	group_msg = fmt_string(_(txt_group), cCOLS - strlen(_(txt_group)) + 2 - 3, group->name);
+
+	/* get the number of fields per over-record as announced by LIST OVERVIEW.FMT */
+	if (ofmt) {
+		for (; ofmt[over_fields].name; over_fields++)
+			;
+	}
+	if (!--over_fields) { /* e.g. nntp_caps.type == CAPABILITIES && !nntp_caps.list_overview_fmt -> assume defaults */
+		ofmt = my_realloc(ofmt, sizeof(struct t_overview_fmt) * (8 + 1));
+		ofmt[0].type = OVER_T_INT;
+		ofmt[0].name = my_strdup("Artnum:");
+		ofmt[1].type = OVER_T_STRING;
+		ofmt[1].name = my_strdup("Subject:");
+		ofmt[2].type = OVER_T_STRING;
+		ofmt[2].name = my_strdup("From:");
+		ofmt[3].type = OVER_T_STRING;
+		ofmt[3].name = my_strdup("Date:");
+		ofmt[4].type = OVER_T_STRING;
+		ofmt[4].name = my_strdup("Message-ID:");
+		ofmt[5].type = OVER_T_STRING;
+		ofmt[5].name = my_strdup("References:");
+		ofmt[6].type = OVER_T_INT;
+		ofmt[6].name = my_strdup("Bytes:");
+		ofmt[7].type = OVER_T_INT;
+		ofmt[7].name = my_strdup("Lines:");
+		ofmt[8].type = OVER_T_ERROR;
+		ofmt[8].name = NULL;
+		over_fields = 7;
+	}
+
 	while ((buf = tin_fgets(fp, FALSE)) != NULL) {
 		if (need_resize) {
 			handle_resize((need_resize == cRedraw) ? TRUE : FALSE);
@@ -1473,83 +1545,224 @@ read_overview(
 		 *       to check for additions like we do with xref_supported
 		 */
 		for (count = 1; (ptr = tin_strtok(NULL, "\t")) != NULL; count++) {
-			switch (count) {
-				case 1:		/* Subject */
-					/*
-					 * TODO: As eat_re() is also called in batch_mode we need
-					 *       to init (all) regexes (but do not use the others).
-					 *       Calling eat_re() isn't very wise at all as we use
-					 *       the modified subject for -N/-M batch opperations
-					 *       so ppl. can't tell from the subject if the posting
-					 *       was a reply or not.
-					 */
-					art->subject = hash_str(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr))), FALSE));
-					break;
-
-				case 2:		/* From */
-					art->gnksa_code = parse_from(ptr, art_from_addr, art_full_name);
-					art->from = hash_str(buffer_to_ascii(art_from_addr));
-
-					if (*art_full_name)
-						art->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name))));
-					break;
-
-				case 3:		/* Date */
-					art->date = parsedate(ptr, (TIMEINFO *) 0);
+			/* skip unexpected tailing fields */
+			if (count > over_fields) {
 #ifdef DEBUG
-					if (art->date == (time_t) -1)
-						oerror += 1<<count;
-#endif /* DEBUG */
-					break;
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "OVER(%d) Unexpected overview-field %d of %d: %s", artnum, count, over_fields, ptr);
+#endif	/* DEBUG */
 
-				case 4:		/* Message-ID */
-					if (*ptr)
-						art->msgid = my_strdup(ptr);
-					handle_overview_fmt_error();
-					break;
-
-				case 5:		/* References */
-					if (*ptr)
-						art->refs = my_strdup(ptr);
-					break;
-
-				case 6:		/* Bytes */
+				/* "common error" Xref:full in overview-data but not in OVERVIEW.FTM */
+				if (count == over_fields + 1) {
+					if (!strncasecmp(ptr, "Xref: ", 6)) {
 #ifdef DEBUG
-					if (!isdigit((unsigned char) *ptr))
-						oerror += 1<<count;
-#endif /* DEBUG */
-					break;
-
-				case 7:		/* Lines */
-					if (isdigit((unsigned char) *ptr))
-						art->line_count = atoi(ptr);
-					handle_overview_fmt_error();
-					break;
-
-				case 8:		/* Xref: */
-					if (!xref_supported)
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "OVER: found unexpected Xref: on semi std. position");
+#endif  /* DEBUG */
+						over_fields++;
+						ofmt = my_realloc(ofmt, sizeof(struct t_overview_fmt) * (over_fields + 2)); /* + 2 = artnum and end-marker */
+						ofmt[over_fields].type = OVER_T_FSTRING;
+						ofmt[over_fields].name = my_strdup("Xref:");
+						ofmt[over_fields + 1].type = OVER_T_ERROR;
+						ofmt[over_fields + 1].name = NULL;
+						xref_supported = TRUE;
+					} else
 						continue;
-					/* TODO: crosscheck artnum against Xref:-line (if Xref:full) */
-					if ((ptr = parse_header(ptr, "Xref", FALSE, FALSE)) != NULL)
-						art->xref = my_strdup(ptr);
-					handle_overview_fmt_error();
-					break;
+				} else
+					continue;
+			}
+
+			if (expensive_over_parse) { /* strange order */
+				/* madatory fields */
+				if (ofmt[count].type == OVER_T_STRING) {
+					if (!strcasecmp(ofmt[count].name, "Subject:")) {
+						if (*ptr)
+							art->subject = hash_str(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr))), FALSE));
+						else {
+							art->subject = hash_str("");
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) empty overview-field %s", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "From:")) {
+						if (*ptr) {
+							art->gnksa_code = parse_from(ptr, art_from_addr, art_full_name);
+							art->from = hash_str(buffer_to_ascii(art_from_addr));
+							if (*art_full_name)
+								art->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name))));
+						} else {
+							art->from = hash_str("");
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) empty overview-field %s", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "Date:")) {
+						art->date = parsedate(ptr, (TIMEINFO *) 0);
+#ifdef DEBUG
+						if ((debug & DEBUG_NNTP) && art->date == (time_t) -1)
+							debug_print_file("NNTP", "OVER(%d) bogus overview-field %s %s", artnum, ofmt[count].name, ptr);
+#endif /* DEBUG */
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "Message-ID:")) {
+						if (*ptr)
+							art->msgid = my_strdup(ptr);
+						else {
+							art->msgid = NULL;
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) empty overview-field %s", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "References:")) {
+						if (*ptr)
+							art->refs = my_strdup(ptr);
+						else
+							art->refs = NULL;
+						continue;
+					}
+				}
+				/* metadata fiels */
+				if (ofmt[count].type == OVER_T_INT) {
+					if (!strcasecmp(ofmt[count].name, "Bytes:")) {
+						if (*ptr) {
+#ifdef DEBUG
+							if ((debug & DEBUG_NNTP) && !isdigit((unsigned char) *ptr))
+									debug_print_file("NNTP", "OVER(%d) overview field %d (%s) missmatch: %s", artnum, count, ofmt[count].name, ptr);
+#endif /* DEBUG */
+						}
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "Lines:")) {
+						if (*ptr) {
+							if (isdigit((unsigned char) *ptr))
+								art->line_count = atoi(ptr);
+							else {
+								art->line_count = 0;
+#ifdef DEBUG
+								if (debug & DEBUG_NNTP)
+									debug_print_file("NNTP", "OVER(%d) overview field %d (%s) missmatch: %s", artnum, count, ofmt[count].name, ptr);
+#endif /* DEBUG */
+							}
+						} else
+							art->line_count = 0;
+						continue;
+					}
+				}
+			} else { /* first 7 fields are in RFC 3977 order */
+				switch(count) {
+					case 1: /* Subject: */
+						if (*ptr)
+							art->subject = hash_str(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr))), FALSE));
+						else {
+							art->subject = hash_str("");
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) empty overview-field %s", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						break;
+
+					case 2:	/* From: */
+						if (*ptr) {
+							art->gnksa_code = parse_from(ptr, art_from_addr, art_full_name);
+							art->from = hash_str(buffer_to_ascii(art_from_addr));
+							if (*art_full_name)
+								art->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name))));
+						} else {
+							art->from = hash_str("");
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) empty overview-field %s", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						break;
+
+					case 3:	/* Date: */
+						art->date = parsedate(ptr, (TIMEINFO *) 0);
+#ifdef DEBUG
+						if ((debug & DEBUG_NNTP) && art->date == (time_t) -1)
+							debug_print_file("NNTP", "OVER(%d) bogus overview-field %s %s", artnum, ofmt[count].name, ptr);
+#endif /* DEBUG */
+						break;
+
+					case 4:	/* Message-ID: */
+						if (*ptr)
+							art->msgid = my_strdup(ptr);
+						else {
+							art->msgid = NULL;
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) empty overview-field %s", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						break;
+
+					case 5:	/* References: */
+						if (*ptr)
+							art->refs = my_strdup(ptr);
+						else
+							art->refs = NULL;
+						break;
+
+					case 6:	/* :bytes || Bytes: */
+						if (*ptr) {
+#ifdef DEBUG
+							if ((debug & DEBUG_NNTP) && !isdigit((unsigned char) *ptr))
+								debug_print_file("NNTP", "OVER(%d) overview field %d (%s) missmatch: %s", artnum, count, ofmt[count].name, ptr);
+#endif /* DEBUG */
+						}
+						break;
+
+					case 7:	/* :lines || Lines: */
+						if (*ptr) {
+							if (isdigit((unsigned char) *ptr))
+								art->line_count = atoi(ptr);
+							else {
+								art->line_count = 0;
+#ifdef DEBUG
+								if (debug & DEBUG_NNTP)
+									debug_print_file("NNTP", "OVER(%d) overview field %d (%s) missmatch: %s", artnum, count, ofmt[count].name, ptr);
+#endif /* DEBUG */
+							}
+						} else
+							art->line_count = 0;
+						break;
+
+					default:
+						break;
+				}
+			}
+
+			/* optional fields */
+			if (ofmt[count].type == OVER_T_FSTRING) {
+				if (!strcasecmp(ofmt[count].name, "Xref:")) {
+					if ((q = parse_header(ptr, "Xref", FALSE, FALSE)) != NULL)
+						art->xref = my_strdup(q);
+#ifdef DEBUG
+					else {
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "OVER(%d) bogus overview-field %s %s", artnum, ofmt[count].name, ptr);
+					}
+#endif /* DEBUG */
+				}
+				continue;
 			}
 		}
 
-#ifdef DEBUG
-		/* Complain if incorrect # of fields */
-		if (count < (xref_supported ? 8 : 7) || oerror) {
-			char errbuf[LEN];
-
-			error_message(_("%d Bad overview record (%d fields) '%s'"), oerror, count, BlankIfNull(ptr)); /* TODO move to lang.c */
-			snprintf(errbuf, sizeof(errbuf), "%d Bad overview record (%d fields)", oerror, count);
-			debug_nntp("read_overview", errbuf);
-		}
-		debug_print_header(art);
-		oerror = 0;
-#endif /* DEBUG */
-
 		/*
 		 * RFC says Message-ID is mandatory in newsgroups (but not in
 		 * mailgroups etc..) NB. a NULL Message-ID would abort if we ever do
@@ -1564,13 +1777,79 @@ read_overview(
 
 		top_art++;				/* Basically this statement commits the article */
 	}
-	free(group_msg);
 
+	free(group_msg);
 	TIN_FCLOSE(fp);
 
 	if (tin_errno)
 		return -1;
 
+#if defined(NNTP_ABLE) && defined(XHDR_XREF)
+	if (read_news_via_nntp && !read_saved_news && !xref_supported && nntp_caps.hdr_cmd) {
+		char cbuf[HEADER_LEN];
+		static t_bool found;
+		static t_bool first = TRUE;
+
+		if (first) {
+			found = TRUE;
+			/*
+			 * TODO: do once a start and cache full result
+			 *       if "LIST HEADERS RANGE" failed try "LIST HEADERS"?
+			 */
+			if (nntp_caps.type == CAPABILITIES && nntp_caps.list_headers) {
+				int i = new_nntp_command("LIST HEADERS RANGE", 215, cbuf, sizeof(cbuf));
+
+				found = FALSE;
+				switch (i) {
+					case 215:
+						while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
+#	ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "<<< %s", ptr);
+#	endif /* DEBUG */
+							if (!found && ((*ptr == ':' && *(ptr + 1) == '\0') || !strncasecmp(ptr, "Xref", 4)))
+								found = TRUE;
+						}
+						break;
+
+					default:
+						break;
+				}
+				first = FALSE;
+			}
+		}
+
+		if (found) {
+			snprintf(cbuf, sizeof(cbuf), "%s XREF %ld-%ld", nntp_caps.hdr_cmd, min, max);
+			group_msg = fmt_string("%s XREF loop", nntp_caps.hdr_cmd); /* TODO: find a better message, move to lang.c */
+			if ((fp = nntp_command(cbuf, OK_HEAD, NULL, 0)) != NULL) {
+				while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
+					artnum = atol(ptr);
+					if (artnum <= 0 || artnum < group->xmin || artnum > group->xmax)
+						continue;
+					art = &arts[top_art];
+					set_article(art);
+					if (!art->xref && !strstr(ptr, "(none)")) {
+						if ((q = strchr(ptr, ' ')) == NULL) /* skip article number */
+							continue;
+						ptr = q;
+						while (*ptr && isspace((int) *ptr))
+							ptr++;
+						q = strchr(ptr, '\n');
+						if (q)
+							*q = '\0';
+						art->xref = my_strdup(ptr);
+					}
+					/* we might loose accuracy here, but that shouldn't hurt */
+					if (artnum % MODULO_COUNT_NUM == 0)
+						show_progress(group_msg, artnum - min, max - min);
+				}
+			}
+			free(group_msg);
+		}
+	}
+#endif /* NNTP_ABLE && XHDR_XREF */
+
 	return expired;
 }
 
@@ -1624,7 +1903,7 @@ write_overview(
 	if ((fp = open_xover_fp(group, "w", 0L, 0L, FALSE)) == NULL)
 		return;
 
-	if (group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
+	if (group->attribute->sort_article_type != SORT_ARTICLES_BY_NOTHING)
 		SortBy(artnum_comp);
 
 	/*
@@ -1633,12 +1912,13 @@ write_overview(
 	fprintf(fp, "%s\n", group->name);
 
 	for_each_art(i) {
+		char *p;
+		char *q, *ref;
+
 		article = &arts[i];
 
 		if (article->thread != ART_EXPIRED && article->artnum >= group->xmin) {
-			char *p;
-			char *q = NULL, *ref = NULL;
-
+			q = ref = NULL;
 			/*
 			 * TODO: instead of tinrc.mm_local_charset we'd better use UTF-8
 			 *       here and in print_from() in the CHARSET_CONVERSION case.
@@ -1674,8 +1954,8 @@ write_overview(
 
 			fprintf(fp, "%ld\t%s\t%s\t%s\t%s\t%s\t%d\t%d",
 				article->artnum,
-				tinrc.post_8bit_header ? article->subject : p,
-				print_from(article),
+				group->attribute->post_8bit_header ? article->subject : p,
+				print_from(group, article),
 				print_date(article->date),
 				BlankIfNull(article->msgid),
 				BlankIfNull(ref),
@@ -1687,9 +1967,9 @@ write_overview(
 
 			fprintf(fp, "\n");
 			free(p);
-			if (q != ref) {
-				free(ref);
-				ref = q = NULL;
+			if (article->refs) {
+				FreeIfNeeded(ref);
+				q = NULL;
 			}
 		}
 	}
@@ -1762,8 +2042,8 @@ find_nov_file(
 			 */
 #ifndef NNTP_ONLY
 			if (!read_news_via_nntp) {
-				make_base_group_path(novrootdir, group->name, buf);
-				joinpath(nov_file, buf, novfilename);
+				make_base_group_path(novrootdir, group->name, buf, sizeof(buf));
+				joinpath(nov_file, sizeof(nov_file), buf, novfilename);
 				if (access(nov_file, R_OK) == 0) {
 					if (mode == R_OK)
 						return nov_file;		/* Use system wide overviews */
@@ -1817,7 +2097,7 @@ find_nov_file(
 			 * now that we know it is valid
 			 */
 			if (!once_only)
-				joinpath(local_newsgroups_file, index_newsdir, NEWSGROUPS_FILE);
+				joinpath(local_newsgroups_file, sizeof(local_newsgroups_file), index_newsdir, NEWSGROUPS_FILE);
 
 			dir = index_newsdir;
 			break;
@@ -1839,7 +2119,7 @@ find_nov_file(
 		char *ptr;
 
 		snprintf(buf, sizeof(buf), "%lu.%d", hash, i);
-		joinpath(nov_file, dir, buf);
+		joinpath(nov_file, sizeof(nov_file), dir, buf);
 
 		if ((fp = fopen(nov_file, "r")) == NULL)
 			break;
@@ -2165,7 +2445,7 @@ score_comp_base(
 	if (a == b) {
 		const struct t_article *s1 = &arts[*(const long *)p1];
 		const struct t_article *s2 = &arts[*(const long *)p2];
-		t_compfunc comp_func = eval_sort_arts_func(CURR_GROUP.attribute->sort_art_type);
+		t_compfunc comp_func = eval_sort_arts_func(CURR_GROUP.attribute->sort_article_type);
 
 		if (comp_func)
 			return (*comp_func)(s1, s2);
@@ -2324,6 +2604,7 @@ print_date(
 
 static char *
 print_from(
+	struct t_group *group,
 	struct t_article *article)
 {
 	char *p;
@@ -2335,9 +2616,9 @@ print_from(
 		p = rfc1522_encode(article->name, tinrc.mm_local_charset, FALSE);
 		unfold_header(p);
 		if (strpbrk(article->name, "\".:;<>@[]()\\") != NULL && article->name[0] != '"' && article->name[strlen(article->name)] != '"')
-			snprintf(from, sizeof(from), "\"%s\" <%s>", tinrc.post_8bit_header ? article->name : p, article->from);
+			snprintf(from, sizeof(from), "\"%s\" <%s>", group->attribute->post_8bit_header ? article->name : p, article->from);
 		else
-			snprintf(from, sizeof(from), "%s <%s>", tinrc.post_8bit_header ? article->name : p, article->from);
+			snprintf(from, sizeof(from), "%s <%s>", group->attribute->post_8bit_header ? article->name : p, article->from);
 
 		free(p);
 	}
@@ -2377,7 +2658,7 @@ open_xover_fp(
 				return fp;
 
 			if (*mode != 'r')
-				error_message(_(txt_cannot_open), nov_file);
+				error_message(2, _(txt_cannot_open), nov_file);
 		}
 	}
 	return NULL;
