@@ -3,10 +3,10 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2006-06-28
+ *  Updated   : 2009-01-14
  *  Notes     : Configuration file routines
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -73,7 +73,11 @@ read_config_file(
 {
 	FILE *fp;
 	char buf[LEN], tmp[LEN];
-	int upgrade = RC_CHECK;
+	enum rc_state upgrade = RC_CHECK;
+#ifdef CHARSET_CONVERSION
+	int i;
+	t_bool is_7bit;
+#endif /* CHARSET_CONVERSION */
 
 	if ((fp = fopen(file, "r")) == NULL)
 		return FALSE;
@@ -84,8 +88,10 @@ read_config_file(
 		wait_message(0, _(txt_reading_config_file), (global_file) ? _(txt_global) : "");
 
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
-		if (buf[0] == '#' || buf[0] == '\n') {
-			if (upgrade == RC_CHECK && !global_file) {
+		if (buf[0] == '\n')
+			continue;
+		if (buf[0] == '#') {
+			if (upgrade == RC_CHECK && !global_file && match_string(buf, "# tin configuration file V", NULL, 0)) {
 				upgrade = check_upgrade(buf, "# tin configuration file V", TINRC_VERSION);
 				if (upgrade != RC_IGNORE)
 					upgrade_prompt_quit(upgrade, CONFIG_FILE);
@@ -154,10 +160,7 @@ read_config_file(
 			if (match_boolean(buf, "ask_for_metamail=", &tinrc.ask_for_metamail))
 				break;
 
-			if (match_boolean(buf, "auto_bcc=", &tinrc.auto_bcc))
-				break;
-
-			if (match_boolean(buf, "auto_cc=", &tinrc.auto_cc))
+			if (match_integer(buf, "auto_cc_bcc=", &tinrc.auto_cc_bcc, AUTO_CC_BCC))
 				break;
 
 			if (match_boolean(buf, "auto_list_thread=", &tinrc.auto_list_thread))
@@ -245,6 +248,9 @@ read_config_file(
 			if (match_color(buf, "col_urls=", &tinrc.col_urls, MAX_COLOR))
 				break;
 
+			if (match_color(buf, "col_verbatim=", &tinrc.col_verbatim, MAX_COLOR))
+				break;
+
 			if (match_color(buf, "col_markstar=", &tinrc.col_markstar, MAX_COLOR))
 				break;
 
@@ -266,31 +272,6 @@ read_config_file(
 			if (match_string(buf, "date_format=", tinrc.date_format, sizeof(tinrc.date_format)))
 				break;
 
-			if (match_string(buf, "default_editor_format=", tinrc.editor_format, sizeof(tinrc.editor_format)))
-				break;
-
-			if (match_string(buf, "default_mailer_format=", tinrc.mailer_format, sizeof(tinrc.mailer_format)))
-				break;
-
-			if (match_string(buf, "default_savedir=", tinrc.savedir, sizeof(tinrc.savedir))) {
-				if (tinrc.savedir[0] == '.' && strlen(tinrc.savedir) == 1) {
-					get_cwd(buf);
-					my_strncpy(tinrc.savedir, buf, sizeof(tinrc.savedir) - 1);
-				}
-				break;
-			}
-
-			if (match_string(buf, "default_maildir=", tinrc.maildir, sizeof(tinrc.maildir)))
-				break;
-
-#ifndef DISABLE_PRINTING
-			if (match_string(buf, "default_printer=", tinrc.printer, sizeof(tinrc.printer)))
-				break;
-#endif /* !DISABLE_PRINTING */
-
-			if (match_string(buf, "default_sigfile=", tinrc.sigfile, sizeof(tinrc.sigfile)))
-				break;
-
 			if (match_integer(buf, "default_filter_days=", &tinrc.filter_days, 0)) {
 				if (tinrc.filter_days <= 0)
 					tinrc.filter_days = 1;
@@ -396,6 +377,13 @@ read_config_file(
 
 			break;
 
+		case 'e':
+			if (match_string(buf, "editor_format=", tinrc.editor_format, sizeof(tinrc.editor_format)))
+				break;
+
+			break;
+
+
 		case 'f':
 			if (match_boolean(buf, "force_screen_redraw=", &tinrc.force_screen_redraw))
 				break;
@@ -448,14 +436,17 @@ read_config_file(
 			break;
 
 		case 'm':
+			if (match_string(buf, "maildir=", tinrc.maildir, sizeof(tinrc.maildir)))
+				break;
+
+			if (match_string(buf, "mailer_format=", tinrc.mailer_format, sizeof(tinrc.mailer_format)))
+				break;
+
 			if (match_list(buf, "mail_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.mail_mime_encoding))
 				break;
 
-			if (match_boolean(buf, "mail_8bit_header=", &tinrc.mail_8bit_header)) {
-				if (strcasecmp(txt_mime_encodings[tinrc.mail_mime_encoding], txt_8bit))
-					tinrc.mail_8bit_header = FALSE;
+			if (match_boolean(buf, "mail_8bit_header=", &tinrc.mail_8bit_header))
 				break;
-			}
 
 #ifndef CHARSET_CONVERSION
 			if (match_string(buf, "mm_charset=", tinrc.mm_charset, sizeof(tinrc.mm_charset)))
@@ -465,6 +456,10 @@ read_config_file(
 				break;
 			if (match_list(buf, "mm_network_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset))
 				break;
+#	ifdef NO_LOCALE
+			if (match_string(buf, "mm_local_charset=", tinrc.mm_local_charset, sizeof(tinrc.mm_local_charset)))
+				break;
+#	endif /* NO_LOCALE */
 #endif /* !CHARSET_CONVERSION */
 
 			if (match_boolean(buf, "mark_ignore_tags=", &tinrc.mark_ignore_tags))
@@ -506,22 +501,12 @@ read_config_file(
 			}
 
 			/* pick which news headers to display */
-			if (match_string(buf, "news_headers_to_display=", tinrc.news_headers_to_display, sizeof(tinrc.news_headers_to_display))) {
-				if (news_headers_to_display_array)
-					FreeIfNeeded(*news_headers_to_display_array);
-				FreeIfNeeded(news_headers_to_display_array);
-				news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
+			if (match_string(buf, "news_headers_to_display=", tinrc.news_headers_to_display, sizeof(tinrc.news_headers_to_display)))
 				break;
-			}
 
 			/* pick which news headers to NOT display */
-			if (match_string(buf, "news_headers_to_not_display=", tinrc.news_headers_to_not_display, sizeof(tinrc.news_headers_to_not_display))) {
-				if (news_headers_to_not_display_array)
-					FreeIfNeeded(*news_headers_to_not_display_array);
-				FreeIfNeeded(news_headers_to_not_display_array);
-				news_headers_to_not_display_array = ulBuildArgv(tinrc.news_headers_to_not_display, &num_headers_to_not_display);
+			if (match_string(buf, "news_headers_to_not_display=", tinrc.news_headers_to_not_display, sizeof(tinrc.news_headers_to_not_display)))
 				break;
-			}
 
 			if (match_string(buf, "news_quote_format=", tinrc.news_quote_format, sizeof(tinrc.news_quote_format)))
 				break;
@@ -544,14 +529,13 @@ read_config_file(
 			if (match_list(buf, "post_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.post_mime_encoding))
 				break;
 
-			if (match_boolean(buf, "post_8bit_header=", &tinrc.post_8bit_header)) {
-				/* if post_mime_encoding != 8bit, post_8bit_header is disabled */
-				if (strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_8bit))
-					tinrc.post_8bit_header = FALSE;
+			if (match_boolean(buf, "post_8bit_header=", &tinrc.post_8bit_header))
 				break;
-			}
 
 #ifndef DISABLE_PRINTING
+			if (match_string(buf, "printer=", tinrc.printer, sizeof(tinrc.printer)))
+				break;
+
 			if (match_boolean(buf, "print_header=", &tinrc.print_header))
 				break;
 #endif /* !DISABLE_PRINTING */
@@ -559,7 +543,7 @@ read_config_file(
 			if (match_boolean(buf, "pos_first_unread=", &tinrc.pos_first_unread))
 				break;
 
-			if (match_integer(buf, "post_process_type=", &tinrc.post_process, POST_PROC_YES))
+			if (match_integer(buf, "post_process_type=", &tinrc.post_process_type, POST_PROC_YES))
 				break;
 
 			if (match_boolean(buf, "post_process_view=", &tinrc.post_process_view))
@@ -613,6 +597,14 @@ read_config_file(
 			break;
 
 		case 's':
+			if (match_string(buf, "savedir=", tinrc.savedir, sizeof(tinrc.savedir))) {
+				if (tinrc.savedir[0] == '.' && strlen(tinrc.savedir) == 1) {
+					get_cwd(buf);
+					my_strncpy(tinrc.savedir, buf, sizeof(tinrc.savedir) - 1);
+				}
+				break;
+			}
+
 			if (match_integer(buf, "score_limit_kill=", &tinrc.score_limit_kill, 0))
 				break;
 
@@ -637,7 +629,7 @@ read_config_file(
 				break;
 			}
 
-			if (match_boolean(buf, "show_only_unread=", &tinrc.show_only_unread_arts))
+			if (match_boolean(buf, "show_only_unread_arts=", &tinrc.show_only_unread_arts))
 				break;
 
 			if (match_boolean(buf, "show_only_unread_groups=", &tinrc.show_only_unread_groups))
@@ -646,6 +638,9 @@ read_config_file(
 			if (match_boolean(buf, "sigdashes=", &tinrc.sigdashes))
 				break;
 
+			if (match_string(buf, "sigfile=", tinrc.sigfile, sizeof(tinrc.sigfile)))
+				break;
+
 			if (match_boolean(buf, "signature_repost=", &tinrc.signature_repost))
 				break;
 
@@ -718,6 +713,9 @@ read_config_file(
 				break;
 #endif /* HAVE_ICONV_OPEN_TRANSLIT && CHARSET_CONVERSION */
 
+			if (match_integer(buf, "trim_article_body=", &tinrc.trim_article_body, NUM_TRIM_ARTICLE_BODY))
+				break;
+
 			break;
 
 		case 'u':
@@ -762,6 +760,9 @@ read_config_file(
 			if (match_string(buf, "verbatim_end_regex=", tinrc.verbatim_end_regex, sizeof(tinrc.verbatim_end_regex)))
 				break;
 
+			if (match_boolean(buf, "verbatim_handling=", &tinrc.verbatim_handling))
+				break;
+
 			break;
 
 		case 'w':
@@ -804,28 +805,47 @@ read_config_file(
 	if (!(tinrc.draw_arrow || tinrc.inverse_okay))
 		tinrc.draw_arrow = TRUE;
 
+#ifdef CHARSET_CONVERSION
 	/*
-	 * determine local charset, or in case of NO_LOCALE use
-	 * mm{_network}_charset
+	 * check if we have a 7bit charset but a !7bit encoding
+	 * or a 8bit charset but a !8bit encoding, update encoding if needed
 	 */
-	{
-#ifndef NO_LOCALE
-		const char *p;
-
-		if ((p = tin_nl_langinfo(CODESET)) != NULL) {
-			if (!strcmp(p, "ANSI_X3.4-1968"))
-				strcpy(tinrc.mm_local_charset, "US-ASCII");
-			else
-				strcpy(tinrc.mm_local_charset, p);
-		} else
-#endif /* !NO_LOCALE */
-			if (!*tinrc.mm_local_charset)
-#ifndef CHARSET_CONVERSION
-				strcpy(tinrc.mm_local_charset, tinrc.mm_charset);
-#else
-				strcpy(tinrc.mm_local_charset, txt_mime_charsets[tinrc.mm_network_charset]);
-#endif /* !CHARSET_CONVERSION */
+	is_7bit = FALSE;
+	for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
+		if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
+			is_7bit = TRUE;
+			break;
+		}
 	}
+	if (is_7bit) {
+		if (tinrc.mail_mime_encoding != MIME_ENCODING_7BIT)
+			tinrc.mail_mime_encoding = MIME_ENCODING_7BIT;
+		if (tinrc.post_mime_encoding != MIME_ENCODING_7BIT)
+			tinrc.post_mime_encoding = MIME_ENCODING_7BIT;
+	} else {
+		if (tinrc.mail_mime_encoding == MIME_ENCODING_7BIT)
+			tinrc.mail_mime_encoding = MIME_ENCODING_QP;
+		if (tinrc.post_mime_encoding == MIME_ENCODING_7BIT)
+			tinrc.post_mime_encoding = MIME_ENCODING_8BIT;
+	}
+#endif /* CHARSET_CONVERSION */
+
+	/* do not use 8 bit headers if mime encoding is not 8bit */
+	if (tinrc.mail_mime_encoding != MIME_ENCODING_8BIT)
+		tinrc.mail_8bit_header = FALSE;
+	if (tinrc.post_mime_encoding != MIME_ENCODING_8BIT)
+		tinrc.post_8bit_header = FALSE;
+
+	/* set defaults if blank */
+	if (!*tinrc.editor_format)
+		STRCPY(tinrc.editor_format, TIN_EDITOR_FMT_ON);
+	if (!*tinrc.date_format)
+		STRCPY(tinrc.date_format, DEFAULT_DATE_FORMAT);
+
+	/* determine local charset */
+#if defined(NO_LOCALE) && !defined(CHARSET_CONVERSION)
+	strcpy(tinrc.mm_local_charset, tinrc.mm_charset);
+#endif /* NO_LOCALE && !CHARSET_CONVERSION */
 	return TRUE;
 }
 
@@ -848,7 +868,7 @@ write_config_file(
 	file_tmp = get_tmpfilename(file);
 
 	if ((fp = fopen(file_tmp, "w")) == NULL) {
-		error_message(_(txt_filesystem_full_backup), CONFIG_FILE);
+		error_message(2, _(txt_filesystem_full_backup), CONFIG_FILE);
 		free(file_tmp);
 		return;
 	}
@@ -856,13 +876,10 @@ write_config_file(
 	if (!cmd_line)
 		wait_message(0, _(txt_saving));
 
-	if (!*tinrc.editor_format)
-		strcpy(tinrc.editor_format, TIN_EDITOR_FMT_ON);
-
 	fprintf(fp, txt_tinrc_header, PRODUCT, TINRC_VERSION, tin_progname, VERSION, RELEASEDATE, RELEASENAME);
 
 	fprintf(fp, _(txt_savedir.tinrc));
-	fprintf(fp, "default_savedir=%s\n\n", tinrc.savedir);
+	fprintf(fp, "savedir=%s\n\n", tinrc.savedir);
 
 	fprintf(fp, _(txt_auto_save.tinrc));
 	fprintf(fp, "auto_save=%s\n\n", print_boolean(tinrc.auto_save));
@@ -870,8 +887,8 @@ write_config_file(
 	fprintf(fp, _(txt_mark_saved_read.tinrc));
 	fprintf(fp, "mark_saved_read=%s\n\n", print_boolean(tinrc.mark_saved_read));
 
-	fprintf(fp, _(txt_post_process.tinrc));
-	fprintf(fp, "post_process_type=%d\n\n", tinrc.post_process);
+	fprintf(fp, _(txt_post_process_type.tinrc));
+	fprintf(fp, "post_process_type=%d\n\n", tinrc.post_process_type);
 
 	fprintf(fp, _(txt_post_process_view.tinrc));
 	fprintf(fp, "post_process_view=%s\n\n", print_boolean(tinrc.post_process_view));
@@ -901,7 +918,7 @@ write_config_file(
 	fprintf(fp, "pos_first_unread=%s\n\n", print_boolean(tinrc.pos_first_unread));
 
 	fprintf(fp, _(txt_show_only_unread_arts.tinrc));
-	fprintf(fp, "show_only_unread=%s\n\n", print_boolean(tinrc.show_only_unread_arts));
+	fprintf(fp, "show_only_unread_arts=%s\n\n", print_boolean(tinrc.show_only_unread_arts));
 
 	fprintf(fp, _(txt_show_only_unread_groups.tinrc));
 	fprintf(fp, "show_only_unread_groups=%s\n\n", print_boolean(tinrc.show_only_unread_groups));
@@ -935,16 +952,10 @@ write_config_file(
 	fprintf(fp, "show_author=%d\n\n", tinrc.show_author);
 
 	fprintf(fp, _(txt_news_headers_to_display.tinrc));
-	fprintf(fp, "news_headers_to_display=");
-	for (i = 0; i < num_headers_to_display; i++)
-		fprintf(fp, "%s ", news_headers_to_display_array[i]);
-	fprintf(fp, "\n\n");
+	fprintf(fp, "news_headers_to_display=%s\n\n", tinrc.news_headers_to_display);
 
 	fprintf(fp, _(txt_news_headers_to_not_display.tinrc));
-	fprintf(fp, "news_headers_to_not_display=");
-	for (i = 0; i < num_headers_to_not_display; i++)
-		fprintf(fp, "%s ", news_headers_to_not_display_array[i]);
-	fprintf(fp, "\n\n");
+	fprintf(fp, "news_headers_to_not_display=%s\n\n", tinrc.news_headers_to_not_display);
 
 	fprintf(fp, _(txt_tinrc_info_in_last_line));
 	fprintf(fp, "info_in_last_line=%s\n\n", print_boolean(tinrc.info_in_last_line));
@@ -956,7 +967,7 @@ write_config_file(
 	fprintf(fp, "sort_threads_type=%d\n\n", tinrc.sort_threads_type);
 
 	fprintf(fp, _(txt_maildir.tinrc));
-	fprintf(fp, "default_maildir=%s\n\n", tinrc.maildir);
+	fprintf(fp, "maildir=%s\n\n", tinrc.maildir);
 
 	fprintf(fp, _(txt_mailbox_format.tinrc));
 	fprintf(fp, "mailbox_format=%s\n\n", txt_mailbox_formats[tinrc.mailbox_format]);
@@ -966,7 +977,7 @@ write_config_file(
 	fprintf(fp, "print_header=%s\n\n", print_boolean(tinrc.print_header));
 
 	fprintf(fp, _(txt_printer.tinrc));
-	fprintf(fp, "default_printer=%s\n\n", tinrc.printer);
+	fprintf(fp, "printer=%s\n\n", tinrc.printer);
 #endif /* !DISABLE_PRINTING */
 
 	fprintf(fp, _(txt_batch_save.tinrc));
@@ -976,10 +987,10 @@ write_config_file(
 	fprintf(fp, "start_editor_offset=%s\n\n", print_boolean(tinrc.start_editor_offset));
 
 	fprintf(fp, _(txt_editor_format.tinrc));
-	fprintf(fp, "default_editor_format=%s\n\n", tinrc.editor_format);
+	fprintf(fp, "editor_format=%s\n\n", tinrc.editor_format);
 
 	fprintf(fp, _(txt_mailer_format.tinrc));
-	fprintf(fp, "default_mailer_format=%s\n\n", tinrc.mailer_format);
+	fprintf(fp, "mailer_format=%s\n\n", tinrc.mailer_format);
 
 	fprintf(fp, _(txt_interactive_mailer.tinrc));
 	fprintf(fp, "interactive_mailer=%d\n\n", tinrc.interactive_mailer);
@@ -1003,7 +1014,7 @@ write_config_file(
 	fprintf(fp, "add_posted_to_filter=%s\n\n", print_boolean(tinrc.add_posted_to_filter));
 
 	fprintf(fp, _(txt_sigfile.tinrc));
-	fprintf(fp, "default_sigfile=%s\n\n", tinrc.sigfile);
+	fprintf(fp, "sigfile=%s\n\n", tinrc.sigfile);
 
 	fprintf(fp, _(txt_sigdashes.tinrc));
 	fprintf(fp, "sigdashes=%s\n\n", print_boolean(tinrc.sigdashes));
@@ -1071,11 +1082,8 @@ write_config_file(
 	fprintf(fp, "mail_quote_format=%s\n", tinrc.mail_quote_format);
 	fprintf(fp, "xpost_quote_format=%s\n\n", tinrc.xpost_quote_format);
 
-	fprintf(fp, _(txt_auto_cc.tinrc));
-	fprintf(fp, "auto_cc=%s\n\n", print_boolean(tinrc.auto_cc));
-
-	fprintf(fp, _(txt_auto_bcc.tinrc));
-	fprintf(fp, "auto_bcc=%s\n\n", print_boolean(tinrc.auto_bcc));
+	fprintf(fp, _(txt_auto_cc_bcc.tinrc));
+	fprintf(fp, "auto_cc_bcc=%d\n\n", tinrc.auto_cc_bcc);
 
 	fprintf(fp, _(txt_art_marked_deleted.tinrc));
 	fprintf(fp, "art_marked_deleted=%c\n\n", SPACE_TO_DASH(tinrc.art_marked_deleted));
@@ -1214,6 +1222,9 @@ write_config_file(
 
 	fprintf(fp, _(txt_col_urls.tinrc));
 	fprintf(fp, "col_urls=%d\n\n", tinrc.col_urls);
+
+	fprintf(fp, _(txt_col_verbatim.tinrc));
+	fprintf(fp, "col_verbatim=%d\n\n", tinrc.col_verbatim);
 #endif /* HAVE_COLOR */
 
 	fprintf(fp, _(txt_url_highlight.tinrc));
@@ -1256,6 +1267,9 @@ write_config_file(
 	fprintf(fp, _(txt_wrap_column.tinrc));
 	fprintf(fp, "wrap_column=%d\n\n", tinrc.wrap_column);
 
+	fprintf(fp, _(txt_trim_article_body.tinrc));
+	fprintf(fp, "trim_article_body=%d\n\n", tinrc.trim_article_body);
+
 #ifndef CHARSET_CONVERSION
 	fprintf(fp, _(txt_mm_charset.tinrc));
 	fprintf(fp, "mm_charset=%s\n\n", tinrc.mm_charset);
@@ -1263,6 +1277,10 @@ write_config_file(
 	fprintf(fp, _(txt_mm_network_charset.tinrc));
 	fprintf(fp, "mm_network_charset=%s\n\n", txt_mime_charsets[tinrc.mm_network_charset]);
 
+#	ifdef NO_LOCALE
+	fprintf(fp, _(txt_mm_local_charset.tinrc));
+	fprintf(fp, "mm_local_charset=%s\n\n", tinrc.mm_local_charset);
+#	endif /* NO_LOCALE */
 #	ifdef HAVE_ICONV_OPEN_TRANSLIT
 	fprintf(fp, _(txt_translit.tinrc));
 	fprintf(fp, "translit=%s\n\n", print_boolean(tinrc.translit));
@@ -1293,6 +1311,9 @@ write_config_file(
 	fprintf(fp, _(txt_alternative_handling.tinrc));
 	fprintf(fp, "alternative_handling=%s\n\n", print_boolean(tinrc.alternative_handling));
 
+	fprintf(fp, _(txt_verbatim_handling.tinrc));
+	fprintf(fp, "verbatim_handling=%s\n\n", print_boolean(tinrc.verbatim_handling));
+
 	fprintf(fp, _(txt_strip_newsrc.tinrc));
 	fprintf(fp, "strip_newsrc=%s\n\n", print_boolean(tinrc.strip_newsrc));
 
@@ -1374,7 +1395,7 @@ write_config_file(
 	fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR)); /* rename_file() preserves mode */
 
 	if (ferror(fp) || fclose(fp))
-		error_message(_(txt_filesystem_full), CONFIG_FILE);
+		error_message(2, _(txt_filesystem_full), CONFIG_FILE);
 	else
 		rename_file(file_tmp, file);
 
@@ -1679,6 +1700,8 @@ rc_update(
 {
 	char buf[1024];
 	const char *env;
+	t_bool auto_bcc = FALSE;
+	t_bool auto_cc = FALSE;
 	t_bool confirm_to_quit = FALSE;
 	t_bool confirm_action = FALSE;
 	t_bool compress_quotes = FALSE;
@@ -1694,8 +1717,6 @@ rc_update(
 	t_bool show_score = FALSE;
 	t_bool space_goto_next_unread = FALSE;
 	t_bool tab_goto_next_unread = FALSE;
-	t_bool thread_articles = FALSE;
-	t_bool thread_perc = FALSE;
 	t_bool use_builtin_inews = FALSE;
 	t_bool use_getart_limit = FALSE;
 	t_bool use_mailreader_i = FALSE;
@@ -1710,6 +1731,14 @@ rc_update(
 			continue;
 
 		switch (tolower((unsigned char) buf[0])) {
+			case 'a':
+				if (match_boolean(buf, "auto_bcc=", &auto_bcc))
+					break;
+
+				if (match_boolean(buf, "auto_cc=", &auto_cc))
+					break;
+				break;
+
 			case 'c':
 				if (match_boolean(buf, "confirm_action=", &confirm_action))
 					break;
@@ -1720,9 +1749,34 @@ rc_update(
 				break;
 
 			case 'd':
+				/* simple rename */
+				if (match_string(buf, "default_editor_format=", tinrc.editor_format, sizeof(tinrc.editor_format)))
+					break;
+				/* simple rename */
+				if (match_string(buf, "default_maildir=", tinrc.maildir, sizeof(tinrc.maildir)))
+					break;
+				/* simple rename */
+				if (match_string(buf, "default_mailer_format=", tinrc.mailer_format, sizeof(tinrc.mailer_format)))
+					break;
+				/* simple rename */
+#ifndef DISABLE_PRINTING
+				if (match_string(buf, "default_printer=", tinrc.printer, sizeof(tinrc.printer)))
+					break;
+#endif /* !DISABLE_PRINTING */
 				/* simple rename */
 				if (match_string(buf, "default_regex_pattern=", tinrc.default_pattern, sizeof(tinrc.default_pattern)))
 					break;
+				/* simple rename */
+				if (match_string(buf, "default_savedir=", tinrc.savedir, sizeof(tinrc.savedir))) {
+					if (tinrc.savedir[0] == '.' && strlen(tinrc.savedir) == 1) {
+						get_cwd(buf);
+						my_strncpy(tinrc.savedir, buf, sizeof(tinrc.savedir) - 1);
+					}
+					break;
+				}
+				/* simple rename */
+				if (match_string(buf, "default_sigfile=", tinrc.sigfile, sizeof(tinrc.sigfile)))
+					break;
 				break;
 
 			case 'h':
@@ -1760,6 +1814,9 @@ rc_update(
 					break;
 				if (match_boolean(buf, "show_lines=", &show_lines))
 					break;
+				/* simple rename */
+				if (match_boolean(buf, "show_only_unread=", &tinrc.show_only_unread_arts))
+					break;
 				if (match_boolean(buf, "show_score=", &show_score))
 					break;
 				break;
@@ -1769,8 +1826,6 @@ rc_update(
 					set_goto_next_unread = TRUE;
 					break;
 				}
-				if (match_boolean(buf, "thread_articles=", &thread_articles))
-					break;
 				break;
 
 			case 'u':
@@ -1790,6 +1845,7 @@ rc_update(
 	}
 
 	/* update the values */
+	tinrc.auto_cc_bcc = (auto_cc ? 1 : 0) + (auto_bcc ? 2 : 0);
 	tinrc.confirm_choice = (confirm_action ? 1 : 0) + (confirm_to_quit ? 3 : 0);
 
 	if (!use_getart_limit)
@@ -1818,12 +1874,6 @@ rc_update(
 	if (show_last_line_prev_page)
 		tinrc.scroll_lines = -1;
 
-	if (thread_articles)
-		tinrc.thread_articles = THREAD_BOTH;
-
-	if (thread_perc)
-		tinrc.thread_perc = THREAD_PERC_DEFAULT;
-
 	if (use_builtin_inews)
 		strncpy(tinrc.inews_prog, INTERNAL_CMD, sizeof(tinrc.inews_prog) - 1);
 
@@ -1850,7 +1900,7 @@ read_server_config(
 	char file[PATH_LEN];
 	char newnews_info[LEN];
 	char serverdir[PATH_LEN];
-	int upgrade = RC_CHECK;
+	enum rc_state upgrade = RC_CHECK;
 
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp && !read_saved_news && nntp_tcp_port != IPPORT_NNTP)
@@ -1860,9 +1910,9 @@ read_server_config(
 	{
 		STRCPY(file, quote_space_to_dash(nntp_server));
 	}
-	joinpath(serverdir, rcdir, file);
-	joinpath(file, serverdir, SERVERCONFIG_FILE);
-	joinpath(local_newsgroups_file, serverdir, NEWSGROUPS_FILE);
+	joinpath(serverdir, sizeof(serverdir), rcdir, file);
+	joinpath(file, sizeof(file), serverdir, SERVERCONFIG_FILE);
+	joinpath(local_newsgroups_file, sizeof(local_newsgroups_file), serverdir, NEWSGROUPS_FILE);
 	if ((fp = fopen(file, "r")) == NULL)
 		return;
 	while (NULL != (line = tin_fgets(fp, FALSE))) {
@@ -1918,8 +1968,8 @@ write_server_config(
 	{
 		STRCPY(file, nntp_server);
 	}
-	joinpath(serverdir, rcdir, file);
-	joinpath(file, serverdir, SERVERCONFIG_FILE);
+	joinpath(serverdir, sizeof(serverdir), rcdir, file);
+	joinpath(file, sizeof(file), serverdir, SERVERCONFIG_FILE);
 
 	if ((no_write || post_article_and_exit || post_postponed_and_exit) && file_size(file) != -1L)
 		return;
@@ -1934,7 +1984,7 @@ write_server_config(
 	file_tmp = get_tmpfilename(file);
 
 	if ((fp = fopen(file_tmp, "w")) == NULL) {
-		error_message(_(txt_filesystem_full_backup), SERVERCONFIG_FILE);
+		error_message(2, _(txt_filesystem_full_backup), SERVERCONFIG_FILE);
 		free(file_tmp);
 		return;
 	}
@@ -1949,7 +1999,7 @@ write_server_config(
 	fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR)); /* rename_file() preserves mode */
 
 	if (ferror(fp) || fclose(fp))
-		error_message(_(txt_filesystem_full), SERVERCONFIG_FILE);
+		error_message(2, _(txt_filesystem_full), SERVERCONFIG_FILE);
 	else
 		rename_file(file_tmp, file);
 
