@@ -3,10 +3,10 @@
  *  Module    : debug.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2005-05-04
+ *  Updated   : 2009-01-20
  *  Notes     : debug routines
  *
- * Copyright (c) 1991-2007 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,14 +39,11 @@
 #	include "tin.h"
 #endif /* !TIN_H */
 
-#if defined(DEBUG) || defined(DEBUG_NEWSRC)
+#ifdef DEBUG
 #	ifndef NEWSRC_H
 #		include "newsrc.h"
 #	endif /* !NEWSRC_H */
-#	ifndef TCURSES_H
-#		include "tcurses.h"
-#	endif /* !TCURSES_H */
-#endif /* DEBUG || DEBUG_NEWSRC */
+#endif /* DEBUG */
 
 int debug;
 
@@ -56,6 +53,7 @@ int debug;
  */
 static void debug_print_attributes(struct t_attribute *attr, FILE *fp);
 static void debug_print_filter(FILE *fp, int num, struct t_filter *the_filter);
+static void debug_print_newsrc(struct t_newsrc *NewSrc, FILE *fp);
 
 
 /*
@@ -68,58 +66,26 @@ debug_delete_files(
 	char file[PATH_LEN];
 
 	if (debug) {
-		joinpath(file, TMPDIR, "NNTP");
+		joinpath(file, sizeof(file), TMPDIR, "NNTP");
 		unlink(file);
-		joinpath(file, TMPDIR, "ARTS");
+		joinpath(file, sizeof(file), TMPDIR, "ARTS");
 		unlink(file);
-		joinpath(file, TMPDIR, "SAVE_COMP");
+		joinpath(file, sizeof(file), TMPDIR, "ACTIVE");
 		unlink(file);
-		joinpath(file, TMPDIR, "BASE");
+		joinpath(file, sizeof(file), TMPDIR, "BITMAP");
 		unlink(file);
-		joinpath(file, TMPDIR, "ACTIVE");
+		joinpath(file, sizeof(file), TMPDIR, "MALLOC");
 		unlink(file);
-		joinpath(file, TMPDIR, "BITMAP");
+		joinpath(file, sizeof(file), TMPDIR, "FILTER");
 		unlink(file);
-		joinpath(file, TMPDIR, "MALLOC");
+		joinpath(file, sizeof(file), TMPDIR, "ATTRIBUTES");
 		unlink(file);
-		joinpath(file, TMPDIR, "FILTER");
+		joinpath(file, sizeof(file), TMPDIR, "SCOPES");
 		unlink(file);
 	}
 }
 
 
-/*
- * nntp specific debug routines
- */
-void
-debug_nntp(
-	const char *func,
-	const char *fmt,
-	...)
-{
-	FILE *fp;
-	char *buf;
-	char file[PATH_LEN];
-	va_list ap;
-
-	if (!debug)
-		return;
-
-	va_start(ap, fmt);
-	buf = fmt_message(fmt, ap);
-
-	joinpath(file, TMPDIR, "NNTP");
-
-	if ((fp = fopen(file, "a+")) != NULL) {
-		fprintf(fp,"%s: %s\n", func, buf);
-		fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
-		fclose(fp);
-	}
-	free(buf);
-	va_end(ap);
-}
-
-
 /*
  * tin specific debug routines
  */
@@ -129,7 +95,7 @@ debug_print_arts(
 {
 	int i;
 
-	if (debug != 2)
+	if (!(debug & DEBUG_FILTER))
 		return;
 
 	for_each_art(i)
@@ -144,10 +110,10 @@ debug_print_header(
 	FILE *fp;
 	char file[PATH_LEN];
 
-	if (debug != 2)
+	if (!(debug & DEBUG_FILTER))
 		return;
 
-	joinpath(file, TMPDIR, "ARTS");
+	joinpath(file, sizeof(file), TMPDIR, "ARTS");
 
 	if ((fp = fopen(file, "a+")) != NULL) {
 		fprintf(fp,"art=[%5ld] tag=[%s] kill=[%s] selected=[%s]\n", s->artnum,
@@ -182,15 +148,16 @@ debug_print_active(
 {
 	FILE *fp;
 	char file[PATH_LEN];
-	int i;
-	struct t_group *group;
 
-	if (debug != 2)
+	if (!(debug & DEBUG_MISC))
 		return;
 
-	joinpath(file, TMPDIR, "ACTIVE");
+	joinpath(file, sizeof(file), TMPDIR, "ACTIVE");
 
 	if ((fp = fopen(file, "w")) != NULL) {
+		int i;
+		struct t_group *group;
+
 		for_each_group(i) {
 			group = &active[i];
 			fprintf(fp, "[%4d]=[%s] type=[%s] spooldir=[%s]\n",
@@ -199,12 +166,12 @@ debug_print_active(
 				group->spooldir);
 			fprintf(fp, "count=[%4ld] max=[%4ld] min=[%4ld] mod=[%c]\n",
 				group->count, group->xmax, group->xmin, group->moderated);
-			fprintf(fp, " nxt=[%4d] hash=[%ld]  description=[%s]\n", group->next,
+			fprintf(fp, " nxt=[%4d] hash=[%lu]  description=[%s]\n", group->next,
 				hash_groupname(group->name), BlankIfNull(group->description));
-#	ifdef DEBUG_NEWSRC
-			debug_print_newsrc(&group->newsrc, fp);
-#	endif /* DEBUG_NEWSRC */
-			debug_print_attributes(group->attribute, fp);
+			if (debug & DEBUG_NEWSRC)
+				debug_print_newsrc(&group->newsrc, fp);
+			if (debug & DEBUG_ATTRIB)
+				debug_print_attributes(group->attribute, fp);
 		}
 		fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
 		fclose(fp);
@@ -222,14 +189,14 @@ debug_print_attributes(
 
 	fprintf(fp, "global=[%d] show=[%d] thread=[%d] sort=[%d] author=[%d] auto_select=[%d] auto_save=[%d] batch_save=[%d] process=[%d]\n",
 		attr->global,
-		attr->show_only_unread,
-		attr->thread_arts,
-		attr->sort_art_type,
+		attr->show_only_unread_arts,
+		attr->thread_articles,
+		attr->sort_article_type,
 		attr->show_author,
 		attr->auto_select,
 		attr->auto_save,
 		attr->batch_save,
-		attr->post_proc_type);
+		attr->post_process_type);
 	fprintf(fp, "select_header=[%d] select_global=[%s] select_expire=[%s]\n",
 		attr->quick_select_header,
 		BlankIfNull(attr->quick_select_scope),
@@ -258,17 +225,14 @@ debug_print_malloc(
 {
 	FILE *fp;
 	char file[PATH_LEN];
-	static int total = 0;
+	static size_t total = 0;
 
-	if (debug == 4) {
-		joinpath(file, TMPDIR, "MALLOC");
+	if (debug & DEBUG_MEM) {
+		joinpath(file, sizeof(file), TMPDIR, "MALLOC");
 		if ((fp = fopen(file, "a+")) != NULL) {
 			total += size;
 			/* sometimes size_t is long */
-			if (is_malloc)
-				fprintf(fp, "%10s:%-4d Malloc  %6ld. Total %d\n", xfile, line, (long) size, total);
-			else
-				fprintf(fp, "%10s:%-4d Realloc %6ld. Total %d\n", xfile, line, (long) size, total);
+			fprintf(fp, "%12s:%-4d %s(%6lu). Total %lu\n", xfile, line, is_malloc ? " malloc" : "realloc" , (unsigned long) size, (unsigned long) total);
 			fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
 			fclose(fp);
 		}
@@ -282,11 +246,11 @@ debug_print_filter(
 	int num,
 	struct t_filter *the_filter)
 {
-	fprintf(fp, "[%3d]  group=[%s] inscope=[%s] score=[%d] case=[%d] lines=[%d %d]\n",
+	fprintf(fp, "[%3d]  group=[%s] inscope=[%s] score=[%d] case=[%s] lines=[%d %d]\n",
 		num, BlankIfNull(the_filter->scope),
 		(the_filter->inscope ? "TRUE" : "FILTER"),
 		the_filter->score,
-		the_filter->icase,
+		the_filter->icase ? "C" : "I",
 		the_filter->lines_cmp, the_filter->lines_num);
 
 	fprintf(fp, "       subj=[%s] from=[%s] msgid=[%s]\n",
@@ -295,7 +259,7 @@ debug_print_filter(
 		BlankIfNull(the_filter->msgid));
 
 	if (the_filter->time)
-		fprintf(fp, "       time=[%ld][%s", the_filter->time, (the_filter->time ? ctime(&the_filter->time) : "]\n"));
+		fprintf(fp, "       time=[%ld][%s]\n", the_filter->time, BlankIfNull(str_trim(ctime(&the_filter->time))));
 }
 
 
@@ -308,10 +272,10 @@ debug_print_filters(
 	int i, num;
 	struct t_filter *the_filter;
 
-	if (debug < 2)
+	if (!(debug & DEBUG_FILTER))
 		return;
 
-	joinpath(file, TMPDIR, "FILTER");
+	joinpath(file, sizeof(file), TMPDIR, "FILTER");
 
 	if ((fp = fopen(file, "w")) != NULL) {
 		/*
@@ -332,41 +296,43 @@ debug_print_filters(
 }
 
 
-/* TODO: print out all fields of t_capabilities */
-#	ifdef NNTP_ABLE
 void
-debug_print_nntp_extensions(
-	void)
+debug_print_file(
+	const char *fname,
+	const char *fmt,
+	...)
 {
-	if (debug < 2)
+	FILE *fp;
+	char *buf;
+	char file[PATH_LEN];
+	va_list ap;
+
+	if (!debug)
 		return;
-	debug_nntp("###", "NNTP EXTENSIONS/CAPABILITIES");
-	debug_nntp("###", "Type/Version : %d/%d", nntp_caps.type, nntp_caps.version);
-	debug_nntp("###", "Command-names: %s %s", BlankIfNull(nntp_caps.over_cmd), BlankIfNull(nntp_caps.hdr_cmd));
-	debug_nntp("###", "List         : %s", nntp_caps.list_motd ? "MOTD" : "");
+
+	va_start(ap, fmt);
+	buf = fmt_message(fmt, ap);
+
+	joinpath(file, sizeof(file), TMPDIR, fname);
+
+	if ((fp = fopen(file, "a+")) != NULL) {
+		fprintf(fp,"%s\n", buf);
+		fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
+		fclose(fp);
+	}
+	free(buf);
+	va_end(ap);
 }
-#	endif /* NNTP_ABLE */
-#endif /* DEBUG */
 
 
-#ifdef DEBUG_NEWSRC
 void
 debug_print_comment(
 	const char *comment)
 {
-	FILE *fp;
-	char file[PATH_LEN];
-
-	if (debug < 2)
+	if (!(debug & DEBUG_NEWSRC))
 		return;
 
-	joinpath(file, TMPDIR, "BITMAP");
-
-	if ((fp = fopen(file, "a+")) != NULL) {
-		fprintf(fp,"\n%s\n", comment);
-		fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
-		fclose(fp);
-	}
+	debug_print_file("BITMAP", comment);
 }
 
 
@@ -378,34 +344,36 @@ debug_print_bitmap(
 	FILE *fp;
 	char file[PATH_LEN];
 
-	if (debug != 3)
+	if (!(debug & DEBUG_NEWSRC))
 		return;
 
-	joinpath(file, TMPDIR, "BITMAP");
-	if ((fp = fopen(file, "a+")) != NULL) {
-		fprintf(fp, "\nActive: Group=[%s] sub=[%c] min=[%ld] max=[%ld] count=[%ld] num_unread=[%ld]\n",
-			group->name, SUB_CHAR(group->subscribed),
-			group->xmin, group->xmax, group->count,
-			group->newsrc.num_unread);
-		if (art != NULL) {
-			fprintf(fp, "art=[%5ld] tag=[%s] kill=[%s] selected=[%s] subj=[%s]\n",
-				art->artnum,
-				bool_unparse(art->tagged),
-				bool_unparse(art->killed),
-				bool_unparse(art->selected),
-				art->subject);
-			fprintf(fp, "thread=[%d]  prev=[%d]  status=[%s]\n",
-				art->thread, art->prev,
-				(art->status == ART_READ ? "READ" : "UNREAD"));
+	joinpath(file, sizeof(file), TMPDIR, "BITMAP");
+	if (group != NULL) {
+		if ((fp = fopen(file, "a+")) != NULL) {
+			fprintf(fp, "\nActive: Group=[%s] sub=[%c] min=[%ld] max=[%ld] count=[%ld] num_unread=[%ld]\n",
+				group->name, SUB_CHAR(group->subscribed),
+				group->xmin, group->xmax, group->count,
+				group->newsrc.num_unread);
+			if (art != NULL) {
+				fprintf(fp, "art=[%5ld] tag=[%s] kill=[%s] selected=[%s] subj=[%s]\n",
+					art->artnum,
+					bool_unparse(art->tagged),
+					bool_unparse(art->killed),
+					bool_unparse(art->selected),
+					art->subject);
+				fprintf(fp, "thread=[%d]  prev=[%d]  status=[%s]\n",
+					art->thread, art->prev,
+					(art->status == ART_READ ? "READ" : "UNREAD"));
+			}
+			debug_print_newsrc(&group->newsrc, fp);
 		}
-		debug_print_newsrc(&group->newsrc, fp);
 		fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
 		fclose(fp);
 	}
 }
 
 
-void
+static void
 debug_print_newsrc(
 	struct t_newsrc *lnewsrc,
 	FILE *fp)
@@ -429,4 +397,4 @@ debug_print_newsrc(
 	fprintf(fp, "]\n");
 	fflush(fp);
 }
-#endif /* DEBUG_NEWSRC */
+#endif /* DEBUG */
