@@ -3,10 +3,10 @@
  *  Module    : feed.c
  *  Author    : I. Lea
  *  Created   : 1991-08-31
- *  Updated   : 2006-09-02
+ *  Updated   : 2008-11-22
  *  Notes     : provides same interface to mail,pipe,print,save & repost commands
  *
- * Copyright (c) 1991-2007 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -66,7 +66,7 @@ struct t_counters {
  * Local prototypes
  */
 static char *get_save_filename(struct t_group *group, int function, char *filename, int filelen, int respnum);
-static t_bool expand_feed_filename(char *outpath, const char *path);
+static t_bool expand_feed_filename(char *outpath, size_t outpath_len, const char *path);
 static t_bool feed_article(int art, int function, struct t_counters *counter, t_bool use_current, const char *data, struct t_group *group);
 static t_function get_feed_key(int function, int level, struct t_group *group, struct t_art_stat *thread, int respnum);
 static t_function get_post_proc_type(void);
@@ -102,7 +102,7 @@ get_save_filename(
 	/*
 	 * Group attribute savefile overrides tinrc default savefile
 	 */
-	strncpy(default_savefile, (group->attribute->savefile ? group->attribute->savefile : tinrc.default_save_file), sizeof(default_savefile) - 1);
+	my_strncpy(default_savefile, (group->attribute->savefile ? group->attribute->savefile : tinrc.default_save_file), sizeof(default_savefile) - 1);
 
 	/*
 	 * We don't ask when auto'S'aving or Archive-Name saving with auto_save
@@ -156,6 +156,7 @@ get_save_filename(
 static t_bool
 expand_feed_filename(
 	char *outpath,
+	size_t outpath_len,
 	const char *path)
 {
 	int ret = strfpath(path, outpath, PATH_LEN, curr_group);
@@ -168,8 +169,8 @@ expand_feed_filename(
 		char buf[PATH_LEN];
 
 		if (!strfpath(curr_group->attribute->savedir, buf, sizeof(buf), curr_group))
-			joinpath(buf, homedir, DEFAULT_SAVEDIR);
-		joinpath(outpath, buf, path);
+			joinpath(buf, sizeof(buf), homedir, DEFAULT_SAVEDIR);
+		joinpath(outpath, outpath_len, buf, path);
 		return FALSE;
 	} else
 		return (ret == 1);
@@ -191,7 +192,7 @@ get_post_proc_type(
 	char keyshar[MAXKEYLEN];
 	t_function default_func, func;
 
-	switch (curr_group->attribute->post_proc_type) {
+	switch (curr_group->attribute->post_process_type) {
 		case POST_PROC_YES:
 			default_func = POSTPROCESS_YES;
 			break;
@@ -505,7 +506,7 @@ feed_article(
 	 * Mark read for the SAVE cases (but not print/pipe etc..)
 	 */
 	if (function == FEED_SAVE || function == FEED_AUTOSAVE) {
-		if (ok && tinrc.mark_saved_read)
+		if (ok && curr_group->attribute->mark_saved_read)
 			art_mark(curr_group, &arts[art], ART_READ);
 	}
 
@@ -558,7 +559,7 @@ feed_articles(
 
 #ifdef DONT_HAVE_PIPING
 	if (function == FEED_PIPE) {
-		error_message(_(txt_piping_not_enabled));
+		error_message(2, _(txt_piping_not_enabled));
 		clear_message();
 		return;
 	}
@@ -639,7 +640,7 @@ feed_articles(
 				if (get_save_filename(group, function, savefile, sizeof(savefile), respnum) == NULL)
 					return;
 
-				switch (curr_group->attribute->post_proc_type) {
+				switch (curr_group->attribute->post_process_type) {
 					case POST_PROC_YES:
 						pproc_func = POSTPROCESS_YES;
 						break;
@@ -655,7 +656,7 @@ feed_articles(
 				}
 
 				/* We don't postprocess mailboxen */
-				if ((is_mailbox = expand_feed_filename(outpath, savefile)) == TRUE)
+				if ((is_mailbox = expand_feed_filename(outpath, sizeof(outpath), savefile)) == TRUE)
 					pproc_func = POSTPROCESS_NO;
 				else {
 					if (function != FEED_AUTOSAVE && (pproc_func = get_post_proc_type()) == GLOBAL_ABORT)
@@ -752,12 +753,12 @@ feed_articles(
 		case FEED_THREAD:		/* thread */
 			/* Get accurate count first */
 			for_each_art_in_thread(art, which_thread(respnum)) {
-				if (!(tinrc.process_only_unread && arts[art].status == ART_READ))
+				if (!(curr_group->attribute->process_only_unread && arts[art].status == ART_READ))
 					counter.max++;
 			}
 
 			for_each_art_in_thread(art, which_thread(respnum)) {
-				if (!(tinrc.process_only_unread && arts[art].status == ART_READ)) {
+				if (!(curr_group->attribute->process_only_unread && arts[art].status == ART_READ)) {
 					/* Keep going - don't abort on errors */
 					if (!feed_article(art, function, &counter, use_current, outpath, group))
 						handle_SIGPIPE();
@@ -795,7 +796,7 @@ feed_articles(
 					} else if (!arts[art].selected)
 						continue;
 
-					if (tinrc.process_only_unread && arts[art].status == ART_READ)
+					if (curr_group->attribute->process_only_unread && arts[art].status == ART_READ)
 						continue;
 
 					arts[art].matched = TRUE;
@@ -881,7 +882,7 @@ got_sig_pipe_while_piping:
 	}
 
 	if (level != PAGE_LEVEL)
-		ret1 = tinrc.mark_saved_read;
+		ret1 = curr_group->attribute->mark_saved_read;
 
 	if (ret1 || post_processed_ok)
 		redraw_screen = TRUE;
@@ -957,7 +958,7 @@ print_file(
 		return FALSE;
 	}
 
-	if (tinrc.print_header)
+	if (curr_group->attribute->print_header)
 		rewind(artinfo->raw);
 	else {
 		if (hdr->newsgroups)
