@@ -3,10 +3,10 @@
  *  Module    : filter.c
  *  Author    : I. Lea
  *  Created   : 1992-12-28
- *  Updated   : 2006-06-20
+ *  Updated   : 2008-11-28
  *  Notes     : Filter articles. Kill & auto selection are supported.
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -209,7 +209,7 @@ test_regex(
 			if (regex_errpos >= 0)
 				return 1;
 			else if (regex_errpos != PCRE_ERROR_NOMATCH) {
-				error_message(_(txt_pcre_error_num), regex_errpos);
+				error_message(2, _(txt_pcre_error_num), regex_errpos);
 				return -1;
 			}
 		}
@@ -310,7 +310,7 @@ read_filter_file(
 	t_bool expired_time = FALSE;
 	time_t current_secs = (time_t) 0;
 	static t_bool first_read = TRUE;
-	int upgrade = RC_CHECK;
+	enum rc_state upgrade = RC_CHECK;
 
 	if ((fp = fopen(file, "r")) == NULL)
 		return FALSE;
@@ -327,13 +327,14 @@ read_filter_file(
 		free_filter_array(&glob_filter);
 
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
-		if (*buf == '#' || *buf == '\n') {
-			if (upgrade == RC_CHECK && first_read) {
+		if (*buf == '\n')
+			continue;
+		if (*buf == '#') {
+			if (upgrade == RC_CHECK && first_read && match_string(buf, "# Filter file V", NULL, 0)) {
 				first_read = FALSE;
 				upgrade = check_upgrade(buf, "# Filter file V", FILTER_VERSION);
 				if (upgrade != RC_IGNORE)
-					upgrade_prompt_quit(upgrade, FILTER_FILE);
-				/* TODO: do something (more) useful here */
+					upgrade_prompt_quit(upgrade, FILTER_FILE); /* TODO: do something (more) useful here */
 			}
 			continue;
 		}
@@ -346,8 +347,10 @@ read_filter_file(
 
 					break;
 				}
-				if (match_string(buf + 1, "omment=", comment_line, sizeof(comment_line)))
+				if (match_string(buf + 1, "omment=", comment_line, sizeof(comment_line))) {
+					str_trim(comment_line);
 					comment = add_filter_comment(comment, comment_line);
+				}
 				break;
 
 			case 'f':
@@ -368,11 +371,10 @@ read_filter_file(
 
 			case 'g':
 				if (match_string(buf + 1, "roup=", scope, sizeof(scope))) {
+					str_trim(scope);
 #ifdef DEBUG
-					if (debug) {
-						my_printf("group=[%s] num=[%d]\n", scope, glob_filter.num);
-						my_flush();
-					}
+					if (debug & DEBUG_FILTER)
+						debug_print_file("FILTER", "\nnum=[%d] group=[%s]", glob_filter.num, scope);
 #endif /* DEBUG */
 					if (glob_filter.num >= glob_filter.max)
 						expand_filter_array(&glob_filter);
@@ -505,12 +507,9 @@ read_filter_file(
 							ptr[i].subj = my_strdup(subj);
 						}
 					}
-
 #ifdef DEBUG
-					if (debug) {
-						my_printf("6. buf=[%s]  Gsubj=[%s]\n", ptr[i].subj, glob_filter.filter[i].subj);
-						my_flush();
-					}
+					if (debug & DEBUG_FILTER)
+						debug_print_file("FILTER","buf=[%s]  Gsubj=[%s]", ptr[i].subj, glob_filter.filter[i].subj);
 #endif /* DEBUG */
 					break;
 				}
@@ -521,10 +520,8 @@ read_filter_file(
 				if (match_string(buf + 1, "core=", scbuf, PATH_LEN)) {
 					score = atoi(scbuf);
 #ifdef DEBUG
-					if (debug) {
-						my_printf("score=[%d]\n", score);
-						my_flush();
-					}
+					if (debug & DEBUG_FILTER)
+						debug_print_file("FILTER","score=[%d]", score);
 #endif /* DEBUG */
 					if (ptr && !expired_time) {
 						if (score > SCORE_MAX)
@@ -555,10 +552,8 @@ read_filter_file(
 						/* rule expired? */
 						if (secs && current_secs > (time_t) secs) {
 #ifdef DEBUG
-							if (debug) {
-								my_printf("EXPIRED  secs=[%lu]  current_secs=[%lu]\n", (unsigned long int) secs, (unsigned long int) current_secs);
-								my_flush();
-							}
+							if (debug & DEBUG_FILTER)
+								debug_print_file("FILTER", "EXPIRED  secs=[%lu]  current_secs=[%lu]", (unsigned long int) secs, (unsigned long int) current_secs);
 #endif /* DEBUG */
 							glob_filter.num--;
 							expired_time = TRUE;
@@ -574,6 +569,7 @@ read_filter_file(
 				 *       should we comment out older xref rules like below?
 				 */
 				if (match_string(buf + 1, "ref=", xref, sizeof(xref))) {
+					str_trim(xref);
 					if (ptr && !expired_time) {
 						if (tinrc.wildcard && ptr[i].xref != NULL) {
 							/* merge with already read value */
@@ -644,7 +640,7 @@ write_filter_file(
 	file_tmp = get_tmpfilename(filename);
 
 	if (!backup_file(filename, file_tmp)) {
-		error_message(_(txt_filesystem_full_backup), filename);
+		error_message(2, _(txt_filesystem_full_backup), filename);
 		free(file_tmp);
 		return;
 	}
@@ -663,7 +659,7 @@ write_filter_file(
 	write_filter_array(fp, &glob_filter);
 
 	if (ferror(fp) || fclose(fp)) {
-		error_message(_(txt_filesystem_full), filename);
+		error_message(2, _(txt_filesystem_full), filename);
 		rename_file(file_tmp, filename);
 	} else
 		unlink(file_tmp);
@@ -685,14 +681,18 @@ write_filter_array(
 		return;
 
 	for (i = 0; i < ptr->num; i++) {
-/* my_printf("WRITE i=[%d] subj=[%s] from=[%s]\n", i, BlankIfNull(ptr->filter[i].subj), BlankIfNull(ptr->filter[i].from)); */
+#ifdef DEBUG
+		if (debug & DEBUG_FILTER)
+			debug_print_file("FILTER", "WRITE i=[%d] subj=[%s] from=[%s]\n", i, BlankIfNull(ptr->filter[i].subj), BlankIfNull(ptr->filter[i].from));
+#endif /* DEBUG */
 
 		if (ptr->filter[i].time && theTime > ptr->filter[i].time)
 				continue;
-/*
-my_printf("Scope=[%s]" cCRLF, (ptr->filter[i].scope != NULL ? ptr->filter[i].scope : "*"));
-my_flush();
-*/
+#ifdef DEBUG
+		if (debug & DEBUG_FILTER)
+			debug_print_file("FILTER", "Scope=[%s]" cCRLF, (ptr->filter[i].scope != NULL ? ptr->filter[i].scope : "*"));
+#endif /* DEBUG */
+
 		fprintf(fp, "\n");		/* makes filter file more readable */
 
 		/* comments appear always first, if there are any... */
@@ -1409,7 +1409,7 @@ filter_menu(
 		case FILTER_EDIT:
 			add_filter_rule(group, art, &rule, FALSE); /* save the rule */
 			rule.comment = free_filter_comment(rule.comment);
-			if (!invoke_editor(filter_file, FILTER_FILE_OFFSET))
+			if (!invoke_editor(filter_file, FILTER_FILE_OFFSET, NULL))
 				return FALSE;
 			unfilter_articles();
 			(void) read_filter_file(filter_file);
@@ -1471,10 +1471,8 @@ quick_filter(
 	}
 
 #ifdef DEBUG
-	if (debug)
-		error_message("%s header=[%d] scope=[%s] expire=[%s] case=[%d]",
-			(type == GLOBAL_QUICK_FILTER_KILL) ? "KILL" : "SELECT", header,
-			BlankIfNull(scope), txt_onoff[expire != FALSE ? 1 : 0], icase);
+	if (debug & DEBUG_FILTER)
+		error_message(2, "%s header=[%d] scope=[%s] expire=[%s] case=[%d]", (type == GLOBAL_QUICK_FILTER_KILL) ? "KILL" : "SELECT", header, BlankIfNull(scope), txt_onoff[expire != FALSE ? 1 : 0], icase);
 #endif /* DEBUG */
 
 	/*
@@ -1724,7 +1722,7 @@ add_filter_rule(
 			 * So the thread remains open (in group level). To overcome this,
 			 * the first msgid from references field is taken in this case.
 			 */
-			if (quick_filter_rule && group->attribute->thread_arts == THREAD_REFS &&
+			if (quick_filter_rule && group->attribute->thread_articles == THREAD_REFS &&
 				(group->attribute->quick_kill_header == FILTER_MSGID ||
 				 group->attribute->quick_kill_header == FILTER_REFS_ONLY) &&
 				 art->refptr->parent != NULL)
@@ -1750,18 +1748,9 @@ add_filter_rule(
 
 	if (filtered) {
 #ifdef DEBUG
-		if (debug)
-			wait_message(2, "inscope=[%s] scope=[%s]  case=[%d] subj=[%s] from=[%s] msgid=[%s] fullref=[%d] line=[%d %d] time=[%lu]",
-				bool_unparse(ptr[i].inscope),
-				BlankIfNull(rule->scope),
-				ptr[i].icase,
-				BlankIfNull(ptr[i].subj),
-				BlankIfNull(ptr[i].from),
-				BlankIfNull(ptr[i].msgid),
-				ptr[i].fullref, ptr[i].lines_cmp,
-				ptr[i].lines_num, (unsigned long int) ptr[i].time);
+		if (debug & DEBUG_FILTER)
+			wait_message(2, "inscope=[%s] scope=[%s]  case=[%d] subj=[%s] from=[%s] msgid=[%s] fullref=[%d] line=[%d %d] time=[%lu]", bool_unparse(ptr[i].inscope), BlankIfNull(rule->scope), ptr[i].icase, BlankIfNull(ptr[i].subj), BlankIfNull(ptr[i].from), BlankIfNull(ptr[i].msgid), ptr[i].fullref, ptr[i].lines_cmp, ptr[i].lines_num, (unsigned long int) ptr[i].time);
 #endif /* DEBUG */
-
 		write_filter_file(filter_file);
 	}
 	return filtered;
@@ -1962,6 +1951,10 @@ filter_articles(
 					switch (x) {
 						case 1:
 							SET_FILTER(group, i, j);
+#ifdef DEBUG
+							if (debug & DEBUG_FILTER)
+								debug_print_file("FILTER", "Group[%s] Rule[%d][%s] ArtMSGID[%s] Artnum[%d]", group->name, j, ptr[j].msgid, mymsgid, i);
+#endif /* DEBUG */
 							break;
 
 						case -1:
@@ -2051,7 +2044,7 @@ wait_message(1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].lin
 						t_bool skip = FALSE;
 
 						s = arts[i].xref;
-						while (*s && !isspace((int) *s))
+						while (*s && !isspace((int) *s))	/* skip server name */
 							s++;
 						while (*s && isspace((int) *s))
 							s++;
