@@ -3,9 +3,9 @@
  *  Module    : header.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-03-10
- *  Updated   : 2006-02-15
+ *  Updated   : 2008-11-22
  *
- * Copyright (c) 1997-2006 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2009 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,7 @@ get_host_name(
 #endif /* HAVE_GETHOSTNAME */
 #ifdef HAVE_SYS_UTSNAME_H
 	if (!*hostname)
-		strncpy(hostname, system_info.nodename, sizeof(hostname) - 1);
+		my_strncpy(hostname, system_info.nodename, sizeof(hostname) - 1);
 #endif /* HAVE_SYS_UTSNAME_H */
 	if (!*hostname) {
 		if ((ptr = getenv("HOST")) != NULL)
@@ -133,7 +133,7 @@ get_fqdn(
 	char name[MAXHOSTNAMELEN + 1];
 	static char fqdn[1024];
 	struct hostent *hp;
-	struct in_addr in;
+	struct in_addr in = {0};
 
 	*fqdn = '\0';
 	domain = NULL;
@@ -142,7 +142,7 @@ get_fqdn(
 	if (host) {
 		if (strchr(host, '.'))
 			return host;
-		strncpy(name, host, sizeof(name) - 1);
+		my_strncpy(name, host, sizeof(name) - 1);
 	} else {
 #	ifdef HAVE_GETHOSTNAME
 		if (gethostname(name, sizeof(name) - 1))
@@ -236,7 +236,7 @@ get_user_name(
 		strcpy(username, pw->pw_name);
 	else {
 		if (!*username) {
-			error_message(_(txt_error_passwd_missing));
+			error_message(2, _(txt_error_passwd_missing));
 			tin_done(EXIT_FAILURE);
 		}
 	}
@@ -299,14 +299,7 @@ get_from_name(
 	char *from_name,
 	struct t_group *thisgrp)
 {
-#	ifdef USE_INN_NNTPLIB
-	char *fromhost = GetConfigValue(_CONF_FROMHOST);
-#	else
-	char *fromhost = NULL;
-#	endif /* USE_INN_NNTPLIB */
-
-	if (!(fromhost && *fromhost))
-		fromhost = domain_name;
+	char *fromhost = domain_name;
 
 	if (thisgrp && *thisgrp->attribute->from != '\0') {
 		strcpy(from_name, thisgrp->attribute->from);
@@ -316,8 +309,8 @@ get_from_name(
 	sprintf(from_name, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), get_full_name(), get_user_name(), fromhost);
 
 #	ifdef DEBUG
-	if (debug == 2)
-		error_message("FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, get_user_name(), domain_name, get_full_name());
+	if (debug & DEBUG_MISC)
+		error_message(2, "FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, get_user_name(), domain_name, get_full_name());
 #	endif /* DEBUG */
 
 }
