@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2006-06-28
+ *  Updated   : 2009-02-02
  *  Notes     :
  *
- * Copyright (c) 1991-2007 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,16 +55,11 @@
  * local prototypes
  */
 static int read_site_config(void);
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	static t_bool utf8_pcre(void);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #ifdef HAVE_COLOR
 	static void preinit_colors(void);
 #endif /* HAVE_COLOR */
 
 
-char **news_headers_to_display_array;	/* array of which headers to display */
-char **news_headers_to_not_display_array;	/* array of which headers to not display */
 char active_times_file[PATH_LEN];
 char article_name[PATH_LEN];			/* ~/TIN_ARTICLE_NAME file */
 char bug_nntpserver1[PATH_LEN];		/* welcome message of NNTP server used */
@@ -83,7 +78,6 @@ char index_maildir[PATH_LEN];
 char index_newsdir[PATH_LEN];	/* directory for private overview data */
 char index_savedir[PATH_LEN];
 char inewsdir[PATH_LEN];
-char libdir[PATH_LEN];			/* directory where news config files are (ie. active) */
 char local_attributes_file[PATH_LEN];
 char local_config_file[PATH_LEN];
 char local_input_history_file[PATH_LEN];
@@ -104,7 +98,8 @@ char postponed_articles_file[PATH_LEN];	/* ~/.tin/postponed.articles file */
 char rcdir[PATH_LEN];
 char save_active_file[PATH_LEN];
 char spooldir[PATH_LEN];		/* directory where news is */
-char subscriptions_file[PATH_LEN];
+char overviewfmt_file[PATH_LEN];	/* full path to overview.fmt */
+char subscriptions_file[PATH_LEN];	/* full path to subscriptions */
 char tin_progname[PATH_LEN];		/* program name */
 char txt_help_bug_report[LEN];		/* address to send bug reports to */
 char userid[PATH_LEN];
@@ -121,8 +116,6 @@ t_function last_search;	/* for repeated search */
 int hist_last[HIST_MAXNUM + 1];
 int hist_pos[HIST_MAXNUM + 1];
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
-int num_headers_to_display;		/* num headers to display -- swp */
-int num_headers_to_not_display;		/* num headers to not display -- swp */
 int system_status;
 int xmouse, xrow, xcol;			/* xterm button pressing information */
 
@@ -241,7 +234,7 @@ struct t_config tinrc = {
 #else
 	-1,		/* mm_network_charset, defaults to $MM_CHARSET */
 #endif /* !CHARSET_CONVERSION */
-	"",		/* mm_local_charset, display charset */
+	"US-ASCII",		/* mm_local_charset, display charset */
 #ifdef HAVE_ICONV_OPEN_TRANSLIT
 	FALSE,	/* translit */
 #endif /* HAVE_ICONV_OPEN_TRANSLIT */
@@ -278,9 +271,9 @@ struct t_config tinrc = {
 	32,		/* groupname_max_length */
 	UUE_NO,	/* hide_uue */
 	KILL_UNREAD,		/* kill_level */
-	MIME_ENCODING_7BIT,		/* mail_mime_encoding */
-	MIME_ENCODING_7BIT,		/* post_mime_encoding */
-	POST_PROC_NO,			/* post_process */
+	MIME_ENCODING_QP,		/* mail_mime_encoding */
+	MIME_ENCODING_8BIT,		/* post_mime_encoding */
+	POST_PROC_NO,			/* post_process_type */
 	REREAD_ACTIVE_FILE_SECS,	/* reread_active_file_secs */
 	1,		/* scroll_lines */
 	SHOW_FROM_NAME,				/* show_author */
@@ -295,6 +288,7 @@ struct t_config tinrc = {
 	50,		/* score_limit_select */
 	-100,		/* score_kill */
 	100,		/* score_select */
+	0,		/* trim_article_body */
 #ifdef HAVE_COLOR
 	0,		/* col_back (initialised later) */
 	0,		/* col_from (initialised later) */
@@ -316,6 +310,7 @@ struct t_config tinrc = {
 	0,		/* col_response (initialised later) */
 	0,		/* col_signature (initialised later) */
 	0,		/* col_urls (initialised later) */
+	0,		/* col_verbatim (initialised later) */
 	0,		/* col_subject (initialised later) */
 	0,		/* col_text (initialised later) */
 	0,		/* col_title (initialised later) */
@@ -334,8 +329,7 @@ struct t_config tinrc = {
 	TRUE,		/* add_posted_to_filter */
 	TRUE,		/* advertising */
 	TRUE,		/* alternative_handling */
-	FALSE,		/* auto_bcc */
-	FALSE,		/* auto_cc */
+	0,			/* auto_cc_bcc */
 	TRUE,		/* auto_list_thread */
 	FALSE,		/* auto_reconnect */
 	FALSE,		/* auto_save */
@@ -388,6 +382,7 @@ struct t_config tinrc = {
 	FALSE,		/* tex2iso_conv */
 	TRUE,		/* thread_catchup_on_exit */
 	TRUE,		/* unlink_article */
+	TRUE,		/* verbatim_handling */
 	"",		/* inews_prog */
 	INTERACTIVE_NONE,		/* interactive_mailer */
 	FALSE,		/* use_mouse */
@@ -415,7 +410,7 @@ struct t_config tinrc = {
 };
 
 struct t_capabilities nntp_caps = {
-	0, /* type (none, LIST EXTENSIONS, CAPABILITIES) */
+	NONE, /* type (none, LIST EXTENSIONS, CAPABILITIES, BROKEN) */
 	0, /* CAPABILITIES version */
 	FALSE, /* MODE-READER: "MODE READER" */
 	FALSE, /* READER: "ARTICLE", "BODY" */
@@ -441,17 +436,18 @@ struct t_capabilities nntp_caps = {
 	FALSE, /* STARTTLS */
 	FALSE, /* AUTHINFO USER/PASS */
 	FALSE, /* AUTHINFO SASL */
-#if 0
 	FALSE, /* SASL CRAM-MD5 */
 	FALSE, /* SASL DIGEST-MD5 */
 	FALSE, /* SASL PLAIN */
 	FALSE, /* SASL GSSAPI */
 	FALSE, /* SASL EXTERNAL */
+#if 0
 	FALSE, /* STREAMING: "MODE STREAM", "CHECK", "TAKETHIS" */
 	FALSE /* IHAVE */
 #endif /* 0 */
 };
 
+static char libdir[PATH_LEN];			/* directory where news config files are (ie. active) */
 static mode_t real_umask;
 
 #ifdef HAVE_COLOR
@@ -484,11 +480,13 @@ static const struct {
 	{ &tinrc.col_response,    2 },
 	{ &tinrc.col_signature,   4 },
 	{ &tinrc.col_urls,       -1 },
+	{ &tinrc.col_verbatim,    5 },
 	{ &tinrc.col_subject,     6 },
 	{ &tinrc.col_text,       DFT_FORE },
 	{ &tinrc.col_title,       4 },
 };
 
+
 static void
 preinit_colors(
 	void)
@@ -545,9 +543,7 @@ init_selfinfo(
 
 #ifdef HAVE_SYS_UTSNAME_H
 #	ifdef HAVE_UNAME
-	if (uname(&system_info) != -1)
-		;
-	else
+	if (uname(&system_info) == -1)
 #	endif /* HAVE_UNAME */
 	{
 		strcpy(system_info.sysname, "unknown");
@@ -576,7 +572,7 @@ init_selfinfo(
 	(void) umask(real_umask);
 
 	if ((myentry = getpwuid(getuid())) == NULL) {
-		error_message(_(txt_error_passwd_missing));
+		error_message(2, _(txt_error_passwd_missing));
 		giveup();
 	}
 
@@ -602,8 +598,6 @@ init_selfinfo(
 		iso2asc_supported = -1;
 	list_active = FALSE;
 	newsrc_active = FALSE;
-	num_headers_to_display = 0;
-	num_headers_to_not_display = 0;
 	post_article_and_exit = FALSE;
 	post_postponed_and_exit = FALSE;
 	read_local_newsgroups_file = FALSE;
@@ -630,8 +624,6 @@ init_selfinfo(
 	snprintf(cvers, sizeof(cvers), txt_copyright_notice, page_header);
 
 	default_organization[0] = '\0';
-	news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
-	news_headers_to_not_display_array = NULL;
 
 	strncpy(bug_addr, BUG_REPORT_ADDRESS, sizeof(bug_addr) - 1);
 
@@ -650,11 +642,6 @@ init_selfinfo(
 	my_strncpy(default_organization, get_val("ORGANIZATION", ""), sizeof(default_organization) - 1);
 #endif /* apollo */
 
-#ifdef USE_INN_NNTPLIB
-	if ((ptr = GetConfigValue(_CONF_ORGANIZATION)) != NULL)
-		my_strncpy(default_organization, ptr, sizeof(default_organization) - 1);
-#endif /* USE_INN_NNTPLIB */
-
 #ifndef NNTP_ONLY
 	my_strncpy(libdir, get_val("TIN_LIBDIR", NEWSLIBDIR), sizeof(libdir) - 1);
 	my_strncpy(novrootdir, get_val("TIN_NOVROOTDIR", NOVROOTDIR), sizeof(novrootdir) - 1);
@@ -665,6 +652,7 @@ init_selfinfo(
 	news_active_file[0] = '\0';
 	active_times_file[0] = '\0';
 	newsgroups_file[0] = '\0';
+	overviewfmt_file[0] = '\0';
 	subscriptions_file[0] = '\0';
 
 	/*
@@ -678,8 +666,7 @@ init_selfinfo(
 	 * if it's still unset fall into no posting mode.
 	 */
 	if (domain_name[0] == '\0') {
-		error_message(_(txt_error_no_domain_name));
-		sleep(2);
+		error_message(4, _(txt_error_no_domain_name));
 		force_no_post = TRUE;
 	}
 
@@ -691,18 +678,20 @@ init_selfinfo(
 	 * values given in env-vars? ($MM_CHARSET, $TIN_ACTIVEFILE)
 	 */
 	if (!*news_active_file) /* TODO: really prepend libdir here in case of $TIN_ACTIVEFILE is set? */
-		joinpath(news_active_file, libdir, get_val("TIN_ACTIVEFILE", ACTIVE_FILE));
+		joinpath(news_active_file, sizeof(news_active_file), libdir, get_val("TIN_ACTIVEFILE", ACTIVE_FILE));
 	if (!*active_times_file)
-		joinpath(active_times_file, libdir, ACTIVE_TIMES_FILE);
+		joinpath(active_times_file, sizeof(active_times_file), libdir, ACTIVE_TIMES_FILE);
 	if (!*newsgroups_file)
-		joinpath(newsgroups_file, libdir, NEWSGROUPS_FILE);
+		joinpath(newsgroups_file, sizeof(newsgroups_file), libdir, NEWSGROUPS_FILE);
 	if (!*subscriptions_file)
-		joinpath(subscriptions_file, libdir, SUBSCRIPTIONS_FILE);
+		joinpath(subscriptions_file, sizeof(subscriptions_file), libdir, SUBSCRIPTIONS_FILE);
+	if (!*overviewfmt_file)
+		joinpath(overviewfmt_file, sizeof(overviewfmt_file), libdir, OVERVIEW_FMT);
 	if (!*default_organization) {
-		char buf[LEN];
+		char buf[LEN], filename[PATH_LEN];
 
-		joinpath(buf, libdir, "organization");
-		if ((fp = fopen(buf, "r")) != NULL) {
+		joinpath(filename, sizeof(filename), libdir, "organization");
+		if ((fp = fopen(filename, "r")) != NULL) {
 			if (fgets(buf, (int) sizeof(buf), fp) != NULL) {
 				ptr = strrchr(buf, '\n');
 				if (ptr != NULL)
@@ -736,15 +725,15 @@ init_selfinfo(
 #endif /* !CHARSET_CONVERSION */
 
 #ifdef TIN_DEFAULTS_DIR
-	joinpath(global_attributes_file, TIN_DEFAULTS_DIR, ATTRIBUTES_FILE);
-	joinpath(global_config_file, TIN_DEFAULTS_DIR, CONFIG_FILE);
+	joinpath(global_attributes_file, sizeof(global_attributes_file), TIN_DEFAULTS_DIR, ATTRIBUTES_FILE);
+	joinpath(global_config_file, sizeof(global_config_file), TIN_DEFAULTS_DIR, CONFIG_FILE);
 #else
 	/* read_site_config() might have changed the value of libdir */
-	joinpath(global_attributes_file, libdir, ATTRIBUTES_FILE);
-	joinpath(global_config_file, libdir, CONFIG_FILE);
+	joinpath(global_attributes_file, sizeof(global_attributes_file), libdir, ATTRIBUTES_FILE);
+	joinpath(global_config_file, sizeof(global_config_file), libdir, CONFIG_FILE);
 #endif /* TIN_DEFAULTS_DIR */
 
-	joinpath(rcdir, homedir, RCDIR);
+	joinpath(rcdir, sizeof(rcdir), homedir, RCDIR);
 	if (stat(rcdir, &sb) == -1) {
 		created_rcdir = TRUE;
 		my_mkdir(rcdir, (mode_t) (S_IRWXU));
@@ -755,45 +744,45 @@ init_selfinfo(
 	strcpy(tinrc.printer, DEFAULT_PRINTER);
 #endif /* !DISABLE_PRINTING */
 	strcpy(tinrc.inews_prog, PATH_INEWS);
-	joinpath(article_name, homedir, TIN_ARTICLE_NAME);
+	joinpath(article_name, sizeof(article_name), homedir, TIN_ARTICLE_NAME);
 #ifdef APPEND_PID
 	snprintf(article_name + strlen(article_name), sizeof(article_name) - strlen(article_name), ".%d", (int) process_id);
 #endif /* APPEND_PID */
-	joinpath(dead_article, homedir, "dead.article");
-	joinpath(dead_articles, homedir, "dead.articles");
-	joinpath(tinrc.maildir, homedir, DEFAULT_MAILDIR);
-	joinpath(tinrc.savedir, homedir, DEFAULT_SAVEDIR);
-	joinpath(tinrc.sigfile, homedir, ".Sig");
-	joinpath(default_signature, homedir, ".signature");
+	joinpath(dead_article, sizeof(dead_article), homedir, "dead.article");
+	joinpath(dead_articles, sizeof(dead_articles), homedir, "dead.articles");
+	joinpath(tinrc.maildir, sizeof(tinrc.maildir), homedir, DEFAULT_MAILDIR);
+	joinpath(tinrc.savedir, sizeof(tinrc.savedir), homedir, DEFAULT_SAVEDIR);
+	joinpath(tinrc.sigfile, sizeof(tinrc.sigfile), homedir, ".Sig");
+	joinpath(default_signature, sizeof(default_signature), homedir, ".signature");
 
 	if (!index_newsdir[0])
-		joinpath(index_newsdir, get_val("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
-	joinpath(index_maildir, get_val("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
+		joinpath(index_newsdir, sizeof(index_newsdir), get_val("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
+	joinpath(index_maildir, sizeof(index_maildir), get_val("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
 	if (stat(index_maildir, &sb) == -1)
 		my_mkdir(index_maildir, (mode_t) S_IRWXU);
-	joinpath(index_savedir, get_val("TIN_INDEX_SAVEDIR", rcdir), INDEX_SAVEDIR);
+	joinpath(index_savedir, sizeof(index_savedir), get_val("TIN_INDEX_SAVEDIR", rcdir), INDEX_SAVEDIR);
 	if (stat(index_savedir, &sb) == -1)
 		my_mkdir(index_savedir, (mode_t) S_IRWXU);
-	joinpath(local_attributes_file, rcdir, ATTRIBUTES_FILE);
-	joinpath(local_config_file, rcdir, CONFIG_FILE);
-	joinpath(filter_file, rcdir, FILTER_FILE);
-	joinpath(local_input_history_file, rcdir, INPUT_HISTORY_FILE);
-	joinpath(local_newsrctable_file, rcdir, NEWSRCTABLE_FILE);
+	joinpath(local_attributes_file, sizeof(local_attributes_file), rcdir, ATTRIBUTES_FILE);
+	joinpath(local_config_file, sizeof(local_config_file), rcdir, CONFIG_FILE);
+	joinpath(filter_file, sizeof(filter_file), rcdir, FILTER_FILE);
+	joinpath(local_input_history_file, sizeof(local_input_history_file), rcdir, INPUT_HISTORY_FILE);
+	joinpath(local_newsrctable_file, sizeof(local_newsrctable_file), rcdir, NEWSRCTABLE_FILE);
 #ifdef HAVE_MH_MAIL_HANDLING
-	joinpath(mail_active_file, rcdir, ACTIVE_MAIL_FILE);
+	joinpath(mail_active_file, sizeof(mail_active_file), rcdir, ACTIVE_MAIL_FILE);
 #endif /* HAVE_MH_MAIL_HANDLING */
-	joinpath(mailbox, DEFAULT_MAILBOX, userid);
+	joinpath(mailbox, sizeof(mailbox), DEFAULT_MAILBOX, userid);
 #ifdef HAVE_MH_MAIL_HANDLING
-	joinpath(mailgroups_file, rcdir, MAILGROUPS_FILE);
+	joinpath(mailgroups_file, sizeof(mailgroups_file), rcdir, MAILGROUPS_FILE);
 #endif /* HAVE_MH_MAIL_HANDLING */
-	joinpath(newsrc, homedir, NEWSRC_FILE);
-	joinpath(newnewsrc, homedir, NEWNEWSRC_FILE);
+	joinpath(newsrc, sizeof(newsrc), homedir, NEWSRC_FILE);
+	joinpath(newnewsrc, sizeof(newnewsrc), homedir, NEWNEWSRC_FILE);
 #ifdef APPEND_PID
 	snprintf(newnewsrc + strlen(newnewsrc), sizeof(newnewsrc) - strlen(newnewsrc), ".%d", (int) process_id);
 #endif /* APPEND_PID */
-	joinpath(posted_info_file, rcdir, POSTED_FILE);
-	joinpath(postponed_articles_file, rcdir, POSTPONED_FILE);
-	joinpath(save_active_file, rcdir, ACTIVE_SAVE_FILE);
+	joinpath(posted_info_file, sizeof(posted_info_file), rcdir, POSTED_FILE);
+	joinpath(postponed_articles_file, sizeof(postponed_articles_file), rcdir, POSTPONED_FILE);
+	joinpath(save_active_file, sizeof(save_active_file), rcdir, ACTIVE_SAVE_FILE);
 
 	snprintf(lock_file, sizeof(lock_file), INDEX_LOCK, TMPDIR, userid);
 
@@ -809,9 +798,6 @@ init_selfinfo(
 		}
 	}
 
-	if (stat(local_attributes_file, &sb) == -1)
-		write_attributes_file(local_attributes_file);
-
 	init_postinfo();
 	snprintf(txt_help_bug_report, sizeof(txt_help_bug_report), _(txt_help_bug), bug_addr);
 
@@ -835,7 +821,7 @@ read_site_config(
 	void)
 {
 	FILE *fp = (FILE *) 0;
-	char buf[LEN];
+	char buf[LEN], filename[PATH_LEN];
 	static const char *tin_defaults[] = { TIN_DEFAULTS };
 	int i = 0;
 
@@ -843,8 +829,8 @@ read_site_config(
 	 * try to find tin.defaults in some different locations
 	 */
 	while (tin_defaults[i] != NULL) {
-		joinpath(buf, tin_defaults[i++], "tin.defaults");
-		if ((fp = fopen(buf, "r")) != NULL)
+		joinpath(filename, sizeof(filename), tin_defaults[i++], "tin.defaults");
+		if ((fp = fopen(filename, "r")) != NULL)
 			break;
 	}
 
@@ -873,6 +859,8 @@ read_site_config(
 			continue;
 		if (match_string(buf, "subscriptionsfile=", subscriptions_file, sizeof(subscriptions_file)))
 			continue;
+		if (match_string(buf, "overviewfmtfile=", overviewfmt_file, sizeof(overviewfmt_file)))
+			continue;
 		if (match_string(buf, "domainname=", domain_name, sizeof(domain_name)))
 			continue;
 		if (match_string(buf, "inewsdir=", inewsdir, sizeof(inewsdir)))
@@ -987,7 +975,7 @@ postinit_regexp(
 
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-static t_bool
+t_bool
 utf8_pcre(
 	void)
 {
