@@ -3,10 +3,10 @@
  *  Module    : mail.c
  *  Author    : I. Lea
  *  Created   : 1992-10-02
- *  Updated   : 2006-02-15
+ *  Updated   : 2008-01-21
  *  Notes     : Mail handling routines for creating pseudo newsgroups
  *
- * Copyright (c) 1992-2007 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2009 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,11 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-
+#ifdef NNTP_ABLE
+#	ifndef TNNTP_H
+#		include "tnntp.h"
+#	endif /* !TNNTP_H */
+#endif  /* NNTP_ABLE */
 /*
  * local prototypes
  */
@@ -103,7 +107,7 @@ read_mail_active_file(
 			my_fputc('\n', stderr);
 
 		if (!created_rcdir) /* no error on first start */
-			error_message(_(txt_cannot_open), mail_active_file);
+			error_message(2, _(txt_cannot_open), mail_active_file);
 		/*
 		 * TODO: do an autoscan of maildir, create & reopen?
 		 */
@@ -118,7 +122,7 @@ read_mail_active_file(
 		/*
 		 * Update mailgroup info
 		 */
-		if ((ptr = group_find(buf)) != NULL) {
+		if ((ptr = group_find(buf, FALSE)) != NULL) {
 			if (strcmp(ptr->spooldir, my_spooldir) != 0) {
 				free(ptr->spooldir);
 				strfpath(my_spooldir, buf2, sizeof(buf2) - 1, ptr);
@@ -184,7 +188,7 @@ write_mail_active_file(
 	file_tmp = get_tmpfilename(mail_active_file);
 
 	if (!backup_file(mail_active_file, file_tmp)) {
-		error_message(_(txt_filesystem_full_backup), mail_active_file);
+		error_message(2, _(txt_filesystem_full_backup), mail_active_file);
 		/* free memory for tmp-filename */
 		free(file_tmp);
 		return;
@@ -196,13 +200,13 @@ write_mail_active_file(
 		for_each_group(i) {
 			group = &active[i];
 			if (group->type == GROUP_TYPE_MAIL) {
-				make_base_group_path(group->spooldir, group->name, group_path);
+				make_base_group_path(group->spooldir, group->name, group_path, sizeof(group_path));
 				find_art_max_min(group_path, &group->xmax, &group->xmin);
 				print_group_line(fp, group->name, group->xmax, group->xmin, group->spooldir);
 			}
 		}
 		if (ferror(fp) || fclose(fp)) {
-			error_message(_(txt_filesystem_full), mail_active_file);
+			error_message(2, _(txt_filesystem_full), mail_active_file);
 			rename(file_tmp, mail_active_file);
 		} else
 			unlink(file_tmp);
@@ -239,9 +243,14 @@ read_mailgroups_file(
 
 
 /*
- * If reading via NNTP the newsgroups file will be saved to ~/.tin/newsgroups
- * so that any subsequent rereads on the active file will not have to waste
- * net bandwidth and the local copy of the newsgroups file can be accessed.
+ * If reading via NNTP the newsgroups file will be saved to
+ * ~/.tin/$NNTPSERVER/newsgroups so that any subsequent rereads on the
+ * active file will not have to waste net bandwidth and the local copy
+ * of the newsgroups file can be accessed.
+ *
+ * in the newsrc_active case (-n cmd-line switch) we use "LIST NEWSGROUPS grp"
+ * instead of "LIST NEWSGROUPS" if we have just a few groups in the newsrc,
+ * due to pipelining the code is a bit complex.
  */
 static FILE *
 open_newsgroups_fp(
@@ -249,6 +258,7 @@ open_newsgroups_fp(
 {
 #ifdef NNTP_ABLE
 	FILE *result;
+	static int no_more_wildmat = 0;
 
 	if (read_news_via_nntp && !read_saved_news) {
 		if (read_local_newsgroups_file) {
@@ -258,7 +268,8 @@ open_newsgroups_fp(
 				if (buf.st_size > 0) {
 					if ((result = fopen(local_newsgroups_file, "r")) != NULL) {
 #	ifdef DEBUG
-						debug_nntp("open_newsgroups_fp", "Using local copy of newsgroups file");
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "open_newsgroups_fp Using local copy of newsgroups file");
 #	endif /* DEBUG */
 						return result;
 					}
@@ -266,15 +277,95 @@ open_newsgroups_fp(
 					unlink(local_newsgroups_file);
 			}
 		}
-#	if 0 /* TODO: */
-		if (list_newsgroups_wildmat_supported && newsrc_active
-		    && !list_active && num_active < some_useful_limit) {
-			for_each_group(i) {
-				snprintf(buff, sizeof(buff), "LIST NEWSGROUPS %s", active[i].name);
-				nntp_command(buff, OK_LIST, NULL, 0);
+		/*
+		 * TODO: test me, find a usefull limit,
+		 *       optimize more than n groups (e.g. 5) of the same
+		 *       subhierarchie to a wildmat?
+		 */
+		if (((nntp_caps.type == CAPABILITIES && nntp_caps.list_newsgroups) || nntp_caps.type != CAPABILITIES) && newsrc_active && !list_active && !no_more_wildmat && num_active < PIPELINE_LIMIT) {
+			char *ptr;
+			char buff[NNTP_STRLEN];
+			char line[NNTP_STRLEN];
+			char file[PATH_LEN];
+			char serverdir[PATH_LEN];
+			struct t_group *group;
+			int resp, i;
+
+			if (nntp_tcp_port != IPPORT_NNTP)
+				snprintf(file, sizeof(file), "%s:%d", nntp_server, nntp_tcp_port);
+			else
+				STRCPY(file, quote_space_to_dash(nntp_server));
+
+			joinpath(serverdir, sizeof(serverdir), rcdir, file);
+			joinpath(file, sizeof(file), serverdir, NEWSGROUPS_FILE".tmp");
+
+			if ((result = fopen(file, "w")) != NULL) {
+				for_each_group(i) {
+					if ((group = group_find(active[i].name, FALSE)) != NULL) {
+						if (group->type == GROUP_TYPE_NEWS) {
+							snprintf(buff, sizeof(buff), "LIST NEWSGROUPS %s", active[i].name);
+#		ifdef DISABLE_PIPELINING
+							if ((resp = new_nntp_command(buff, OK_GROUPS, line, sizeof(line))) != OK_GROUPS) {
+								no_more_wildmat = resp;
+								break;
+							}
+							while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
+#			ifdef DEBUG
+								if (debug & DEBUG_NNTP)
+									debug_print_file("NNTP", "<<< %s", ptr);
+#			endif /* DEBUG */
+								fprintf(result, "%s\n", str_trim(ptr));
+							}
+#		else
+							put_server(buff);
+#		endif /* DISABLE_PIPELINING */
+						}
+					}
+				}
+#		ifndef DISABLE_PIPELINING
+				for_each_group(i) {
+					/*
+					 * don't use get_respcode() as it will try to auth if we
+					 * see a 480 but that could fail as there might be
+					 * pending data
+					 */
+					if ((group = group_find(active[i].name, FALSE)) != NULL) {
+						if (group->type == GROUP_TYPE_NEWS) {
+							if ((resp = get_only_respcode(line, sizeof(line))) != OK_GROUPS) {
+								if (!no_more_wildmat)
+									no_more_wildmat = resp;
+								continue;
+							}
+							while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
+#			ifdef DEBUG
+								if (debug & DEBUG_NNTP)
+									debug_print_file("NNTP", "<<< %s", ptr);
+#			endif /* DEBUG */
+								fprintf(result, "%s\n", str_trim(ptr));
+							}
+						}
+					}
+				}
+				/* TODO: add 483 (RFC 3977) support */
+				if (no_more_wildmat == ERR_NOAUTH || no_more_wildmat == NEED_AUTHINFO) {
+					if (!authenticate(nntp_server, userid, FALSE)) {
+						error_message(2, _(txt_auth_failed), ERR_ACCESS);
+						tin_done(EXIT_FAILURE);
+					}
+				}
+#		endif /* !DISABLE_PIPELINING */
+				fclose(result);
+				result = fopen(file, "r");
+				unlink(file); /* unlink on close */
 			}
-		} else
-#	endif /* 0 */
+
+			if (result != NULL) {
+				if (!no_more_wildmat)
+					return result;
+				else /* AUTH request while pipeling or some error */
+					fclose(result);
+			}
+		}
 		return (nntp_command("LIST NEWSGROUPS", OK_GROUPS, NULL, 0));
 	} else
 #endif /* NNTP_ABLE */
@@ -363,7 +454,7 @@ read_groups_descriptions(
 		 * with the "-q" option.
 		 */
 		if ((fp_save != NULL) && read_news_via_nntp)
-			fprintf(fp_save, "%s\n", ptr);
+			fprintf(fp_save, "%s\n", str_trim(ptr));
 
 		if (!space) { /* initial malloc */
 			space = strlen(ptr) + 1;
@@ -383,11 +474,11 @@ read_groups_descriptions(
 		while (*p == '\t' || *p == ' ')
 			p++;
 
-		group = group_find(groupname);
+		group = group_find(groupname, FALSE);
 
 		if (group != NULL && group->description == NULL) {
 			char *r;
-			int r_len;
+			size_t r_len;
 
 			q = p;
 			while ((q = strchr(q, '\t')) != NULL)
@@ -501,12 +592,12 @@ grp_del_mail_arts(
 		 * at least for GROUP_TYPE_SAVE a wait is annoying - nuke the message?
 		 */
 		wait_message(0, (group->type == GROUP_TYPE_MAIL) ? _(txt_processing_mail_arts) : _(txt_processing_saved_arts));
-		make_base_group_path(group->spooldir, group->name, group_path);
+		make_base_group_path(group->spooldir, group->name, group_path, sizeof(group_path));
 		for_each_art(i) {
 			article = &arts[i];
 			if (article->delete_it) {
 				snprintf(artnum, sizeof(artnum), "%ld", article->artnum);
-				joinpath(article_filename, group_path, artnum);
+				joinpath(article_filename, sizeof(article_filename), group_path, artnum);
 				unlink(article_filename);
 				article->thread = ART_EXPIRED;
 #if 0 /* see comment below */
@@ -544,18 +635,18 @@ art_edit(
 	/*
 	 * Check if news / mail group
 	 */
-	if (group->type == GROUP_TYPE_NEWS)
+	if (group->type != GROUP_TYPE_MAIL)
 		return FALSE;
 
-	make_base_group_path(group->spooldir, group->name, temp_filename);
+	make_base_group_path(group->spooldir, group->name, temp_filename, sizeof(temp_filename));
 	snprintf(buf, sizeof(buf), "%ld", article->artnum);
-	joinpath(article_filename, temp_filename, buf);
+	joinpath(article_filename, sizeof(article_filename), temp_filename, buf);
 	snprintf(temp_filename, sizeof(temp_filename), "%s%d.art", TMPDIR, (int) process_id);
 
 	if (!backup_file(article_filename, temp_filename))
 		return FALSE;
 
-	if (!invoke_editor(temp_filename, 1)) {
+	if (!invoke_editor(temp_filename, 1, group)) {
 		unlink(temp_filename);
 		return FALSE;
 	} else
