@@ -3,10 +3,10 @@
  *  Module    : memory.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2006-03-11
+ *  Updated   : 2009-02-01
  *  Notes     :
  *
- * Copyright (c) 1991-2007 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,8 @@ int num_newnews = 0;
 int max_art;
 int max_save;
 int num_save = 0;
+int max_scope;
+int num_scope = -1;
 
 /*
  * Dynamic arrays
@@ -58,6 +60,7 @@ int num_save = 0;
 int *my_group;				/* .newsrc --> active[] */
 long *base;				/* base articles for each thread */
 struct t_group *active;			/* active newsgroups */
+struct t_scope *scopes = NULL;	/* attributes stores in .tin/attributes */
 struct t_newnews *newnews;		/* active file sizes on differnet servers */
 struct t_article *arts;			/* articles headers in current group */
 struct t_save *save;			/* sorts articles before saving them */
@@ -66,9 +69,11 @@ struct t_save *save;			/* sorts articles before saving them */
  * Local prototypes
  */
 static void free_active_arrays(void);
+static void free_attributes(struct t_attribute *attributes);
+static void free_scopes_arrays(void);
 static void free_newnews_array(void);
+static void free_if_not_default(char **attrib, char *deflt);
 static void free_input_history(void);
-static void free_global_arrays(void);
 
 
 /*
@@ -107,6 +112,12 @@ init_alloc(
 
 	save = my_malloc(sizeof(*save) * max_save);
 
+	/*
+	 * scope array
+	 */
+	max_scope = DEFAULT_SCOPE_NUM;
+	expand_scope();
+
 #ifndef USE_CURSES
 	screen = (struct t_screen *) 0;
 #endif /* !USE_CURSES */
@@ -147,6 +158,21 @@ expand_save(
 }
 
 
+void
+expand_scope(
+	void)
+{
+	if ((scopes == NULL) || (num_scope < 0)) {
+		if (scopes == NULL)
+			scopes = my_malloc(sizeof(*scopes) * max_scope);
+		num_scope = 0;
+	} else {
+		max_scope += max_scope >> 1;	/* increase by 50 % */
+		scopes = my_realloc(scopes, sizeof(*scopes) * max_scope);
+	}
+}
+
+
 void
 expand_newnews(
 	void)
@@ -188,6 +214,8 @@ void
 free_all_arrays(
 	void)
 {
+	int i;
+
 	hash_reclaim();
 
 #ifndef USE_CURSES
@@ -200,7 +228,7 @@ free_all_arrays(
 	FreeAndNull(arts);
 	free_filter_array(&glob_filter);
 	free_active_arrays();
-	free_global_arrays();
+	free_scopes_arrays();
 
 #ifdef HAVE_COLOR
 	FreeIfNeeded(quote_regex.re);
@@ -258,6 +286,12 @@ free_all_arrays(
 
 	FreeAndNull(nntp_caps.implementation);
 
+	if (ofmt) { /* ofmt might not be allocated yet on early abort */
+		for (i = 0; ofmt[i].name; i++)
+			free(ofmt[i].name);
+		free(ofmt);
+	}
+
 	tin_fgets(NULL, FALSE);
 }
 
@@ -296,7 +330,7 @@ free_art_array(
  * Use this only for attributes that have a fixed default of a static string
  * in tinrc
  */
-void
+static void
 free_if_not_default(
 	char **attrib,
 	char *deflt)
@@ -307,103 +341,72 @@ free_if_not_default(
 
 
 /*
- * TODO: fix the leaks in the global case
+ * Free memory of one attributes struct only
  */
-void
-free_attributes_array(
-	void)
+static void
+free_attributes(
+	struct t_attribute *attributes)
 {
-	int i;
-	struct t_group *group;
-
-	for_each_group(i) {
-		group = &active[i];
-		if (!group->bogus && group->attribute && !group->attribute->global) {
-			free_if_not_default(&group->attribute->maildir, tinrc.maildir);
-			free_if_not_default(&group->attribute->savedir, tinrc.savedir);
-
-			FreeAndNull(group->attribute->savefile);
-
-			free_if_not_default(&group->attribute->sigfile, tinrc.sigfile);
-			free_if_not_default(&group->attribute->organization, default_organization);
-
-			FreeAndNull(group->attribute->followup_to);
-
-			FreeAndNull(group->attribute->fcc);
-
-			FreeAndNull(group->attribute->mailing_list);
-			FreeAndNull(group->attribute->x_headers);
-			FreeAndNull(group->attribute->x_body);
-
-			free_if_not_default(&group->attribute->from, tinrc.mail_address);
-			free_if_not_default(&group->attribute->news_quote_format, tinrc.news_quote_format);
-			free_if_not_default(&group->attribute->quote_chars, tinrc.quote_chars);
-
-			FreeAndNull(group->attribute->mime_types_to_save);
-
+	free_if_not_default(&attributes->date_format, tinrc.date_format);
+	free_if_not_default(&attributes->editor_format, tinrc.editor_format);
+	FreeAndNull(attributes->fcc);
+	free_if_not_default(&attributes->from, tinrc.mail_address);
+	FreeAndNull(attributes->followup_to);
 #ifdef HAVE_ISPELL
-			FreeAndNull(group->attribute->ispell);
+	FreeAndNull(attributes->ispell);
 #endif /* HAVE_ISPELL */
-
-			FreeAndNull(group->attribute->quick_kill_scope);
-			FreeAndNull(group->attribute->quick_select_scope);
-
-#ifdef CHARSET_CONVERSION
-			FreeAndNull(group->attribute->undeclared_charset);
-#endif /* CHARSET_CONVERSION */
-
-			free(group->attribute);
-		}
-		group->attribute = (struct t_attribute *) 0;
+	free_if_not_default(&attributes->maildir, tinrc.maildir);
+	FreeAndNull(attributes->mailing_list);
+	FreeAndNull(attributes->mime_types_to_save);
+	free_if_not_default(&attributes->news_headers_to_display, tinrc.news_headers_to_display);
+	free_if_not_default(&attributes->news_headers_to_not_display, tinrc.news_headers_to_not_display);
+	if (attributes->headers_to_display) {
+		if (attributes->headers_to_display->header)
+			FreeIfNeeded(*attributes->headers_to_display->header);
+		FreeAndNull(attributes->headers_to_display->header);
+		free(attributes->headers_to_display);
+		attributes->headers_to_display = (struct t_newsheader *) 0;
 	}
-
-	/* free the global attributes array */
-	free_if_not_default(&glob_attributes.maildir, tinrc.maildir);
-	free_if_not_default(&glob_attributes.savedir, tinrc.savedir);
-
-	FreeAndNull(glob_attributes.savefile);
-
-	free_if_not_default(&glob_attributes.sigfile, tinrc.sigfile);
-	free_if_not_default(&glob_attributes.organization, default_organization);
-
-	FreeAndNull(glob_attributes.followup_to);
-
-	FreeAndNull(glob_attributes.fcc);
-
-	FreeAndNull(glob_attributes.mailing_list);
-	FreeAndNull(glob_attributes.x_headers);
-	FreeAndNull(glob_attributes.x_body);
-
-	free_if_not_default(&glob_attributes.from, tinrc.mail_address);
-	free_if_not_default(&glob_attributes.news_quote_format, tinrc.news_quote_format);
-	free_if_not_default(&glob_attributes.quote_chars, tinrc.quote_chars);
-
-	FreeAndNull(glob_attributes.mime_types_to_save);
-
-#ifdef HAVE_ISPELL
-	FreeAndNull(glob_attributes.ispell);
-#endif /* HAVE_ISPELL */
-
-	FreeAndNull(glob_attributes.quick_kill_scope);
-	FreeAndNull(glob_attributes.quick_select_scope);
-
-#ifdef CHARSET_CONVERSATION
-	FreeAndNull(glob_attributes.undeclared_charset);
-#endif /* CHARSET_CONVERSATION */
+	if (attributes->headers_to_not_display) {
+		if (attributes->headers_to_not_display->header)
+			FreeIfNeeded(*attributes->headers_to_not_display->header);
+		FreeAndNull(attributes->headers_to_not_display->header);
+		free(attributes->headers_to_not_display);
+		attributes->headers_to_not_display = (struct t_newsheader *) 0;
+	}
+	free_if_not_default(&attributes->news_quote_format, tinrc.news_quote_format);
+	free_if_not_default(&attributes->organization, default_organization);
+	FreeAndNull(attributes->quick_kill_scope);
+	FreeAndNull(attributes->quick_select_scope);
+	free_if_not_default(&attributes->quote_chars, tinrc.quote_chars);
+	free_if_not_default(&attributes->savedir, tinrc.savedir);
+	FreeAndNull(attributes->savefile);
+	free_if_not_default(&attributes->sigfile, tinrc.sigfile);
+#ifdef CHARSET_CONVERSION
+	FreeAndNull(attributes->undeclared_charset);
+#endif /* CHARSET_CONVERSION */
+	FreeAndNull(attributes->x_headers);
+	FreeAndNull(attributes->x_body);
 }
 
 
 static void
-free_global_arrays(
+free_scopes_arrays(
 	void)
 {
-	if (news_headers_to_display_array)
-		FreeIfNeeded(*news_headers_to_display_array);
-	FreeAndNull(news_headers_to_display_array);
+	struct t_scope *scope;
 
-	if (news_headers_to_not_display_array)
-		FreeIfNeeded(*news_headers_to_not_display_array);
-	FreeAndNull(news_headers_to_not_display_array);
+	while (num_scope > 0) {
+		scope = &scopes[--num_scope];
+		FreeAndNull(scope->scope);
+		free_attributes(scope->attribute);
+		free(scope->attribute);
+		scope->attribute = (struct t_attribute *) 0;
+		free(scope->state);
+		scope->state = (struct t_attribute_state *) 0;
+	}
+	FreeAndNull(scopes);
+	num_scope = -1;
 }
 
 
@@ -420,12 +423,15 @@ free_active_arrays(
 			FreeAndNull(active[i].name);
 			FreeAndNull(active[i].description);
 			FreeAndNull(active[i].aliasedto);
-			if (active[i].type == GROUP_TYPE_MAIL) {
+			if (active[i].type == GROUP_TYPE_MAIL || active[i].type == GROUP_TYPE_SAVE) {
 				FreeAndNull(active[i].spooldir);
 			}
 			FreeAndNull(active[i].newsrc.xbitmap);
+			if (active[i].attribute && !active[i].attribute->global) {
+				free(active[i].attribute);
+				active[i].attribute = (struct t_attribute *) 0;
+			}
 		}
-		free_attributes_array();
 		FreeAndNull(active);
 	}
 	num_active = -1;
@@ -488,7 +494,7 @@ my_malloc1(
 #endif /* DEBUG */
 
 	if ((p = malloc(size)) == NULL) {
-		error_message(txt_out_of_memory, tin_progname, size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, size, file, line);
 		giveup();
 	}
 	return p;
@@ -512,7 +518,7 @@ my_calloc1(
 #endif /* DEBUG */
 
 	if ((p = calloc(nmemb, size)) == NULL) {
-		error_message(txt_out_of_memory, tin_progname, nmemb * size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, nmemb * size, file, line);
 		giveup();
 	}
 	return p;
@@ -530,10 +536,17 @@ my_realloc1(
 	debug_print_malloc(FALSE, file, line, size);
 #endif /* DEBUG */
 
-	p = ((!p) ? (calloc(1, size)) : realloc(p, size));
+	if (!size) {
+		if (p)
+			free(p);
+
+		return NULL;
+	}
+
+	p = ((!p) ? (malloc(size)) : realloc(p, size));
 
 	if (p == NULL) {
-		error_message(txt_out_of_memory, tin_progname, size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, size, file, line);
 		giveup();
 	}
 	return p;
