@@ -3,10 +3,10 @@
  *  Module    : newsrc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2006-02-15
+ *  Updated   : 2008-11-28
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -109,7 +109,7 @@ read_newsrc(
 			line_count++;
 
 			if (sub == SUBSCRIBED) {
-				if ((i = my_group_add(grp)) >= 0) {
+				if ((i = my_group_add(grp, FALSE)) >= 0) {
 					if (!active[my_group[i]].bogus) {
 						active[my_group[i]].subscribed = SUB_BOOL(sub);
 						parse_bitmap_seq(&active[my_group[i]], seq);
@@ -159,7 +159,7 @@ write_newsrc_line(
 	 * are set to auto removal. Also check for bogus flag just in case
 	 * strip_bogus was changed since tin started
 	 */
-	group = group_find(line);
+	group = group_find(line, FALSE);
 
 	if (tinrc.strip_bogus == BOGUS_REMOVE) {
 		if (group == NULL || group->bogus) { /* group dosen't exist */
@@ -225,7 +225,7 @@ write_newsrc(
 		 * Don't rename if either fclose() fails or ferror() is set
 		 */
 		if (ferror(fp_op) || fclose(fp_op)) {
-			error_message(_(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 			unlink(newnewsrc);
 		} else
 			write_ok = TRUE;
@@ -234,7 +234,8 @@ write_newsrc(
 	fclose(fp_ip);
 
 	if (tot < 1) {
-		error_message(_(txt_newsrc_nogroups));
+		error_message(2, _(txt_newsrc_nogroups));
+		unlink(newnewsrc);
 		return 0L;		/* So we don't get prompted to try again */
 	}
 
@@ -262,7 +263,7 @@ create_newsrc(
 			fprintf(fp, "%s!\n", active[i].name);
 
 		if (ferror(fp) || fclose(fp)) {
-			error_message(_(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 			return FALSE;
 		}
 		return TRUE; /* newsrc created */
@@ -281,7 +282,11 @@ open_subscription_fp(
 	if (!read_saved_news) {
 #ifdef NNTP_ABLE
 		if (read_news_via_nntp) {
-			/* if (nntp_caps.type = 2 && !nntp_caps.list_subscriptions)
+			/*
+			 * RFC 3977 doesn't list SUBSCRIPTIONS, so we keep this check
+			 * disabled till this is fixed in the next RFC
+			 */
+			/* if (nntp_caps.type == CAPABILITIES && !nntp_caps.list_subscriptions)
 				return NULL;
 			else */
 				return (nntp_command("LIST SUBSCRIPTIONS", OK_GROUPS, NULL, 0));
@@ -325,7 +330,7 @@ auto_subscribe_groups(
 	/* TODO: test me! */
 	while ((ptr = tin_fgets(fp_subs, FALSE)) != NULL) {
 		if (ptr[0] != '#') {
-			if (group_find(ptr) != 0)
+			if (group_find(ptr, FALSE) != 0)
 				fprintf(fp_newsrc, "%s:\n", ptr);
 		}
 	}
@@ -333,7 +338,7 @@ auto_subscribe_groups(
 	/* We ignore user 'q'uits here. They will get them next time in any case */
 
 	if (ferror(fp_newsrc) || fclose(fp_newsrc))
-		error_message(_(txt_filesystem_full), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 
 	TIN_FCLOSE(fp_subs);
 }
@@ -358,17 +363,17 @@ backup_newsrc(
 	{
 		STRCPY(filebuf, quote_space_to_dash(nntp_server));
 	}
-	joinpath(dirbuf, rcdir, filebuf);
-	joinpath(filebuf, dirbuf, OLDNEWSRC_FILE);
+	joinpath(dirbuf, sizeof(dirbuf), rcdir, filebuf);
+	joinpath(filebuf, sizeof(filebuf), dirbuf, OLDNEWSRC_FILE);
 
 	if (-1 == stat(dirbuf, &statbuf)) {
 		if (-1 == my_mkdir(dirbuf, (mode_t) (S_IRWXU)))
 			/* Can't create directory: Fall back on Homedir */
-			joinpath(filebuf, homedir, OLDNEWSRC_FILE);
+			joinpath(filebuf, sizeof(filebuf), homedir, OLDNEWSRC_FILE);
 	}
 
 	if (!backup_file(newsrc, filebuf))
-		error_message(_(txt_filesystem_full_backup), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full_backup), NEWSRC_FILE);
 }
 
 
@@ -388,23 +393,23 @@ group_get_art_info(
 {
 	DIR *dir;
 	DIR_BUF *direntry;
-	char buf[NNTP_STRLEN];
 	long artnum;
 
 	if (read_news_via_nntp && grouptype == GROUP_TYPE_NEWS) {
 #ifdef NNTP_ABLE
 		char line[NNTP_STRLEN];
 
-		snprintf(buf, sizeof(buf), "GROUP %s", groupname);
+		snprintf(line, sizeof(line), "GROUP %s", groupname);
 #	ifdef DEBUG
-		debug_nntp("group_get_art_info", buf);
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "group_get_art_info %s", line);
 #	endif /* DEBUG */
-		put_server(buf);
+		put_server(line);
 
 		switch (get_respcode(line, sizeof(line))) {
 			case OK_GROUP:
 				if (sscanf(line, "%ld %ld %ld", art_count, art_min, art_max) != 3)
-					error_message(_(txt_error_invalid_response_to_group), line);
+					error_message(2, _(txt_error_invalid_response_to_group), line);
 				break;
 
 			case ERR_NOGROUP:
@@ -414,7 +419,7 @@ group_get_art_info(
 				return -ERR_NOGROUP;
 
 			case ERR_ACCESS:
-				error_message("%s%s", cCRLF, line);
+				error_message(2, "%s%s", cCRLF, line);
 				tin_done(NNTP_ERROR_EXIT);
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -422,7 +427,8 @@ group_get_art_info(
 
 			default:
 #	ifdef DEBUG
-				debug_nntp("NOT_OK", line);
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "NOT_OK %s", line);
 #	endif /* DEBUG */
 				return -1;
 		}
@@ -431,13 +437,14 @@ group_get_art_info(
 		return 0;
 #endif /* NNTP_ABLE */
 	} else {
+		char group_path[PATH_LEN];
 		*art_count = 0;
 		*art_min = 1;
 		*art_max = 0;
 
-		make_base_group_path(tin_spooldir, groupname, buf);
+		make_base_group_path(tin_spooldir, groupname, group_path, sizeof(group_path));
 
-		if ((dir = opendir(buf)) != NULL) {
+		if ((dir = opendir(group_path)) != NULL) {
 			while ((direntry = readdir(dir)) != NULL) {
 				artnum = atol(direntry->d_name); /* should be '\0' terminated... */
 				if (artnum >= 1) {
@@ -473,8 +480,10 @@ get_subscribe_info(
 
 	if (grp->newsrc.num_unread > grp->count) {
 #ifdef DEBUG
-		my_printf(cCRLF "Unread WRONG %s unread=[%ld] count=[%ld]", grp->name, grp->newsrc.num_unread, grp->count);
-		my_flush();
+		if (debug & DEBUG_NEWSRC) { /* TODO: is this the right debug-level? */
+			my_printf(cCRLF "Unread WRONG %s unread=[%ld] count=[%ld]", grp->name, grp->newsrc.num_unread, grp->count);
+			my_flush();
+		}
 #endif /* DEBUG */
 		grp->newsrc.num_unread = grp->count;
 	}
@@ -482,8 +491,10 @@ get_subscribe_info(
 	if (grp->xmin != oldmin || grp->xmax != oldmax) {
 		expand_bitmap(grp, 0);
 #ifdef DEBUG
-		my_printf(cCRLF "Min/Max DIFF %s old=[%ld-%ld] new=[%ld-%ld]", grp->name, oldmin, oldmax, grp->xmin, grp->xmax);
-		my_flush();
+		if (debug & DEBUG_NEWSRC) { /* TODO: is this the right debug-level? */
+			my_printf(cCRLF "Min/Max DIFF %s old=[%ld-%ld] new=[%ld-%ld]", grp->name, oldmin, oldmax, grp->xmin, grp->xmax);
+			my_flush();
+		}
 #endif /* DEBUG */
 	}
 }
@@ -518,18 +529,19 @@ subscribe(
 
 	if ((fp = fopen(newsrc, "r")) != NULL) {
 		while ((line = tin_fgets(fp, FALSE)) != NULL) {
-			seq = parse_newsrc_line(line, &sub);
-			if (STRCMPEQ(line, group->name)) {
-				fprintf(newfp, "%s%c %s\n", line, sub_state, seq);
-				group->subscribed = SUB_BOOL(sub_state);
+			if ((seq = parse_newsrc_line(line, &sub))) {
+				if (STRCMPEQ(line, group->name)) {
+					fprintf(newfp, "%s%c %s\n", line, sub_state, seq);
+					group->subscribed = SUB_BOOL(sub_state);
 
-				/* If previously subscribed to in .newsrc, load up any existing information */
-				if (sub_state == SUBSCRIBED)
-					parse_bitmap_seq(group, seq);
+					/* If previously subscribed to in .newsrc, load up any existing information */
+					if (sub_state == SUBSCRIBED)
+						parse_bitmap_seq(group, seq);
 
-				found = TRUE;
-			} else
-				fprintf(newfp, "%s%c %s\n", line, sub, seq);
+					found = TRUE;
+				} else
+					fprintf(newfp, "%s%c %s\n", line, sub, seq);
+			}
 		}
 
 		fclose(fp);
@@ -550,7 +562,7 @@ subscribe(
 	}
 
 	if (ferror(newfp) || fclose(newfp)) {
-		error_message(_(txt_filesystem_full), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 		unlink(newnewsrc);
 	} else
 		rename_file(newnewsrc, newsrc);
@@ -579,7 +591,7 @@ reset_newsrc(
 			fclose(fp);
 		}
 		if (ferror(newfp) || fclose(newfp)) {
-			error_message(_(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 			unlink(newnewsrc);
 		} else
 			rename_file(newnewsrc, newsrc);
@@ -612,17 +624,16 @@ delete_group(
 
 		if ((fp = fopen(newsrc, "r")) != NULL) {
 			while ((line = tin_fgets(fp, FALSE)) != NULL) {
-				seq = parse_newsrc_line(line, &sub);
-
-				if (!STRCMPEQ(line, group))
-					fprintf(newfp, "%s%c %s\n", line, sub, seq);
-
+				if ((seq = parse_newsrc_line(line, &sub))) {
+					if (!STRCMPEQ(line, group))
+						fprintf(newfp, "%s%c %s\n", line, sub, seq);
+				}
 			}
 			fclose(fp);
 		}
 
 		if (ferror(newfp) || fclose(newfp)) {
-			error_message(_(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 			unlink(newnewsrc);
 		} else
 			rename_file(newnewsrc, newsrc);
@@ -643,9 +654,10 @@ grp_mark_read(
 {
 	int i;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("c/C command");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_comment("c/C command");
+#endif /* DEBUG */
 
 	if (art != NULL) {
 		for_each_art(i)
@@ -668,9 +680,10 @@ grp_mark_unread(
 	int bitlength;
 	t_bitmap *newbitmap = (t_bitmap *)0;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Z command");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_comment("Z command");
+#endif /* DEBUG */
 
 	group_get_art_info(group->spooldir, group->name, group->type, &group->count, &group->xmax, &group->xmin);
 
@@ -692,9 +705,10 @@ grp_mark_unread(
 	if (bitlength)
 		NSETRNG1(group->newsrc.xbitmap, 0L, bitlength - 1L);
 
-#ifdef DEBUG_NEWSRC
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_bitmap(group, NULL);
+#endif /* DEBUG */
 }
 
 
@@ -705,9 +719,10 @@ thd_mark_read(
 {
 	int i;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Mark thread read K command");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_comment("Mark thread read K command");
+#endif /* DEBUG */
 
 	for (i = (int) thread; i >= 0; i = arts[i].thread)
 		art_mark(group, &arts[i], ART_READ);
@@ -721,9 +736,10 @@ thd_mark_unread(
 {
 	int i;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Mark thread unread Z command");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_comment("Mark thread unread Z command");
+#endif /* DEBUG */
 
 	for (i = (int) thread; i >= 0; i = arts[i].thread)
 		art_mark(group, &arts[i], ART_WILL_RETURN);
@@ -752,15 +768,15 @@ parse_bitmap_seq(
 	while (ptr && *ptr && (*ptr < '0' || *ptr > '9'))
 		ptr++;
 
-#ifdef DEBUG_NEWSRC
-	{
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
 		char buf[NEWSRC_LINE];
 
 		snprintf(buf, sizeof(buf), "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE - strlen(group->name) - 20), ptr);
 		debug_print_comment(buf);
 		debug_print_bitmap(group, NULL);
 	}
-#endif /* DEBUG_NEWSRC */
+#endif /* DEBUG */
 
 	if (ptr) {
 		gotseq = TRUE;
@@ -828,9 +844,10 @@ wait_message(2, "BITMAP Grp=[%s] MinMax=[%ld-%ld] Len=[%ld]\n",
 		sum = (int) ((group->count >= 0) ? group->count : ((group->newsrc.xmax - group->newsrc.xmin) + 1));
 
 	group->newsrc.num_unread = sum;
-#ifdef DEBUG_NEWSRC
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_bitmap(group, NULL);
+#endif /* DEBUG */
 }
 
 
@@ -1027,10 +1044,12 @@ print_bitmap_seq(
 	long i, last;
 	t_bool flag = FALSE;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("print_bitmap_seq()");
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
+		debug_print_comment("print_bitmap_seq()");
+		debug_print_bitmap(group, NULL);
+	}
+#endif /* DEBUG */
 
 	if (group->count == 0 || group->xmin > group->xmax) {
 		if (group->newsrc.xmax > 1)
@@ -1038,9 +1057,10 @@ print_bitmap_seq(
 
 		fprintf(fp, "\n");
 		fflush(fp);
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("print_bitmap_seq(): group->count == 0");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("print_bitmap_seq(): group->count == 0");
+#endif /* DEBUG */
 		return;
 	}
 
@@ -1082,9 +1102,10 @@ print_bitmap_seq(
 		if (group->newsrc.xmin > 2)
 			fprintf(fp, "-%ld", group->newsrc.xmin - 1);
 
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("print_bitmap_seq(): !flag && group->newsrc.xmin > 1");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("print_bitmap_seq(): !flag && group->newsrc.xmin > 1");
+#endif /* DEBUG */
 	}
 
 	fprintf(fp, "\n");
@@ -1104,9 +1125,9 @@ pos_group_in_newsrc(
 	FILE *fp_sub = NULL, *fp_unsub = NULL;
 	char *newsgroup = NULL;
 	char *line;
-	char buf[HEADER_LEN];
-	char sub[HEADER_LEN];
-	char unsub[HEADER_LEN];
+	char filename[PATH_LEN];
+	char sub[PATH_LEN];
+	char unsub[PATH_LEN];
 	int subscribed_pos = 1;
 	size_t group_len;
 	t_bool found = FALSE;
@@ -1131,11 +1152,11 @@ pos_group_in_newsrc(
 	if (newsrc_mode)
 		fchmod(fileno(fp_out), newsrc_mode);
 
-	joinpath(buf, TMPDIR, ".subrc");
-	snprintf(sub, sizeof(sub), "%s.%d", buf, (int) process_id);
+	joinpath(filename, sizeof(filename), TMPDIR, ".subrc");
+	snprintf(sub, sizeof(sub), "%s.%d", filename, (int) process_id);
 
-	joinpath(buf, TMPDIR, ".unsubrc");
-	snprintf(unsub, sizeof(unsub), "%s.%d", buf, (int) process_id);
+	joinpath(filename, sizeof(filename), TMPDIR, ".unsubrc");
+	snprintf(unsub, sizeof(unsub), "%s.%d", filename, (int) process_id);
 
 	if ((fp_sub = fopen(sub, "w")) == NULL)
 		goto rewrite_group_done;
@@ -1168,7 +1189,7 @@ pos_group_in_newsrc(
 	}
 
 	if (ferror(fp_sub) || fclose(fp_sub) || ferror(fp_unsub) || fclose(fp_unsub)) {
-		error_message(_(txt_filesystem_full), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 		fp_sub = fp_unsub = NULL;
 		goto rewrite_group_done;
 	}
@@ -1226,7 +1247,7 @@ pos_group_in_newsrc(
 	 * Try and cleanly close out the newnewsrc file
 	 */
 	if (ferror(fp_out) || fclose(fp_out))
-		error_message(_(txt_filesystem_full), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 	else {
 		if (repositioned) {
 			rename_file(newnewsrc, newsrc);
@@ -1367,18 +1388,20 @@ expand_bitmap(
 		bitlen = 0;
 		FreeIfNeeded(group->newsrc.xbitmap);
 		group->newsrc.xbitmap = (t_bitmap *) 0;
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("expand_bitmap: group->newsrc.bitlen == 0");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("expand_bitmap: group->newsrc.bitlen == 0");
+#endif /* DEBUG */
 	} else if (group->newsrc.xbitmap == NULL) {
 		group->newsrc.xbitmap = my_malloc(BITS_TO_BYTES(bitlen));
 		if (group->newsrc.xmin > first)
 			NSETRNG0(group->newsrc.xbitmap, 0L, group->newsrc.xmin - first - 1L);
 		if (bitlen > group->newsrc.xmin - first)
 			NSETRNG1(group->newsrc.xbitmap, group->newsrc.xmin - first, bitlen - 1);
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("expand_bitmap: group->newsrc.xbitmap == NULL");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("expand_bitmap: group->newsrc.xbitmap == NULL");
+#endif /* DEBUG */
 	} else if (need_full_copy) {
 		t_bitmap *newbitmap = my_malloc(BITS_TO_BYTES(bitlen));
 
@@ -1417,9 +1440,10 @@ expand_bitmap(
 
 		free(group->newsrc.xbitmap);
 		group->newsrc.xbitmap = newbitmap;
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1 and need full copy");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1 and need full copy");
+#endif /* DEBUG */
 	} else if (max != group->newsrc.xmax || first != group->newsrc.xmin) {
 		t_bitmap *newbitmap;
 		newbitmap = my_malloc(BITS_TO_BYTES(bitlen));
@@ -1456,9 +1480,10 @@ expand_bitmap(
 
 		free(group->newsrc.xbitmap);
 		group->newsrc.xbitmap = newbitmap;
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1");
+#endif /* DEBUG */
 	}
 	group->newsrc.xmin = first;
 	if (group->newsrc.xmax < max)
@@ -1483,9 +1508,10 @@ art_mark(
 			if (group != NULL) {
 				if (art->artnum >= group->newsrc.xmin && art->artnum <= group->newsrc.xmax)
 					NSET0(group->newsrc.xbitmap, art->artnum - group->newsrc.xmin);
-#ifdef DEBUG_NEWSRC
-				debug_print_bitmap(group, art);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+				if (debug & DEBUG_NEWSRC)
+					debug_print_bitmap(group, art);
+#endif /* DEBUG */
 			}
 			if ((art->status == ART_UNREAD) || (art->status == ART_WILL_RETURN)) {
 				art_mark_xref_read(art);
@@ -1510,9 +1536,10 @@ art_mark(
 					expand_bitmap(group, art->artnum);
 				else {
 					NSET1(group->newsrc.xbitmap, art->artnum - group->newsrc.xmin);
-#ifdef DEBUG_NEWSRC
-					debug_print_bitmap(group, art);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+					if (debug & DEBUG_NEWSRC)
+						debug_print_bitmap(group, art);
+#endif /* DEBUG */
 				}
 			}
 			break;
@@ -1542,144 +1569,3 @@ set_default_bitmap(
 		group->newsrc.xmax = group->newsrc.xmin - 1;
 	}
 }
-
-
-/* TEST harness */
-#ifdef DEBUG_NEWSRC
-#	if 0
-static void set_bitmap_range_read(struct t_newsrc *my_newsrc, long beg, long end);
-static void set_bitmap_range_unread(struct t_newsrc *my_newsrc, long beg, long end);
-
-void
-newsrc_test_harness(
-	void)
-{
-	FILE *fp = NULL;
-	char seq[20000];
-	int i;
-	int retry = 10; /* max. retrys */
-	long rng_min, rng_max;
-	struct t_group group;
-
-	/*
-	 * those env.-vars are not documented:
-	 * $TIN_RNG_MIN, $TIN_RNG_MAX, $TIN_MIN, $TIN_MAX, $TIN_COUNT
-	 * $TIN_SEQ
-	 */
-	rng_min = atoi(get_val("TIN_RNG_MIN", "1"));
-	rng_max = atoi(get_val("TIN_RNG_MAX", "1"));
-
-	group.name = my_strdup("test.newsrc");
-	group.xmin = atoi(get_val("TIN_MIN", "1"));
-	group.xmax = atoi(get_val("TIN_MAX", "0"));
-	group.count = atoi(get_val("TIN_COUNT", "-1"));
-	group.type = GROUP_TYPE_NEWS;
-	group.subscribed = TRUE;
-	group.newsrc.xbitmap = (t_bitmap *) 0;
-	set_default_bitmap(&group);
-
-	strcpy(seq, get_val("TIN_SEQ", ""));
-
-	fprintf(stderr, "\nENV Min=[%ld] Max=[%ld] Rng=[%ld-%ld] Count=[%ld] Seq=[%s]\n",
-		group.xmin, group.xmax, rng_min, rng_max, group.count, seq);
-
-	for (i = 0; i < 3; i++) {
-		if (group.newsrc.xbitmap != NULL) {
-			free(group.newsrc.xbitmap);
-			group.newsrc.xbitmap = (t_bitmap *) 0;
-			group.newsrc.xbitlen = 0;
-			group.newsrc.num_unread = 0;
-			group.newsrc.xmin = 1;
-			group.newsrc.xmax = 0;
-		}
-		while (retry) {
-		/* FIXME - this is secure now, but doesn't write any debug output */
-		/* (it didn't before too) */
-			if ((fp = tmpfile()) != NULL) {
-				fprintf(stderr, "\n%d. PARSE Seq=[%s]\n", i + 1, seq);
-				parse_bitmap_seq(&group, seq);
-				debug_print_newsrc(&group.newsrc, stderr);
-				print_bitmap_seq(fp, &group);
-				fprintf(stderr, "   PRINT Seq=");
-				print_bitmap_seq(stderr, &group);
-				rewind(fp);
-				break;
-			} else
-				retry--;
-		}
-
-		debug_print_newsrc(&group.newsrc, stderr);
-
-		if (!retry || !fp)
-			error_message(_(txt_cannot_create_uniq_name));
-		else {
-			fgets(seq, (int) sizeof(seq), fp);
-			seq[strlen(seq) - 1] = '\0';
-			fclose(fp);
-		}
-	}
-	set_bitmap_range_read(&group.newsrc, rng_min, rng_max);
-	debug_print_newsrc(&group.newsrc, stderr);
-
-	set_bitmap_range_unread(&group.newsrc, rng_min, rng_max);
-	debug_print_newsrc(&group.newsrc, stderr);
-
-	if (group.newsrc.xbitmap != NULL)
-		NSETBLK0(group.newsrc.xbitmap, group.newsrc.xbitlen);
-	debug_print_newsrc(&group.newsrc, stderr);
-
-	if (group.newsrc.xbitmap != NULL)
-		NSETBLK1(group.newsrc.xbitmap, group.newsrc.xbitlen);
-	debug_print_newsrc(&group.newsrc, stderr);
-	my_printf("\n");
-}
-
-
-static void
-set_bitmap_range_read(
-	struct t_newsrc *my_newsrc,
-	long beg,
-	long end)
-{
-	long length, offset;
-
-	if (beg >= my_newsrc->xmin && end <= my_newsrc->xmax) {
-		offset = beg - my_newsrc->xmin;
-		length = end - my_newsrc->xmin;
-
-fprintf(stderr, "\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n",
-my_newsrc->xmin, my_newsrc->xmax, beg, end, offset, length);
-
-		if (beg == end) {
-			NSET0(my_newsrc->xbitmap, offset);
-		} else {
-			NSETRNG0(my_newsrc->xbitmap, offset, length);
-		}
-	}
-}
-
-
-static void
-set_bitmap_range_unread(
-	struct t_newsrc *my_newsrc,
-	long beg,
-	long end)
-{
-	long length, offset;
-
-	if (beg >= my_newsrc->xmin && end <= my_newsrc->xmax) {
-		offset = beg - my_newsrc->xmin;
-		length = end - my_newsrc->xmin;
-
-fprintf(stderr, "\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n",
-my_newsrc->xmin, my_newsrc->xmax, beg, end, offset, length);
-
-		if (beg == end) {
-			NSET1(my_newsrc->xbitmap, offset);
-		} else {
-			NSETRNG1(my_newsrc->xbitmap, offset, length);
-		}
-	}
-}
-#	endif /* 0 */
-#endif /* DEBUG_NEWSRC */
