@@ -3,7 +3,7 @@
  *  Module    : nntplib.c
  *  Author    : S. Barber & I. Lea
  *  Created   : 1991-01-12
- *  Updated   : 2007-01-09
+ *  Updated   : 2009-01-15
  *  Notes     : NNTP client routines taken from clientlib.c 1.5.11 (1991-02-10)
  *  Copyright : (c) Copyright 1991-99 by Stan Barber & Iain Lea
  *              Permission is hereby granted to copy, reproduce, redistribute
@@ -32,8 +32,12 @@ char *nntp_server = NULL;
 	t_bool can_post = TRUE;
 #endif /* NO_POSTING */
 
-/* Flag to show whether tin did reconnect in last get_server() */
-t_bool reconnected_in_last_get_server = FALSE;
+#ifdef NNTP_ABLE
+	/* Flag to show whether tin did reconnect in last get_server() */
+	t_bool reconnected_in_last_get_server = FALSE;
+	/* Flag used in LIST ACVTIVE loop */
+	t_bool did_reconnect = FALSE;
+#endif /* NNTP_ABLE */
 
 static TCP *nntp_rd_fp = NULL;
 static TCP *nntp_wr_fp = NULL;
@@ -42,10 +46,7 @@ static TCP *nntp_wr_fp = NULL;
 	/* Copy of last NNTP command sent, so we can retry it if needed */
 	static char last_put[NNTP_STRLEN];
 	static constext *xover_cmds = "XOVER";
-#	if 0 /* currently not used */
 	static constext *xhdr_cmds = "XHDR";
-#	endif /* 0 */
-	enum extension_type { NO, LIST_EXTENSIONS, CAPABILITIES };
 	/* Set so we don't reconnect just to QUIT */
 	static t_bool quitting = FALSE;
 #endif /* NNTP_ABLE */
@@ -55,9 +56,8 @@ static TCP *nntp_wr_fp = NULL;
  */
 #ifdef NNTP_ABLE
 	static int mode_reader(t_bool *sec);
-	static int new_nntp_command(const char *command, int success, char *message, size_t mlen);
 	static int reconnect(int retry);
-	static int server_init(char *machine, const char *cservice, int port, char *text, size_t mlen);
+	static int server_init(char *machine, const char *cservice, unsigned short port, char *text, size_t mlen);
 	static int check_extensions(t_bool *sec);
 	static void close_server(void);
 	static void list_motd(void);
@@ -134,7 +134,7 @@ getserverbyfile(
 
 #ifdef NNTP_ABLE
 	if (cmdline_nntpserver[0] != '\0') {
-		get_nntpserver(buf, cmdline_nntpserver);
+		get_nntpserver(buf, sizeof(buf), cmdline_nntpserver);
 #	ifdef HAVE_SETENV
 		setenv("NNTPSERVER", buf, 1);
 #	else
@@ -150,7 +150,7 @@ getserverbyfile(
 	}
 
 	if ((cp = getenv("NNTPSERVER")) != NULL) {
-		get_nntpserver(buf, cp);
+		get_nntpserver(buf, sizeof(buf), cp);
 		return buf;
 	}
 
@@ -173,18 +173,11 @@ getserverbyfile(
 		(void) fclose(fp);
 
 		if (cp != NULL) {
-			get_nntpserver(buf, cp);
+			get_nntpserver(buf, sizeof(buf), cp);
 			return buf;
 		}
 	}
 
-#	ifdef USE_INN_NNTPLIB
-	if ((cp = GetConfigValue(_CONF_SERVER)) != NULL) {
-		(void) STRCPY(buf, cp);
-		return buf;
-	}
-#	endif /* USE_INN_NNTPLIB */
-
 #	ifdef NNTP_DEFAULT_SERVER
 	if (*(NNTP_DEFAULT_SERVER))
 		return strcpy(buf, NNTP_DEFAULT_SERVER);
@@ -215,7 +208,7 @@ static int
 server_init(
 	char *machine,
 	const char *cservice,	/* usually a literal */
-	int port,
+	unsigned short port,
 	char *text,
 	size_t mlen)
 {
@@ -237,9 +230,9 @@ server_init(
 		sockt_rd = get_tcp_socket(machine, service, port);
 #	else
 #		ifdef INET6
-	sockt_rd = get_tcp6_socket(machine, (unsigned short) port);
+	sockt_rd = get_tcp6_socket(machine, port);
 #		else
-	sockt_rd = get_tcp_socket(machine, service, (unsigned short) port);
+	sockt_rd = get_tcp_socket(machine, service, port);
 #		endif /* INET6 */
 #	endif /* DECNET */
 
@@ -628,8 +621,7 @@ get_tcp6_socket(
 	hints.ai_socktype = SOCK_STREAM;
 	res = (struct addrinfo *) 0;
 	res0 = (struct addrinfo *) 0;
-	err = getaddrinfo(mymachine, myport, &hints, &res0);
-	if (err != 0) {
+	if ((err = getaddrinfo(mymachine, myport, &hints, &res0))) {
 		my_fprintf(stderr, "\ngetaddrinfo: %s\n", gai_strerror(err));
 		return -1;
 	}
@@ -749,7 +741,10 @@ u_put_server(
 {
 	s_puts(string, nntp_wr_fp);
 #	ifdef DEBUG
-	debug_nntp(">>>", string);
+	if (debug & DEBUG_NNTP) {
+		if (strcmp(string, "\r\n"))
+			debug_print_file("NNTP", ">>> %s", string);
+	}
 #	endif /* DEBUG */
 }
 
@@ -777,13 +772,21 @@ put_server(
 		s_puts(string, nntp_wr_fp);
 		s_puts("\r\n", nntp_wr_fp);
 #	ifdef DEBUG
-		debug_nntp(">>>", string);
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", ">>> %s", string);
 #	endif /* DEBUG */
 		/*
 		 * remember the last command we wrote to be able to resend it after a
 		 * reconnect. reconnection is handled by get_server()
+		 *
+		 * don't cache "LIST ACTIVE something" as we would need to
+		 * resend all of them but we remeber just the last one. we cache
+		 * "LIST" instead, this will slow down things, but that's ok on
+		 * reconnect.
 		 */
-		if (last_put != string)
+		if (!strncmp(string, "LIST ACTIVE ", 12))
+			STRCPY(last_put, "LIST");
+		else
 			STRCPY(last_put, string);
 	}
 	(void) s_flush(nntp_wr_fp);
@@ -836,12 +839,14 @@ reconnect(
 			put_server(last_put);
 			s_gets(last_put, NNTP_STRLEN, nntp_rd_fp);
 #	ifdef DEBUG
-			debug_nntp("<<<", last_put);
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "<<< %s", last_put);
 #	endif /* DEBUG */
 			DEBUG_IO((stderr, _("Read (%s)\n"), last_put));
 		}
 		DEBUG_IO((stderr, _("Resend last command (%s)\n"), buf));
 		put_server(buf);
+		did_reconnect = TRUE;
 		return 0;
 	}
 
@@ -877,9 +882,12 @@ get_server(
 
 	/*
 	 * NULL socket reads indicates socket has closed. Try a few times more
+	 *
+	 * TODO: add a timeout (some servers do not close the connection but
+	 *       simply do not send any response data -> we need a timeout to
+	 *       leave the s_gets() in that case)
 	 */
 	while (nntp_rd_fp == NULL || s_gets(string, size, nntp_rd_fp) == NULL) {
-
 		if (quitting)						/* Don't bother to reconnect */
 			tin_done(NNTP_ERROR_EXIT);		/* And don't try to disconnect again! */
 
@@ -895,6 +903,15 @@ get_server(
 		 * when user is quitting tin if tinrc.auto_reconnect is false.
 		 */
 		if (strncmp(last_put, "QUIT", 4)) {
+			/*
+			 * Typhoon v2.1.1.363 colses the connection right after an unknown
+			 * command, (i.e. CAPABILITIES) so we avoid the reissue it on a
+			 * reconnect if it was the last command.
+			 */
+			if (!strncmp(last_put, "CAPABILITIES", 12)) {
+				strcpy(last_put, "MODE READER");
+				nntp_caps.type = BROKEN;
+			}
 			retry = reconnect(retry);		/* Will abort when out of tries */
 			reconnected_in_last_get_server = TRUE;
 		} else {
@@ -956,7 +973,6 @@ check_extensions(
 {
 	char *ptr;
 	int ret = 0;
-#	if 1 /* "CAPABILITIES" replaces "LIST EXTENSIONS" */
 	char buf[NNTP_STRLEN];
 	char *d;
 	int i;
@@ -968,11 +984,12 @@ check_extensions(
 			nntp_caps.type = CAPABILITIES;
 			while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
 #		ifdef DEBUG
-				debug_nntp("<<<", ptr);
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "<<< %s", ptr);
 #		endif /* DEBUG */
 				/* look for version number(s) */
 				if (!nntp_caps.version && nntp_caps.type == CAPABILITIES) {
-					if (!strcasecmp(ptr, "VERSION")) {
+					if (!strncasecmp(ptr, "VERSION", 7)) {
 						d = ptr + 7;
 						d = strpbrk(d, " \t");
 						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
@@ -985,44 +1002,47 @@ check_extensions(
 				/* we currently only support CAPABILITIES VERSION 2 */
 				if (nntp_caps.version == 2) {
 					/*
-					 * check for LIST variants - this code is untested
+					 * check for LIST variants
 					 */
-					if (!strcasecmp(ptr, "LIST")) {
+					if (!strncasecmp(ptr, "LIST", 4)) {
 						d = ptr + 4;
 						d = strpbrk(d, " \t");
 						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
 							d++;
-							if (!strcasecmp(d, "ACTIVE.TIMES"))
+							if (!strncasecmp(d, "ACTIVE.TIMES", 12))
 								nntp_caps.list_active_times = TRUE;
-							else if (!strcasecmp(d, "ACTIVE"))
-								nntp_caps.list_active_times = TRUE;
-							else if (!strcasecmp(d, "DISTRIB.PATS"))
+							else if (!strncasecmp(d, "ACTIVE", 6))
+								nntp_caps.list_active = TRUE;
+							else if (!strncasecmp(d, "DISTRIB.PATS", 12))
 								nntp_caps.list_distrib_pats = TRUE;
-							else if (!strcasecmp(d, "DISTRIBUTIONS")) /* LIST DISTRIBUTIONS, "private" extension, RFC 2980 */
+							else if (!strncasecmp(d, "DISTRIBUTIONS", 13)) /* "private" extension, RFC 2980 */
 								nntp_caps.list_distributions = TRUE;
-							else if (!strcasecmp(d, "HEADERS"))
+							else if (!strncasecmp(d, "HEADERS", 7))
 								nntp_caps.list_headers = TRUE; /* HDR requires LIST HEADERS, but not vice versa */
-							else if (!strcasecmp(d, "NEWSGROUPS"))
+							else if (!strncasecmp(d, "NEWSGROUPS", 10))
 								nntp_caps.list_newsgroups = TRUE;
-							else if (!strcasecmp(d, "OVERVIEW.FMT")) /* OVER requires OVERVIEW.FMT, but not vice versa */
+							else if (!strncasecmp(d, "OVERVIEW.FMT", 12)) /* OVER requires OVERVIEW.FMT, but not vice versa */
 								nntp_caps.list_overview_fmt = TRUE;
-							else if (!strcasecmp(d, "MOTD")) /* "private" extension */
+							else if (!strncasecmp(d, "MOTD", 4)) /* "private" extension */
 								nntp_caps.list_motd = TRUE;
-							else if (!strcasecmp(d, "SUBSCRIPTIONS")) /* "private" extension, RFC 2980 */
+							else if (!strncasecmp(d, "SUBSCRIPTIONS", 13)) /* "private" extension, RFC 2980 */
 								nntp_caps.list_subscriptions = TRUE;
-							else if (!strcasecmp(d, "MODERATORS")) /* "private" extension */
+							else if (!strncasecmp(d, "MODERATORS", 10)) /* "private" extension */
 								nntp_caps.list_moderators = TRUE;
 							d = strpbrk(d, " \t");
 						}
-					} else if (!strcasecmp(ptr, "IMPLEMENTATION"))
+					} else if (!strncasecmp(ptr, "IMPLEMENTATION", 14)) {
 						nntp_caps.implementation = my_strdup(ptr + 14);
-					else if (!strcasecmp(ptr, "MODE-READER")) {
+						str_trim(nntp_caps.implementation);
+					} else if (!strcasecmp(ptr, "MODE-READER")) {
 						if (!nntp_caps.reader)
 							nntp_caps.mode_reader = TRUE;
-					} else if (!strcasecmp(ptr, "READER")) {
+					} else if (!strcasecmp(ptr, "READER")) { /* if we saw READER, "LIST ACTIVE" and "LIST NEWSGROUPS" must be implemented */
 						nntp_caps.reader = TRUE;
 						nntp_caps.mode_reader = FALSE;
-					} else if (!strcasecmp(d, "POST"))
+						nntp_caps.list_newsgroups = TRUE;
+						nntp_caps.list_active = TRUE;
+					} else if (!strcasecmp(ptr, "POST"))
 						nntp_caps.post = TRUE;
 					else if (!strcasecmp(ptr, "NEWNEWS"))
 						nntp_caps.newnews = TRUE;
@@ -1033,9 +1053,8 @@ check_extensions(
 					/*
 					 * NOTE: if we saw OVER, LIST OVERVIEW.FMT _must_ be implemented
 					 */
-					else if (!strcasecmp(ptr, &xover_cmds[1])) {
-						nntp_caps.over = TRUE;
-						nntp_caps.list_overview_fmt = TRUE;
+					else if (!strncasecmp(ptr, &xover_cmds[1], strlen(&xover_cmds[1]))) {
+						nntp_caps.list_overview_fmt = nntp_caps.over = TRUE;
 						nntp_caps.over_cmd = &xover_cmds[1];
 						d = ptr + strlen(&xover_cmds[1]);
 						d = strpbrk(d, " \t");
@@ -1045,134 +1064,90 @@ check_extensions(
 								nntp_caps.over_msgid = TRUE;
 							d = strpbrk(d, " \t");
 						}
-					} else if (!strcasecmp(ptr, "AUTHINFO")) {
+					}
+					/*
+					 * NOTE: if we saw HDR, LIST HEADERS _must_ be implemented
+					 */
+					else if (!strncasecmp(ptr, &xhdr_cmds[1], strlen(&xhdr_cmds[1]))) {
+						nntp_caps.hdr_cmd = &xhdr_cmds[1];
+						nntp_caps.list_headers = nntp_caps.hdr = TRUE;
+					} else if (!strncasecmp(ptr, "AUTHINFO", 8)) {
 						d = ptr + 8;
 						d = strpbrk(d, " \t");
 						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
 							d++;
-							if (!strcasecmp(d, "USER"))
+							if (!strncasecmp(d, "USER", 4))
 								nntp_caps.authinfo_user = TRUE;
-							if (!strcasecmp(d, "SASL"))
+							if (!strncasecmp(d, "SASL", 4))
 								nntp_caps.authinfo_sasl = TRUE;
 							d = strpbrk(d, " \t");
 						}
+					} else if (!strncasecmp(ptr, "SASL", 4)) {
+						d = ptr + 4;
+						d = strpbrk(d, " \t");
+						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
+							d++;
+							if (!strncasecmp(d, "CRAM-MD5", 8)) /* RFC 2195 */
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_cram_md5 = TRUE;
+							if (!strncasecmp(d, "DIGEST-MD5", 10)) /* RFC 2831 */
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_digest_md5 = TRUE;
+							if (!strncasecmp(d, "PLAIN", 5)) /* RFC 4616 */
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_plain = TRUE;
+							if (!strncasecmp(d, "GSSAPI", 6)) /* RFC 4752 */
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_gssapi = TRUE;
+							if (!strncasecmp(d, "EXTERNAL", 8)) /* RFC 4422 */
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_external = TRUE;
+#		if 0 /* inn also can do these */
+							if (!strncasecmp(d, "OTP", 3)) /* RFC 2444 */
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_otp = TRUE;
+							if (!strncasecmp(d, "NTLM", 4)) /* Microsoft */
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_ntlm = TRUE;
+							if (!strncasecmp(d, "LOGIN", 5)) /* Microsoft */
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_login = TRUE;
+#		endif
+						}
 					}
-#		if 0
-					/*
-					 * NOTE: if we saw HDR, LIST HEADERS _must_ be implemented
-					 */
-					else if (!strcasecmp(ptr, &xhdr_cmds[1])) {
-						nntp_caps.hdr_cmd = &xhdr_cmds[1];
-						nntp_caps.hdr = TRUE;
-						nntp_caps.list_headers = TRUE;
-					}
+#		if 0 /* we don't need these */
 					else if (!strcasecmp(ptr, "IHAVE"))
 						nntp_caps.ihave = TRUE;
+					else if (!strcasecmp(ptr, "STREAMING"))
+						nntp_caps.streaming = TRUE;
 #		endif /* 0 */
-					/*
-					 * TODO: SASL, STREAMING
-					 */
 				} else
-					nntp_caps.type = NO;
+					nntp_caps.type = NONE;
 			}
 			break;
 
+		/*
+		 * XanaNewz 2 Server Version 2.0.0.3 doesn't know CAPABILITIES
+		 * but respondes with 400 _without_ closing the connection. If
+		 * you use tin on a XanaNewz 2 Server comment out the following
+		 * case.
+		 */
+#		if 1
 		case ERR_GOODBYE:
 			ret = i;
-			error_message(buf);
+			error_message(2, buf);
 			break;
+#		endif /* 1 */
 
 		default:
 			break;
 	}
-#		ifdef DEBUG
-	debug_print_nntp_extensions();
-#		endif /* DEBUG */
-	if ((ret != ERR_GOODBYE) && !*sec && !nntp_caps.reader) {
-		if (nntp_caps.type == CAPABILITIES && !nntp_caps.mode_reader)
-			return -1; /* no mode-switching and no reader mode, give up */
+
+	if ((ret != ERR_GOODBYE) && !*sec && !nntp_caps.reader && !force_auth_on_conn_open) {
+		if (nntp_caps.type == CAPABILITIES && !nntp_caps.mode_reader) {
+			if (!nntp_caps.post) { /* as a last resort check if post was mentioned */
+				error_message(2, _("CAPABILITIES did not announce any of READER, MODE-READER, POST")); /* TODO: -> lang.c */
+				return -1; /* give up */
+			}
+		}
 		if ((ret = mode_reader(&*sec)) != 0)
 			return ret;
-		else if (nntp_caps.type == CAPABILITIES) /* 2nd pass */
+		else if (nntp_caps.type != BROKEN) /* 2nd pass (we may receive 500 on the first try on an RFC 3977 aware server with mode switching) */
 			ret = check_extensions(&*sec);
 	}
 
-#	else
-	/*
-	 * TODO: the following code can go away in tin 2.0 as it's
-	 *       obsolete by RFC 3977
-	 *
-	 * "LIST EXTENSIONS" is somewhat troublesome as there are a lot
-	 * of broken implementations out there and it is a multiline response
-	 */
-	if (nntp_caps.type == NO) {
-		char buf[NNTP_STRLEN];
-		int i;
-
-		buf[0] = '\0';
-		i = new_nntp_command("LIST EXTENSIONS", OK_EXTENSIONS, buf, sizeof(buf));
-		switch (i) {
-			case 215:	/* Netscape-Collabra/3.52 (badly broken); NetWare-News-Server/5.1 */
-#		ifdef DEBUG
-				debug_nntp("LIST EXTENSIONS", "skipping data");
-#		endif /* DEBUG */
-				while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL)
-					;
-				break;
-
-			case OK_EXTENSIONS:	/* as defined draft-ietf-nntpext-base-27.txt */
-			case 205:	/* M$ Exchange 5.5 */
-				nntp_caps.type = LIST_EXTENSIONS;
-				while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
-					if (nntp_caps.type == LIST_EXTENSIONS) {
-#		ifdef DEBUG
-						debug_nntp("<<<", ptr);
-#		endif /* DEBUG */
-						/*
-						 * some servers (e.g. Hamster 1.3) have leading spaces in
-						 * the extension list
-						 */
-						str_trim(ptr);
-						/*
-						 * Check for (X)OVER
-						 * XOVER should not be listed in EXTENSIONS (but sometimes
-						 * is) checking for it if OVER is not found does no harm.
-						 */
-						if (!nntp_caps.over_cmd) {
-							for (i = 1; i >= 0; i--) {
-								if (strcasecmp(ptr, &xover_cmds[i]) == 0) {
-									nntp_caps.over_cmd = &xover_cmds[i];
-									break;
-								}
-							}
-						}
-					}
-				}
-				/*
-				 * as the server did support LIST EXTENSIONS it's likely that it
-				 * also can do LIST MOTD (we don't bother to parse the LIST
-				 * EXTENSIONS output for MOTD as it never was standardized;
-				 * draft-ietf-nntpext-base-24.txt only described OVER, HDR and
-				 * LISTGROUP).
-				 */
-				nntp_caps.list_motd = TRUE;
-				break;
-
-			case ERR_GOODBYE:
-				ret = i;
-				error_message(buf);
-				break;
-
-			default:
-				break;
-		}
-#		ifdef DEBUG
-		debug_print_nntp_extensions();
-#		endif /* DEBUG */
-		if ((ret != ERR_GOODBYE) && !*sec)
-			ret = mode_reader(&*sec);
-	}
-#	endif /* 1 */
 	return ret;
 }
 
@@ -1189,7 +1164,8 @@ mode_reader(
 	if (!nntp_caps.reader) {
 		char line[NNTP_STRLEN];
 #ifdef DEBUG
-		debug_nntp("mode_reader", "mode reader");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "mode_reader() MODE READER");
 #endif /* DEBUG */
 		DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 		put_server("MODE READER");
@@ -1200,26 +1176,23 @@ mode_reader(
 		 *
 		 *   200 (OK_CANPOST)     Hello, you can post
 		 *   201 (OK_NOPOST)      Hello, you can't post
-		 *  (202 (OK_NOIHAVE)     discussed on the itef mailinglist; withdrawn)
-		 *  (203 (OK_NOPOSTIHAVE) discussed on the itef mailinglist; withdrawn)
 		 *   502 (ERR_ACCESS)     Service unavailable
 		 *
-		 * However, there may be old servers out there that do not implement this
-		 * command and therefore return ERR_COMMAND (500). Unfortunately there
-		 * are some new servers out there (i.e. INN 2.4.0 (20020220 prerelease)
-		 * which do return ERR_COMMAND if they are feed only servers.
+		 * However, there are servers out there (e.g. Delegate 9.8.x) that do
+		 * not implement this command and therefore return ERR_COMMAND (500).
+		 * Unfortunately there are some new servers out there (i.e. INN 2.4.0
+		 * (20020220 prerelease) which do return ERR_COMMAND if they are feed
+		 * only servers.
 		 */
 
 		switch ((ret = get_respcode(line, sizeof(line)))) {
 			case OK_CANPOST:
-	/*		case OK_NOIHAVE: */
 				can_post = TRUE && !force_no_post;
 				*sec = TRUE;
 				ret = 0;
 				break;
 
 			case OK_NOPOST:
-	/*		case OK_NOPOSTIHAVE: */
 				can_post = FALSE;
 				*sec = TRUE;
 				ret = 0;
@@ -1227,10 +1200,15 @@ mode_reader(
 
 			case ERR_GOODBYE:
 			case ERR_ACCESS:
-				error_message(line);
+				error_message(2, line);
 				return ret;
 
 			case ERR_COMMAND:
+#if 1
+				ret = 0;
+				break;
+#endif /* 1 */
+
 			default:
 				break;
 		}
@@ -1263,12 +1241,13 @@ nntp_open(
 		return 0;
 
 #	ifdef DEBUG
-	debug_nntp("nntp_open", "BEGIN");
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_open() BEGIN");
 #	endif /* DEBUG */
 
 	if (nntp_server == NULL) {
-		error_message(_(txt_cannot_get_nntp_server_name));
-		error_message(_(txt_server_name_in_file_env_var), NNTP_SERVER_FILE);
+		error_message(2, _(txt_cannot_get_nntp_server_name));
+		error_message(2, _(txt_server_name_in_file_env_var), NNTP_SERVER_FILE);
 		return -EHOSTUNREACH;
 	}
 
@@ -1280,7 +1259,8 @@ nntp_open(
 	}
 
 #	ifdef DEBUG
-	debug_nntp("nntp_open", nntp_server);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_open() %s", nntp_server);
 #	endif /* DEBUG */
 
 	ret = server_init(nntp_server, NNTP_TCP_NAME, nntp_tcp_port, line, sizeof(line));
@@ -1290,7 +1270,8 @@ nntp_open(
 		my_fputc('\n', stdout);
 
 #	ifdef DEBUG
-	debug_nntp("nntp_open", line);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_open() %s", line);
 #	endif /* DEBUG */
 
 	switch (ret) {
@@ -1301,8 +1282,6 @@ nntp_open(
 		 * According to the ietf-nntp mailinglist:
 		 *   200 you may (try to) do anything
 		 *   201 you may not POST
-		 *  (202 you may not IHAVE)
-		 *  (203 you may not do EITHER)
 		 *   All unrecognised 200 series codes should be assumed as success.
 		 *   All unrecognised 300 series codes should be assumed as notice to continue.
 		 *   All unrecognised 400 series codes should be assumed as temporary error.
@@ -1310,12 +1289,10 @@ nntp_open(
 		 */
 
 		case OK_CANPOST:
-/*		case OK_NOIHAVE: */
 			can_post = TRUE && !force_no_post;
 			break;
 
 		case OK_NOPOST:
-/*		case OK_NOPOSTIHAVE: */
 			can_post = FALSE;
 			break;
 
@@ -1325,9 +1302,9 @@ nntp_open(
 				break;
 			}
 			if (ret < 0)
-				error_message(_(txt_failed_to_connect_to_server), nntp_server);
+				error_message(2, _(txt_failed_to_connect_to_server), nntp_server);
 			else
-				error_message(line);
+				error_message(2, line);
 
 			return ret;
 	}
@@ -1342,12 +1319,21 @@ nntp_open(
 
 	/*
 	 * Find out which NNTP extensions are available
-	 * TODO: The authentication method required may be mentioned in the list of
-	 *       extensions. (For details about authentication methods, see
-	 *       draft-ietf-nntpext-authinfo-07.txt).
+	 * - Typhoon v2.1.1.363 closes the connection after an unknown command
+	 *   (i.e. CAPABILITIES) but as we are not allowed to cache CAPABILITIES
+	 *   we reissue the command on reconnect. To prevent a loop we catch this
+	 *   case.
+	 *
+	 * TODO: The authentication method required may be mentioned in the list
+	 *       of extensions. (For details about authentication methods, see
+	 *       RFC 4643).
 	 */
-	if ((ret = check_extensions(&sec)))
-		return ret; /* required "MODE READER" failed, exit */
+	if (nntp_caps.type != BROKEN) {
+		if ((ret = check_extensions(&sec)))
+			return ret; /* required "MODE READER/READER" failed, exit */
+		if (nntp_caps.type == CAPABILITIES && !nntp_caps.post)
+			can_post = FALSE;
+	}
 
 	/*
 	 * If the user wants us to authenticate on connection startup, do it now.
@@ -1358,11 +1344,17 @@ nntp_open(
 	 */
 	if (force_auth_on_conn_open) {
 #	ifdef DEBUG
-		debug_nntp("nntp_open", "authenticate");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "nntp_open() authenticate()");
 #	endif /* DEBUG */
 		authenticate(nntp_server, userid, TRUE);
-		if ((ret = mode_reader(&sec)))
-			return ret; /* "MODE READER" failed, exit */
+		if (nntp_caps.type == CAPABILITIES && !is_reconnect) {
+			if ((ret = check_extensions(&sec))) /* capabilities may change on auth */
+				return ret; /* required "MODE READER/READER" failed, exit */
+		} else {
+			if ((ret = mode_reader(&sec)))
+				return ret; /* "MODE READER" failed, exit */
+		}
 	}
 
 	if (!is_reconnect) {
@@ -1411,7 +1403,7 @@ nntp_open(
 	 * (successor of XOVER as of latest NNTP Draft (Jan 2002)
 	 * We have to check that we _don't_ get an ERR_COMMAND
 	 */
-	if (nntp_caps.type == NO) {
+	if (nntp_caps.type == NONE) {
 		int j = 0;
 
 		for (i = 0; i < 2 && j >= 0; i++) {
@@ -1420,12 +1412,13 @@ nntp_open(
 				case ERR_COMMAND:
 					break;
 
-				case 224:	/* unexpected multiline ok, e.g.: Synchronet 3.13 NNTP Service 1.92 */
+				case OK_XOVER:	/* unexpected multiline ok, e.g.: Synchronet 3.13 NNTP Service 1.92 */
 					nntp_caps.over_cmd = &xover_cmds[i];
 #	ifdef DEBUG
-					debug_nntp(&xover_cmds[i], "skipping data");
+					if (debug & DEBUG_NNTP)
+						debug_print_file("NNTP", "nntp_open() %s skipping data", &xover_cmds[i]);
 #	endif /* DEBUG */
-					while ((linep = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL)
+					while (tin_fgets(FAKE_NNTP_FP, FALSE))
 						;
 					j = -1;
 					break;
@@ -1436,24 +1429,48 @@ nntp_open(
 					break;
 			}
 		}
+#	ifdef XHDR_XREF
+		for (i = 0, j = 0; i < 2 && j >= 0; i++) {
+			j = new_nntp_command(&xhdr_cmds[i], ERR_CMDSYN, line, sizeof(line));
+			switch (j) {
+				case ERR_COMMAND:
+					break;
+
+				case 221:	/* unexpected multiline ok, e.g.: SoftVelocity Discussions 2.5q */
+					nntp_caps.hdr_cmd = &xhdr_cmds[i];
+#	ifdef DEBUG
+					if (debug & DEBUG_NNTP)
+						debug_print_file("NNTP", "nntp_open() %s skipping data", &xhdr_cmds[i]);
+#	endif /* DEBUG */
+					while (tin_fgets(FAKE_NNTP_FP, FALSE))
+						;
+					j = -1;
+					break;
+
+				default:	/* usualy ERR_CMDSYN (args missing), Typhoon/Twister sends ERR_NCING */
+					nntp_caps.hdr_cmd = &xhdr_cmds[i];
+					j = -1;
+					break;
+			}
+		}
+#	endif /* XHDR_XREF */
 	} else {
 		if (!nntp_caps.over_cmd) {
 			/*
 			 * CAPABILITIES/LIST EXTENSIONS didn't mention OVER or XOVER, try
 			 * XOVER
 			 */
-			i = new_nntp_command(xover_cmds, ERR_NCING, line, sizeof(line));
-
-			switch (i) {
+			switch (new_nntp_command(xover_cmds, ERR_NCING, line, sizeof(line))) {
 				case ERR_COMMAND:
 					break;
 
-				case 224:	/* unexpected multiline ok, e.g.: Synchronet 3.13 NNTP Service 1.92 */
+				case OK_XOVER:	/* unexpected multiline ok, e.g.: Synchronet 3.13 NNTP Service 1.92 */
 					nntp_caps.over_cmd = xover_cmds;
 #	ifdef DEBUG
-					debug_nntp(xover_cmds, "skipping data");
+					if (debug & DEBUG_NNTP)
+						debug_print_file("NNTP", "nntp_open() %s skipping data", xover_cmds);
 #	endif /* DEBUG */
-					while ((linep = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL)
+					while (tin_fgets(FAKE_NNTP_FP, FALSE))
 						;
 					break;
 
@@ -1462,16 +1479,32 @@ nntp_open(
 					break;
 			}
 		}
-#	if 0 /* unused */
+#	ifdef XHDR_XREF
 		if (!nntp_caps.hdr_cmd) {
 			/*
-			 * LIST EXTENSIONS didn't mention HDR or XHDR, try
+			 * CAPABILITIES/LIST EXTENSIONS didn't mention HDR or XHDR, try
 			 * XHDR
 			 */
-			if (!nntp_command(xhdr_cmds, ERR_COMMAND, NULL, 0))
-				nntp_caps.hdr_cmd = xhdr_cmds;
+			switch (new_nntp_command(xhdr_cmds, ERR_CMDSYN, line, sizeof(line))) {
+				case ERR_COMMAND:
+					break;
+
+				case 221:	/* unexpected multiline ok, e.g.: SoftVelocity Discussions 2.5q */
+					nntp_caps.hdr_cmd = xhdr_cmds;
+#	ifdef DEBUG
+					if (debug & DEBUG_NNTP)
+						debug_print_file("NNTP", "nntp_open() %s skipping data", xhdr_cmds);
+#	endif /* DEBUG */
+					while (tin_fgets(FAKE_NNTP_FP, FALSE))
+						;
+					break;
+
+				default:	/* usualy ERR_CMDSYN (args missing), Typhoon/Twister sends ERR_NCING */
+					nntp_caps.hdr_cmd = xhdr_cmds;
+					break;
+			}
 		}
-#	endif /* 0 */
+#	endif /* XHDR_XREF */
 	}
 
 	if (!nntp_caps.over_cmd) {
@@ -1483,22 +1516,13 @@ nntp_open(
 			else
 				wait_message(2, _(txt_caching_off));
 		}
+	}
 #	if 0
-	} else {
+	else {
 		/*
 		 * TODO: issue warning if old index files found?
 		 *	      in index_newsdir?
 		 */
-#	endif /* 0 */
-	}
-
-#	if 0
-	/*
-	 * TODO: if we're using -n, check for LIST NEWSGROUPS <wildmat>
-	 * see also comments in open_newsgroups_fp()
-	 */
-	if (newsrc_active && !list_active) { /* -n */
-		/* code goes here */
 	}
 #	endif /* 0 */
 
@@ -1529,9 +1553,10 @@ nntp_close(
 	void)
 {
 #ifdef NNTP_ABLE
-	if (read_news_via_nntp) {
+	if (read_news_via_nntp && !read_saved_news) {
 #	ifdef DEBUG
-		debug_nntp("nntp_close", "END");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "nntp_close() END");
 #	endif /* DEBUG */
 		close_server();
 	}
@@ -1539,6 +1564,7 @@ nntp_close(
 }
 
 
+#ifdef NNTP_ABLE
 /*
  * Get a response code from the server.
  * Returns:
@@ -1554,21 +1580,22 @@ get_only_respcode(
 	char *message,
 	size_t mlen)
 {
-	int respcode = 0;
-#ifdef NNTP_ABLE
-	char *ptr, *end;
+	int respcode;
+	char *end, *ptr;
 
 	ptr = tin_fgets(FAKE_NNTP_FP, FALSE);
 
 	if (tin_errno || ptr == NULL) {
 #	ifdef DEBUG
-		debug_nntp("<<<", "Error: tin_error<>0 or ptr==NULL in get_only_respcode()");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "<<< Error: tin_error<>0 or ptr==NULL in get_only_respcode()");
 #	endif /* DEBUG */
 		return -1;
 	}
 
 #	ifdef DEBUG
-	debug_nntp("<<<", ptr);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "<<< %s",  ptr);
 #	endif /* DEBUG */
 	respcode = (int) strtol(ptr, &end, 10);
 	DEBUG_IO((stderr, "get_only_respcode(%d)\n", respcode));
@@ -1577,26 +1604,29 @@ get_only_respcode(
 	 * we also reconnect on ERR_FAULT if last_put was ARTICLE or LIST or POST
 	 * as inn (2.2.3) sends ERR_FAULT on timeout
 	 */
-	if (((respcode == ERR_FAULT && !strncmp(last_put, "ARTICLE", 7)) || (respcode == ERR_FAULT && !strncmp(last_put, "POST", 4)) || (respcode == ERR_FAULT && !strcmp(last_put, "LIST")) ||  respcode == ERR_GOODBYE || respcode == OK_GOODBYE) && last_put[0] != '\0' && strcmp(last_put, "QUIT")) {
+	if (last_put[0] != '\0' && ((respcode == ERR_FAULT && !strncmp(last_put, "ARTICLE", 7)) || (respcode == ERR_FAULT && !strcmp(last_put, "POST")) || (respcode == ERR_FAULT && !strcmp(last_put, "LIST")) || respcode == ERR_GOODBYE || respcode == OK_GOODBYE) && strcmp(last_put, "QUIT")) {
 		/*
 		 * Maybe server timed out.
 		 * If so, retrying will force a reconnect.
 		 */
 #	ifdef DEBUG
-		debug_nntp("get_only_respcode", "timeout");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "get_only_respcode() timeout");
 #	endif /* DEBUG */
 		put_server(last_put);
 		ptr = tin_fgets(FAKE_NNTP_FP, FALSE);
 
 		if (tin_errno) {
 #	ifdef DEBUG
-			debug_nntp("<<<", "Error: tin_errno <> 0");
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "<<< Error: tin_errno <> 0");
 #	endif /* DEBUG */
 			return -1;
 		}
 
 #	ifdef DEBUG
-		debug_nntp("<<<", ptr);
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "<<< %s", ptr);
 #	endif /* DEBUG */
 		respcode = (int) strtol(ptr, &end, 10);
 		DEBUG_IO((stderr, "get_only_respcode(%d)\n", respcode));
@@ -1604,7 +1634,6 @@ get_only_respcode(
 	if (message != NULL && mlen > 1)		/* Pass out the rest of the text */
 		my_strncpy(message, end, mlen - 1);
 
-#endif /* NNTP_ABLE */
 	return respcode;
 }
 
@@ -1618,14 +1647,15 @@ get_only_respcode(
  *	code is copied into it.
  * Performs authentication if required and repeats the last command if
  * necessary after a timeout.
+ *
+ * TODO: make this handle 483 (RFC 3977) return codes
  */
 int
 get_respcode(
 	char *message,
 	size_t mlen)
 {
-	int respcode = 0;
-#ifdef NNTP_ABLE
+	int respcode;
 	char savebuf[NNTP_STRLEN];
 	char *ptr, *end;
 
@@ -1635,7 +1665,8 @@ get_respcode(
 		 * Server requires authentication.
 		 */
 #	ifdef DEBUG
-		debug_nntp("get_respcode", "authentication");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "get_respcode() authentication");
 #	endif /* DEBUG */
 		strncpy(savebuf, last_put, sizeof(savebuf) - 1);		/* Take copy, as authenticate() will clobber this */
 
@@ -1646,7 +1677,8 @@ get_respcode(
 				put_server(last_put);
 				s_gets(last_put, NNTP_STRLEN, nntp_rd_fp);
 #	ifdef DEBUG
-				debug_nntp("<<<", last_put);
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "<<< %s", last_put);
 #	endif /* DEBUG */
 				DEBUG_IO((stderr, _("Read (%s)\n"), last_put));
 			}
@@ -1657,30 +1689,30 @@ get_respcode(
 
 			if (tin_errno) {
 #	ifdef DEBUG
-				debug_nntp("<<<", "Error: tin_errno <> 0");
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "<<< Error: tin_errno <> 0");
 #	endif /* DEBUG */
 				return -1;
 			}
 
 #	ifdef DEBUG
-			debug_nntp("<<<", ptr);
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "<<< %s", ptr);
 #	endif /* DEBUG */
 			respcode = (int) strtol(ptr, &end, 10);
 			if (message != NULL && mlen > 1)				/* Pass out the rest of the text */
 				strncpy(message, end, mlen - 1);
 
 		} else {
-			error_message(_(txt_auth_failed), ERR_ACCESS);
+			error_message(2, _(txt_auth_failed), ERR_ACCESS);
 			/*	return -1; */
 			tin_done(EXIT_FAILURE);
 		}
 	}
-#endif /* NNTP_ABLE */
 	return respcode;
 }
 
 
-#ifdef NNTP_ABLE
 /*
  * Do an NNTP command. Send command to server, and read the reply.
  * If the reply code matches success, then return an open file stream
@@ -1697,21 +1729,24 @@ nntp_command(
 {
 DEBUG_IO((stderr, "nntp_command(%s)\n", command));
 #	ifdef DEBUG
-	debug_nntp("nntp command", command);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_command(%s)", command);
 #	endif /* DEBUG */
 	put_server(command);
 
 	if (!bool_equal(dangerous_signal_exit, TRUE)) {
 		if (get_respcode(message, mlen) != success) {
 #	ifdef DEBUG
-			debug_nntp(command, "NOT_OK");
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "nntp_command(%s) NOT_OK", command);
 #	endif /* DEBUG */
-			/* error_message("%s", message); */
+			/* error_message(2, "%s", message); */
 			return (FILE *) 0;
 		}
 	}
 #	ifdef DEBUG
-	debug_nntp(command, "OK");
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_command(%s) OK", command);
 #	endif /* DEBUG */
 	return FAKE_NNTP_FP;
 }
@@ -1722,7 +1757,7 @@ DEBUG_IO((stderr, "nntp_command(%s)\n", command));
  * TODO: use it instead of nntp_command in the rest of the code
  *       (wherever it is more usefull).
  */
-static int
+int
 new_nntp_command(
 	const char *command,
 	int success,
@@ -1731,22 +1766,25 @@ new_nntp_command(
 {
 	int respcode = 0;
 
-DEBUG_IO((stderr, "nntp_command(%s)\n", command));
+DEBUG_IO((stderr, "new_nntp_command(%s)\n", command));
 #	ifdef DEBUG
-	debug_nntp("nntp command", command);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "new_nntp_command(%s)", command);
 #	endif /* DEBUG */
 	put_server(command);
 
 	if (!bool_equal(dangerous_signal_exit, TRUE)) {
 		if ((respcode = get_respcode(message, mlen)) != success) {
 #	ifdef DEBUG
-			debug_nntp(command, "NOT_OK - Expected: %d, got: %d", success, respcode);
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "new_nntp_command(%s) NOT_OK - Expected: %d, got: %d", command, success, respcode);
 #	endif /* DEBUG */
 			return respcode;
 		}
 	}
 #	ifdef DEBUG
-	debug_nntp(command, "OK");
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "new_nntp_command(%s) OK", command);
 #	endif /* DEBUG */
 	return respcode;
 }
@@ -1771,7 +1809,8 @@ list_motd(
 #	endif /* HAVE_COLOR */
 			while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
 #	ifdef DEBUG
-				debug_nntp("<<<", ptr);
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "<<< %s", ptr);
 #	endif /* DEBUG */
 				/*
 				 * TODO: - store a hash value of the entire motd in the server-rc
