@@ -3,10 +3,10 @@
  *  Module    : page.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2006-10-01
+ *  Updated   : 2009-01-20
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,13 +42,14 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-
+#if 0
 #if defined(HAVE_IDNA_H) && !defined(_IDNA_H)
 #	include <idna.h>
 #endif /* HAVE_IDNA_H && !_IDNA_H */
 #if defined(HAVE_STRINGPREP_H) && !defined(_STRINGPREP_H)
 #	include <stringprep.h>
 #endif /* HAVE_STRINGPREP_H & !_STRINGPREP_H */
+#endif /* 0 */
 
 /*
  * PAGE_HEADER is the size in lines of the article page header
@@ -113,7 +114,7 @@ static void invoke_metamail(FILE *fp);
 #	define XFACE_CLEAR()	if (tinrc.use_slrnface) \
 								slrnface_clear_xface()
 #	define XFACE_SUPPRESS()	if (tinrc.use_slrnface) \
-										slrnface_suppress_xface()
+								slrnface_suppress_xface()
 #else
 #	define XFACE_SHOW()	/*nothing*/
 #	define XFACE_CLEAR()	/*nothing*/
@@ -337,7 +338,6 @@ show_page(
 			case GLOBAL_ABORT:       /* Abort */
 				break;
 
-			case DIGIT_0:
 			case DIGIT_1:
 			case DIGIT_2:
 			case DIGIT_3:
@@ -542,9 +542,12 @@ page_goto_next_unread:
 #endif /* !DISABLE_PRINTING */
 
 			case PAGE_REPOST:	/* repost current article */
-				XFACE_SUPPRESS();
-				feed_articles(FEED_REPOST, PAGE_LEVEL, group, this_resp);
-				XFACE_SHOW();
+				if (can_post) {
+					XFACE_SUPPRESS();
+					feed_articles(FEED_REPOST, PAGE_LEVEL, group, this_resp);
+					XFACE_SHOW();
+				} else
+					info_message(_(txt_cannot_post));
 				break;
 
 			case PAGE_SAVE:	/* save article/thread/tagged articles */
@@ -563,7 +566,7 @@ page_goto_next_unread:
 
 			case GLOBAL_SEARCH_SUBJECT_FORWARD:	/* search in article */
 			case GLOBAL_SEARCH_SUBJECT_BACKWARD:
-				if ((i = search_article((func == GLOBAL_SEARCH_SUBJECT_FORWARD), repeat_search, search_line, artlines, artline, reveal_ctrl_l_lines, note_fp)) == -1)
+				if (search_article((func == GLOBAL_SEARCH_SUBJECT_FORWARD), repeat_search, search_line, artlines, artline, reveal_ctrl_l_lines, note_fp) == -1)
 					break;
 
 				if (func == GLOBAL_SEARCH_SUBJECT_BACKWARD && !reveal_ctrl_l) {
@@ -692,7 +695,7 @@ page_goto_next_unread:
 
 			case GLOBAL_EDIT_FILTER:
 				XFACE_CLEAR();
-				if (!invoke_editor(filter_file, FILTER_FILE_OFFSET))
+				if (!invoke_editor(filter_file, FILTER_FILE_OFFSET, NULL))
 					break;
 				unfilter_articles();
 				(void) read_filter_file(filter_file);
@@ -725,7 +728,7 @@ page_goto_next_unread:
 			case CATCHUP:			/* catchup - mark read, goto next */
 			case CATCHUP_NEXT_UNREAD:	/* goto next unread */
 				/*
-				 * TODO: if (group->attribute->thread_arts == THREAD_NONE)
+				 * TODO: if (group->attribute->thread_articles == THREAD_NONE)
 				 *       	snprintf(buf, sizeof(buf), _("Mark article as read%s?"), (func == CATCHUP_NEXT_UNREAD) ? _(" and enter next unread article") : "");
 				 *       else
 				 */
@@ -739,7 +742,7 @@ page_goto_next_unread:
 
 			case MARK_THREAD_UNREAD:
 				thd_mark_unread(group, base[which_thread(this_resp)]);
-				if (group->attribute->thread_arts != THREAD_NONE)
+				if (group->attribute->thread_articles != THREAD_NONE)
 					info_message(_(txt_marked_as_unread), _(txt_thread_upper));
 				else
 					info_message(_(txt_marked_as_unread), _(txt_article_upper));
@@ -967,7 +970,7 @@ return_to_index:
 			case PAGE_VIEW_URL:
 				if (!show_all_headers) { /* cooked mode? */
 					XFACE_SUPPRESS();
-					resize_article(FALSE, &pgart); /* umbreak long lines */
+					resize_article(FALSE, &pgart); /* unbreak long lines */
 					process_url();
 					resize_article(TRUE, &pgart); /* rebreak long lines */
 					draw_page(group->name, 0);
@@ -1265,7 +1268,7 @@ draw_page_header(
 	line_len = LEN + 1;
 	buf = my_malloc(line_len);
 
-	if (!my_strftime(buf, line_len, tinrc.date_format, localtime(&arts[this_resp].date))) {
+	if (!my_strftime(buf, line_len, curr_group->attribute->date_format, localtime(&arts[this_resp].date))) {
 		strncpy(buf, BlankIfNull(note_h->date), line_len);
 		buf[line_len - 1] = '\0';
 	}
@@ -1273,7 +1276,7 @@ draw_page_header(
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	/* convert to wide-char format strings */
 	fmt_thread = char2wchar_t(_(txt_thread_x_of_n));
-	fmt_resp = char2wchar_t(_(txt_resp_x_of_n));
+	fmt_resp = char2wchar_t(_(txt_art_x_of_n));
 
 	/*
 	 * determine the needed space for the text at the right hand margin
@@ -1422,7 +1425,7 @@ draw_page_header(
 		my_fputc(' ', stdout);
 
 	if (whichresp)
-		my_printf(_(txt_resp_x_of_n), whichresp, x_resp);
+		my_printf(_(txt_art_x_of_n), whichresp + 1, x_resp + 1);
 	else {
 		if (!x_resp)
 			my_printf(_(txt_no_responses));
@@ -1500,7 +1503,7 @@ draw_page_header(
 	 * the formating info (%4s) needs 3 positions but we need 4 positions
 	 * on the screen for each counter
 	 */
-	right_len = MAX((strlen(_(txt_thread_x_of_n)) - 6 + 8), (strlen(_(txt_resp_x_of_n)) - 6 + 8));
+	right_len = MAX((strlen(_(txt_thread_x_of_n)) - 6 + 8), (strlen(_(txt_art_x_of_n)) - 6 + 8));
 
 	/*
 	 * first line
@@ -1616,7 +1619,7 @@ draw_page_header(
 		my_fputc(' ', stdout);
 
 	if (whichresp)
-		my_printf(_(txt_resp_x_of_n), whichresp, x_resp);
+		my_printf(_(txt_art_x_of_n), whichresp + 1, x_resp + 1);
 	else {
 		if (!x_resp)
 			my_printf(_(txt_no_responses));
@@ -1688,7 +1691,7 @@ load_article(
 	struct t_group *group)
 {
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		fprintf(stderr, "load_art %s(new=%d, curr=%d)\n", (new_respnum == this_resp) ? "ALREADY OPEN!" : "", new_respnum, this_resp);
 #endif /* DEBUG */
 
@@ -1761,7 +1764,7 @@ load_article(
 	if (*tinrc.metamail_prog == '\0' || getenv("NOMETAMAIL") != NULL)	/* Viewer turned off */
 		return 0;
 
-	if (tinrc.ask_for_metamail) {
+	if (group->attribute->ask_for_metamail) {
 		if (prompt_yn(_(txt_use_mime), TRUE) != 1)
 			return 0;
 	}
@@ -1785,7 +1788,7 @@ prompt_response(
 
 	clear_message();
 
-	if ((num = prompt_num(ch, _(txt_read_resp))) == -1) {
+	if ((num = (prompt_num(ch, _(txt_select_art)) - 1)) == -1) {
 		clear_message();
 		return -1;
 	}
@@ -1901,7 +1904,7 @@ toggle_raw(
 							offset += num_bytes;
 						}
 #else
-						if (my_isprint((int) *p)) {
+						if (my_isprint((unsigned char) *p)) {
 							space--;
 							p++;
 							offset++;
@@ -2055,6 +2058,8 @@ info_pager(
 	info_title = title;
 	curr_info_line = 0;
 	preprocess_info_message(info_fh);
+	if (!info_fh)
+		return;
 	set_xclick_off();
 	display_info_page(0);
 
@@ -2227,17 +2232,17 @@ display_info_page(
 }
 
 
-/*
- * TODO: plug mem leak: malloced mem is not freed on exit
- */
 static void
 preprocess_info_message(
 	FILE *info_fh)
 {
 	int chunk = 50;
 
+	FreeAndNull(infoline);
+	if (!info_fh)
+		return;
+
 	rewind(info_fh);
-	FreeIfNeeded(infoline);
 	infoline = my_malloc(sizeof(t_lineinfo) * chunk);
 	num_info_lines = 0;
 
