@@ -3,12 +3,12 @@
  *  Module    : refs.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1996-05-09
- *  Updated   : 2004-03-01
+ *  Updated   : 2008-12-04
  *  Notes     : Cacheing of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2005 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1996-2009 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,15 +44,10 @@
 #define MAX_REFS	100			/* Limit recursion depth */
 #define REF_SEP	" \t"			/* Separator chars in ref headers */
 
-/* Produce disgusting amounts of output to help me tame this thing */
-#undef DEBUG_REFS
-
-#ifdef DEBUG_REFS
+#ifdef DEBUG
 #	define DEBUG_PRINT(x)	fprintf x
-FILE *dbgfd;
-#else
-#	define DEBUG_PRINT(x)
-#endif /* DEBUG_REFS */
+static FILE *dbgfd;
+#endif /* DEBUG */
 
 /*
  * local prototypes
@@ -65,13 +60,13 @@ static t_bool valid_msgid(const char *msgid);
 static unsigned int hash_msgid(const char *key);
 static void add_to_parent(struct t_msgid *ptr);
 static void build_thread(struct t_msgid *ptr);
-#ifdef DEBUG_REFS
-	static void dump_thread(FILE *fp, struct t_msgid *msgid, int level);
+#ifdef DEBUG
 	static void dump_msgid_thread(struct t_msgid *ptr, int level);
 	static void dump_msgid_threads(void);
-#endif /* DEBUG_REFS */
+#endif /* DEBUG */
 #if 0
 	static void dump_msgids(void);
+	static void dump_thread(FILE *fp, struct t_msgid *msgid, int level);
 #endif /* 0 */
 
 /*
@@ -98,10 +93,10 @@ static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
  *
  *    When threading on Refs, a much better view of the original thread
  *    can be built up using this data, and threading is much faster
- *	   because all the article relationships are automatically available
- *	   to us.
+ *    because all the article relationships are automatically available
+ *    to us.
  *
- *	  NB: We don't cache msgids from the filter file.
+ * NB: We don't cache msgids from the filter file.
  */
 
 /*
@@ -238,17 +233,20 @@ add_msgid(
 	struct t_msgid *newparent)
 {
 	struct t_msgid *ptr;
-	struct t_msgid *i = NULL;
+	struct t_msgid *i;
 	unsigned int h;
 
 	if (!msgid) {
-		error_message("add_msgid: NULL msgid\n");
+		error_message(2, "add_msgid: NULL msgid\n");
 		giveup();
 	}
 
 	h = hash_msgid(msgid + 1);				/* Don't hash the initial '<' */
 
-	DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key == MSGID_REF) ? "MSG" : "REF", msgid, (newparent == NULL) ? _("unchanged") : newparent->txt));
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key == MSGID_REF) ? "MSG" : "REF", msgid, (newparent == NULL) ? _("unchanged") : newparent->txt));
+#endif /* DEBUG */
 
 	/*
 	 * Look for this message id in the cache.
@@ -262,7 +260,10 @@ add_msgid(
 		 * CASE 1a - No parent specified, do nothing
 		 */
 		if (newparent == NULL) {
-			DEBUG_PRINT((dbgfd, "nop: %s No parent specified\n", i->txt));
+#ifdef DEBUG
+			if (debug & DEBUG_REFS)
+				DEBUG_PRINT((dbgfd, "nop: %s No parent specified\n", i->txt));
+#endif /* DEBUG */
 			return i;
 		}
 
@@ -270,7 +271,10 @@ add_msgid(
 		 * CASE 1b - Parent not changed, do nothing
 		 */
 		if (newparent == i->parent) {
-			DEBUG_PRINT((dbgfd, "dup: %s -> %s (no change)\n", i->txt, i->parent ? i->parent->txt : "NULL"));
+#ifdef DEBUG
+			if (debug & DEBUG_REFS)
+				DEBUG_PRINT((dbgfd, "dup: %s -> %s (no change)\n", i->txt, i->parent ? i->parent->txt : "NULL"));
+#endif /* DEBUG */
 			return i;
 		}
 
@@ -285,15 +289,20 @@ add_msgid(
 			 */
 			for (ptr = newparent; ptr != NULL; ptr = ptr->parent) {
 				if (ptr == i) {
-					DEBUG_PRINT((dbgfd, "Avoiding circular reference! (%s)\n", (key == MSGID_REF) ? "MSG" : "REF"));
+#ifdef DEBUG
+					if (debug & DEBUG_REFS)
+						DEBUG_PRINT((dbgfd, "Avoiding circular reference! (%s)\n", (key == MSGID_REF) ? "MSG" : "REF"));
+#endif /* DEBUG */
 					return i;
 				}
 			}
 
 			i->parent = newparent;
 			add_to_parent(i);
-
-			DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : _("None")));
+#ifdef DEBUG
+			if (debug & DEBUG_REFS)
+				DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : _("None")));
+#endif /* DEBUG */
 			return i;
 		}
 
@@ -305,18 +314,24 @@ add_msgid(
 		 * All we can do is ignore the error
 		 */
 		if (i->parent != newparent) {
-			DEBUG_PRINT((dbgfd, "Warning: (%s) Ignoring %s -> %s (already %s)\n",
-				(key == MSGID_REF) ? "MSG" : "REF", i->txt,
-				newparent ? newparent->txt : "None", i->parent->txt));
+#ifdef DEBUG
+			if (debug & DEBUG_REFS)
+				DEBUG_PRINT((dbgfd, "Warning: (%s) Ignoring %s -> %s (already %s)\n",
+					(key == MSGID_REF) ? "MSG" : "REF", i->txt,
+					newparent ? newparent->txt : "None", i->parent->txt));
+#endif /* DEBUG */
 
 			return i;
 		}
 
-		error_message("Error: Impossible combination of conditions !\n");
+		error_message(2, "Error: Impossible combination of conditions !\n");
 		return i;
 	}
 
-	DEBUG_PRINT((dbgfd, "new: %s -> %s\n", msgid, (newparent)?newparent->txt:"None"));
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		DEBUG_PRINT((dbgfd, "new: %s -> %s\n", msgid, (newparent)?newparent->txt:"None"));
+#endif /* DEBUG */
 
 	/*
 	 * This is a new node, so build a structure for it
@@ -385,7 +400,10 @@ parse_references(
 	if (!r)
 		return NULL;
 
-	DEBUG_PRINT((dbgfd, "parse_references: %s\n", r));
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		DEBUG_PRINT((dbgfd, "parse_references: %s\n", r));
+#endif /* DEBUG */
 
 	/*
 	 * Break the refs down, using REF_SEP as delimiters
@@ -435,10 +453,12 @@ _get_references(
 	len += strlen(refptr->txt) + 1;	/* msgid + space */
 	if (refptr->parent == NULL || depth > MAX_REFS) {
 
-#ifdef DEBUG_REFS
-		if (depth > MAX_REFS)
-			error_message("Warning: Too many refs near to %s. Truncated\n", refptr->txt);
-#endif /* DEBUG_REFS */
+#ifdef DEBUG
+		if (debug & DEBUG_REFS) {
+			if (depth > MAX_REFS)
+				error_message(2, "Warning: Too many refs near to %s. Truncated\n", refptr->txt);
+		}
+#endif /* DEBUG */
 		refs = my_malloc(len + 1);	/* total length + nullbyte */
 		pos = 0;
 	} else
@@ -501,6 +521,45 @@ free_msgids(
 
 
 #if 0	/* But please don't remove it */
+/*
+ * Function to dump an ASCII tree map of a thread rooted at msgid.
+ * Output goes to fp, level is the current depth of the tree.
+ */
+static void
+dump_thread(
+	FILE *fp,
+	struct t_msgid *msgid,
+	int level)
+{
+	char buff[120];		/* This is _probably_ enough */
+	char *ptr = buff;
+	int i, len;
+
+	/*
+	 * Dump the current article
+	 */
+	sprintf(ptr, "%3d %*s", msgid->article, 2*level, "  ");
+
+	len = strlen(ptr);
+	i = cCOLS - len - 20;
+
+	if (msgid->article >= 0)
+		sprintf(ptr + len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].name) ? arts[msgid->article].name : arts[msgid->article].from);
+	else
+		sprintf(ptr + len, "%-*.*s", i, i, _("[- Unavailable -]"));
+
+	fprintf(fp, "%s\n", ptr);
+
+	if (msgid->child != NULL)
+		dump_thread(fp, msgid->child, level + 1);
+
+	if (msgid->sibling != NULL)
+		dump_thread(fp, msgid->sibling, level);
+
+	return;
+}
+
+
 static void
 dump_msgids(
 	void)
@@ -580,46 +639,7 @@ clear_art_ptrs(
 }
 
 
-/*
- * Function to dump an ASCII tree map of a thread rooted at msgid.
- * Output goes to fp, level is the current depth of the tree.
- */
-#ifdef DEBUG_REFS
-static void
-dump_thread(
-	FILE *fp,
-	struct t_msgid *msgid,
-	int level)
-{
-	char buff[120];		/* This is _probably_ enough */
-	char *ptr = buff;
-	int i, len;
-
-	/*
-	 * Dump the current article
-	 */
-	sprintf(ptr, "%3d %*s", msgid->article, 2*level, "  ");
-
-	len = strlen(ptr);
-	i = cCOLS - len - 20;
-
-	if (msgid->article >= 0)
-		sprintf(ptr + len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].name) ? arts[msgid->article].name : arts[msgid->article].from);
-	else
-		sprintf(ptr + len, "%-*.*s", i, i, _("[- Unavailable -]"));
-
-	fprintf(fp, "%s\n", ptr);
-
-	if (msgid->child != NULL)
-		dump_thread(fp, msgid->child, level + 1);
-
-	if (msgid->sibling != NULL)
-		dump_thread(fp, msgid->sibling, level);
-
-	return;
-}
-
-
+#ifdef DEBUG
 /*
  * Dump out all the threads from the msgid point of view, show the
  * related article index in arts[] where possible
@@ -666,7 +686,7 @@ dump_msgid_threads(
 
 	fprintf(dbgfd, "Dump complete.\n\n");
 }
-#endif /* DEBUG_REFS */
+#endif /* DEBUG */
 
 
 /*
@@ -787,10 +807,15 @@ thread_by_reference(
 	int i;
 	struct t_msgid *ptr;
 
-#ifdef DEBUG_REFS
-	dbgfd = fopen("Refs.info", "w");
-	dump_msgid_threads();
-#endif /* DEBUG_REFS */
+#ifdef DEBUG
+	if (debug & DEBUG_REFS) {
+		char file[PATH_LEN];
+
+		joinpath(file, sizeof(file), TMPDIR, "REFS.info");
+		dbgfd = fopen(file, "w");
+		dump_msgid_threads();
+	}
+#endif /* DEBUG */
 
 	/*
 	 * Build threads starting from root msgids (ie without parent)
@@ -804,20 +829,22 @@ thread_by_reference(
 		}
 	}
 
-#ifdef DEBUG_REFS
-	fprintf(dbgfd, "Full dump of threading info...\n");
-	fprintf(dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd");
+#ifdef DEBUG
+	if (debug & DEBUG_REFS) {
+		fprintf(dbgfd, "Full dump of threading info...\n");
+		fprintf(dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd");
 
-	for_each_art(i) {
-		fprintf(dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
-			(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
-			(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
-			(arts[i].refptr->child) ? arts[i].refptr->child->article : -2,
-			arts[i].prev, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
+		for_each_art(i) {
+			fprintf(dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
+				(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
+				(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
+				(arts[i].refptr->child) ? arts[i].refptr->child->article : -2,
+				arts[i].prev, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
+		}
+
+		fclose(dbgfd);
 	}
-
-	fclose(dbgfd);
-#endif /* DEBUG_REFS */
+#endif /* DEBUG */
 
 	return;
 }
@@ -914,20 +941,25 @@ build_references(
 	/*
 	 * The articles are currently unsorted, and are as they were put by setup_hard_base()
 	 */
-	if (group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
-		sort_arts(group->attribute->sort_art_type);
+	if (group->attribute->sort_article_type != SORT_ARTICLES_BY_NOTHING)
+		sort_arts(group->attribute->sort_article_type);
 
-	sort_ascend = (group->attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
-	               group->attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND ||
-	               group->attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND ||
-	               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND ||
-	               group->attribute->sort_art_type == SORT_ARTICLES_BY_LINES_ASCEND);
+	sort_ascend = (group->attribute->sort_article_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
+	               group->attribute->sort_article_type == SORT_ARTICLES_BY_FROM_ASCEND ||
+	               group->attribute->sort_article_type == SORT_ARTICLES_BY_DATE_ASCEND ||
+	               group->attribute->sort_article_type == SORT_ARTICLES_BY_SCORE_ASCEND ||
+	               group->attribute->sort_article_type == SORT_ARTICLES_BY_LINES_ASCEND);
 
-#ifdef DEBUG_REFS
-	dbgfd = fopen("Refs.dump", "w");
-	SETVBUF(dbgfd, NULL, _IONBF, 0);
-	fprintf(dbgfd, "MSGID phase\n");
-#endif /* DEBUG_REFS */
+#ifdef DEBUG
+	if (debug & DEBUG_REFS) {
+		char file[PATH_LEN];
+
+		joinpath(file, sizeof(file), TMPDIR, "REFS.dump");
+		dbgfd = fopen(file, "w");
+		SETVBUF(dbgfd, NULL, _IONBF, 0);
+		fprintf(dbgfd, "MSGID phase\n");
+	}
+#endif /* DEBUG */
 
 	/*
 	 * Add the Message-ID headers to the cache, using the last Reference
@@ -951,7 +983,10 @@ build_references(
 					/*
 					 * Remove circular reference to current article
 					 */
-					DEBUG_PRINT((dbgfd, "removing circular reference to: %s\n", s));
+#ifdef DEBUG
+					if (debug & DEBUG_REFS)
+						DEBUG_PRINT((dbgfd, "removing circular reference to: %s\n", s));
+#endif /* DEBUG */
 					*s = '\0';
 				}
 			}
@@ -970,7 +1005,10 @@ build_references(
 		FreeAndNull(art->msgid);	/* Now cached - discard this */
 	}
 
-	DEBUG_PRINT((dbgfd, "REFS phase\n"));
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		DEBUG_PRINT((dbgfd, "REFS phase\n"));
+#endif /* DEBUG */
 	/*
 	 * Add the References data to the cache
 	 */
@@ -993,7 +1031,8 @@ build_references(
 		FreeAndNull(art->refs);
 	}
 
-#ifdef DEBUG_REFS
-	fclose(dbgfd);
-#endif /* DEBUG_REFS */
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		fclose(dbgfd);
+#endif /* DEBUG */
 }
