@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2006-09-02
+ *  Updated   : 2009-01-14
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2009 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,9 +48,10 @@
 #	endif /* !__UUDEVIEW_H__ */
 #endif /* HAVE_UUDEVIEW_H */
 
+#ifndef HAVE_LIBUU
 #undef OFF
-
 enum state { INITIAL, MIDDLE, OFF, END };
+#endif /* !HAVE_LIBUU */
 
 /*
  * Local prototypes
@@ -59,7 +60,7 @@ static FILE *open_save_filename(const char *path, t_bool mbox);
 static int match_content_type(t_part *part, char *type);
 static t_bool check_save_mime_type(t_part *part, const char *mime_types);
 static t_bool decode_save_one(t_part *part, FILE *rawfp, t_bool postproc);
-static t_bool expand_save_filename(char *outpath, const char *path);
+static t_bool expand_save_filename(char *outpath, size_t outpath_len, const char *path);
 static void generate_filename(char *buf, int buflen, const char *suffix);
 static void post_process_uud(void);
 static void post_process_sh(void);
@@ -118,14 +119,14 @@ check_start_save_any_news(
 			break;
 
 		case MAIL_ANY_NEWS:
-			joinpath(savefile, TMPDIR, "tin");
+			joinpath(savefile, sizeof(savefile), TMPDIR, "tin");
 #ifdef APPEND_PID
 			snprintf(savefile + strlen(savefile), sizeof(savefile) - strlen(savefile), ".%d", (int) process_id);
 #endif /* APPEND_PID */
 			/* FALLTHROUGH */
 
 		case SAVE_ANY_NEWS:
-			joinpath(logfile, rcdir, "log");
+			joinpath(logfile, sizeof(logfile), rcdir, "log");
 
 			if (no_write || (fp_log = fopen(logfile, "w")) == NULL) {
 				perror_message(_(txt_cannot_open), logfile);
@@ -160,9 +161,6 @@ check_start_save_any_news(
 		if (group->bogus || !group->subscribed)
 			continue;
 
-		if (!index_group(group))
-			continue;
-
 		if (function == MAIL_ANY_NEWS || function == SAVE_ANY_NEWS) {
 			if (!group->attribute->batch_save)
 				continue;
@@ -176,15 +174,18 @@ check_start_save_any_news(
 			if (function == SAVE_ANY_NEWS) {
 				char tmp[PATH_LEN];
 
-				if (!strfpath(group->attribute->savedir, tmp, sizeof(tmp), group))
-					joinpath(tmp, homedir, DEFAULT_SAVEDIR);
+				if (!strfpath(tinrc.savedir, tmp, sizeof(tmp), group))
+					joinpath(tmp, sizeof(tmp), homedir, DEFAULT_SAVEDIR);
 
 				make_group_path(group->name, group_path);
-				joinpath(path, tmp, group_path);
+				joinpath(path, sizeof(path), tmp, group_path);
 				create_path(path);	/* TODO error handling */
 			}
 		}
 
+		if (!index_group(group))
+			continue;
+
 		/*
 		 * For each article in this group...
 		 */
@@ -199,7 +200,7 @@ check_start_save_any_news(
 						print_first = FALSE;
 					}
 					if (!verbose && !catchup) /* we don't need details */
-						return 2;
+						return NEWS_AVAIL_EXIT;
 					art_count++;
 					if (arts[j].score >= tinrc.score_select)
 						hot_count++;
@@ -218,7 +219,7 @@ check_start_save_any_news(
 
 					if (function == SAVE_ANY_NEWS) {
 						snprintf(buf, sizeof(buf), "%ld", arts[j].artnum);
-						joinpath(savefile, path, buf);
+						joinpath(savefile, sizeof(savefile), path, buf);
 					}
 
 					if ((savefp = fopen(savefile, "w")) == NULL) {
@@ -290,11 +291,11 @@ check_start_save_any_news(
 			 * TODO: shall we return 2 or 0 in the -cZ case?
 			 */
 			if (unread_news && !catchup)
-				return 2;
+				return NEWS_AVAIL_EXIT;
 			else {
 				if (verbose)
 					wait_message(1, _(txt_there_is_no_news));
-				return 0;
+				return EXIT_SUCCESS;
 			}
 			/* NOTREACHED */
 
@@ -465,11 +466,11 @@ save_and_process_art(
 
 		/* Add on the archive name as a directory */
 		/* TODO: maybe a s!/!.! on archive-name would be better */
-		joinpath(archpath, path, artptr->archive->name);
+		joinpath(archpath, sizeof(archpath), path, artptr->archive->name);
 
 		/* Generate the filename part and append it */
 		snprintf(filename, sizeof(filename), "%s%s", partprefix, artptr->archive->partnum);
-		joinpath(path, archpath, filename);
+		joinpath(path, sizeof(path), archpath, filename);
 /*fprintf(stderr, "save_and_process_art archive-name mangled path=(%s)\n", path);*/
 		if (!create_path(path))
 			return FALSE;
@@ -604,6 +605,7 @@ generate_filename(
 static t_bool
 expand_save_filename(
 	char *outpath,
+	size_t outpath_len,
 	const char *path)
 {
 	char base_filename[PATH_LEN];
@@ -615,15 +617,15 @@ expand_save_filename(
 	 * Make sure that externally supplied filename is a filename only and fits
 	 * into buffer
 	 */
-	STRCPY(buf_path, path); base_name(buf_path, base_filename);
+	STRCPY(buf_path, path);
+	base_name(buf_path, base_filename);
 
 	/* Build default path to save to */
 	if (!(ret = strfpath(curr_group->attribute->savedir, buf, sizeof(buf), curr_group)))
-		joinpath(buf, homedir, DEFAULT_SAVEDIR);
+		joinpath(buf, sizeof(buf), homedir, DEFAULT_SAVEDIR);
 
 	/* Join path and filename */
-	/* TODO: make sure full path fits into outpath! */
-	joinpath(outpath, buf, base_filename);
+	joinpath(outpath, outpath_len, buf, base_filename);
 
 	return (ret == 1);	/* should now always evaluate to FALSE */
 }
@@ -752,8 +754,8 @@ post_process_uud(
 			my_printf(cCRLF);
 
 			/* item->mimetype seems not to be available for uudecoded files etc */
-			if (tinrc.post_process_view) {
-				joinpath(path, file_out_dir, item->filename);
+			if (curr_group->attribute->post_process_view) {
+				joinpath(path, sizeof(path), file_out_dir, item->filename);
 				view_file(path, strrchr(path, DIRSEP) + 1);
 			}
 		} else {
@@ -844,7 +846,7 @@ post_process_uud(
 							generate_filename(name, sizeof(name), "uue");
 
 						filename = name;
-						expand_save_filename(path, filename);
+						expand_save_filename(path, sizeof(path), filename);
 						filename = strrchr(path, DIRSEP) + 1;	/* ptr to filename portion */
 						if ((fp_out = fopen(path, "w")) == NULL) {
 							perror_message(_(txt_cannot_open), path);
@@ -905,7 +907,7 @@ post_process_uud(
 				my_printf(_(txt_uu_success), filename);
 				my_printf(cCRLF);
 				sum_file(path, filename);
-				if (tinrc.post_process_view)
+				if (curr_group->attribute->post_process_view)
 					view_file(path, filename);
 				state = INITIAL;
 				continue;
@@ -1112,8 +1114,8 @@ print_art_seperator_line(
 	t_bool is_mailbox)
 {
 #ifdef DEBUG
-	if (debug == 2)
-		error_message("Mailbox=[%d], mailbox_format=[%s]", is_mailbox, txt_mailbox_formats[tinrc.mailbox_format]);
+	if (debug & DEBUG_MISC)
+		error_message(2, "Mailbox=[%d], mailbox_format=[%s]", is_mailbox, txt_mailbox_formats[tinrc.mailbox_format]);
 #endif /* DEBUG */
 
 	fprintf(fp, "%s", (is_mailbox && !strcasecmp(txt_mailbox_formats[tinrc.mailbox_format], "MMDF")) ? MMDFHDRTXT : "\n");
@@ -1132,15 +1134,10 @@ start_viewer(
 	t_mailcap *foo;
 
 	if ((foo = get_mailcap_entry(part, path)) != NULL) {
-		char buff[LEN];
-
 		if (foo->nametemplate)	/* honor nametemplate */
 			rename_file(path, foo->nametemplate);
 
 		wait_message(0, _(txt_starting_command), foo->command);
-
-		/* are the () needed if foo->command holds more than one cmd? */
-		snprintf(buff, sizeof(buff), "(%s)", foo->command);
 		if (foo->needsterminal) {
 			set_xclick_off();
 			EndWin();
@@ -1200,9 +1197,9 @@ decode_save_one(
 
 		lookup_extension(extension, sizeof(extension), content_types[part->type], part->subtype);
 		generate_filename(buf, sizeof(buf), extension);
-		mbox = expand_save_filename(savepath, buf);
+		mbox = expand_save_filename(savepath, sizeof(savepath), buf);
 	} else
-		mbox = expand_save_filename(savepath, name);
+		mbox = expand_save_filename(savepath, sizeof(savepath), name);
 
 	/*
 	 * Not a good idea to dump attachments over a mailbox
@@ -1213,7 +1210,7 @@ decode_save_one(
 	}
 
 	if (!(create_path(savepath))) {
-		error_message(_(txt_cannot_open_for_saving), savepath);
+		error_message(2, _(txt_cannot_open_for_saving), savepath);
 		return FALSE;
 	}
 
@@ -1221,7 +1218,7 @@ decode_save_one(
 	 * Decode/save the attachment
 	 */
 	if ((fp = open_save_filename(savepath, FALSE)) == NULL) {
-		error_message(_(txt_cannot_open_for_saving), savepath);
+		error_message(2, _(txt_cannot_open_for_saving), savepath);
 		return FALSE;
 	}
 
@@ -1266,7 +1263,7 @@ decode_save_one(
 	 * View the attachment
 	 */
 	if (postproc) {
-		if (tinrc.post_process_view) {
+		if (curr_group->attribute->post_process_view) {
 			start_viewer(part, savepath);
 			my_printf(cCRLF);
 		}
@@ -1283,7 +1280,7 @@ decode_save_one(
 	/*
 	 * Save the attachment
 	 */
-	if (postproc && tinrc.post_process_view) {
+	if (postproc && curr_group->attribute->post_process_view) {
 		my_printf(_(txt_uu_success), savepath);
 		my_printf(cCRLF);
 	}
