@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2005-11-04
+ *  Updated   : 2008-03-26
  *  Notes     :
  *
- * Copyright (c) 1997-2007 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2009 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,10 +65,13 @@
 /*
  * local prototypes
  */
-#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	static UChar *char2UChar(const char *str);
-	static char *UChar2char(const UChar *ustr);
-#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && !NO_LOCALE */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	static wchar_t *my_wcsdup(const wchar_t *wstr);
+#	if defined(HAVE_LIBICUUC)
+		static UChar *char2UChar(const char *str);
+		static char *UChar2char(const UChar *ustr);
+#	endif /* HAVE_LIBICUUC */
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 
 /*
@@ -125,6 +128,7 @@ tin_ltoa(
 }
 
 
+#if !defined(USE_DMALLOC) || (defined(USE_DMALLOC) && !defined(HAVE_STRDUP))
 /*
  * Handrolled version of strdup(), presumably to take advantage of
  * the enhanced error detection in my_malloc
@@ -138,15 +142,15 @@ my_strdup(
 	size_t len = strlen(str) + 1;
 	void *ptr = my_malloc(len);
 
-#if 0 /* as my_malloc exits on error, ptr can't be NULL */
+#	if 0 /* as my_malloc exits on error, ptr can't be NULL */
 	if (ptr == NULL)
 		return NULL;
-#endif /* 0 */
+#	endif /* 0 */
 
 	memcpy(ptr, str, len);
 	return (char *) ptr;
 }
-
+#endif /* !USE_DMALLOC || (USE_DMALLOC && !HAVE_STRDUP) */
 
 /*
  * strtok that understands empty tokens
@@ -422,7 +426,7 @@ OUT:
 
 	return (sign * (-val));
 }
-#	undef DIGIT(x)
+#	undef DIGIT
 #	undef MBASE
 #endif /* !HAVE_STRTOL */
 
@@ -890,7 +894,7 @@ wstrunc(
 /*
  * duplicates a wide-char string
  */
-wchar_t *
+static wchar_t *
 my_wcsdup(
 	const wchar_t *wstr)
 {
