@@ -3,10 +3,10 @@
  *  Module    : tags.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1999-12-06
- *  Updated   : 2005-05-10
+ *  Updated   : 2008-11-22
  *  Notes     : Split out from other modules
  *
- * Copyright (c) 1999-2007 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2009 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -253,7 +253,7 @@ line_is_tagged(
 {
 	int code = 0;
 
-	if (curr_group->attribute->thread_arts) {
+	if (curr_group->attribute->thread_articles) {
 		int i;
 		for (i = n; i >= 0; i = arts[i].thread) {
 			if (arts[i].tagged > code)
@@ -374,12 +374,12 @@ set_range(
 			range = tinrc.default_range_thread;
 			break;
 
-		default:
+		default:	/* should no happen */
 			return FALSE;
 	}
 
 #if 0
-	error_message("Min=[%d] Max=[%d] Cur=[%d] DefRng=[%s]", min, max, curr, range);
+	error_message(2, "Min=[%d] Max=[%d] Cur=[%d] DefRng=[%s]", min, max, curr, range);
 #endif /* 0 */
 	prompt = fmt_string(_(txt_enter_range), range);
 
@@ -523,14 +523,15 @@ do_auto_select_arts(
 
 	for_each_art(i) {
 		if (arts[i].status == ART_UNREAD && !arts[i].selected) {
-#	ifdef DEBUG_NEWSRC
-			debug_print_comment("group.c: X command");
-#	endif /* DEBUG_NEWSRC */
+#	ifdef DEBUG
+			if (debug & DEBUG_NEWSRC)
+				debug_print_comment("group.c: X command");
+#	endif /* DEBUG */
 			art_mark(curr_group, &arts[i], ART_READ);
 			arts[i].zombie = TRUE;
 		}
 	}
-	if (curr_group->attribute->show_only_unread)
+	if (curr_group->attribute->show_only_unread_arts)
 		find_base(curr_group);
 
 	grpmenu.curr = 0;
@@ -547,14 +548,15 @@ undo_auto_select_arts(
 
 	for_each_art(i) {
 		if (arts[i].status == ART_READ && arts[i].zombie) {
-#	ifdef DEBUG_NEWSRC
-			debug_print_comment("group.c: + command");
-#	endif /* DEBUG_NEWSRC */
+#	ifdef DEBUG
+			if (debug & DEBUG_NEWSRC)
+				debug_print_comment("group.c: + command");
+#	endif /* DEBUG */
 			art_mark(curr_group, &arts[i], ART_UNREAD);
 			arts[i].zombie = FALSE;
 		}
 	}
-	if (curr_group->attribute->show_only_unread)
+	if (curr_group->attribute->show_only_unread_arts)
 		find_base(curr_group);
 
 	grpmenu.curr = 0;	/* do we want this? */
